YUI.add('contacts-manager', function (Y, NAME) {

"use strict";
var CLICK = 'click',
    Manager;
Y.HistoryHash.hashPrefix = '!',

Manager = Y.Base.create('contactManager', Y.Widget, [], {
    initializer: function() {
        this.set('history', new Y.History());
    },
    bindUI: function() {

        var deleteButton,
            addButton;

        this.get('history').on('change', this._handleHistoryChange, this);
        this.after('selectedCheckboxesChange', function() {
            if (this.get('selectedCheckboxes').length < 1) {
                this._disableActions();
            } else {
                this._enableActions();
            }
        });

        // Select all is only available if there are contacts.
        try {
            this.get('selectAll').on(CLICK, this._selectAll, this);
        } catch(e) {}
        // Feature still hidden on everything but dev and staging.

        deleteButton = this.get('deleteButton');
        if (deleteButton){
            deleteButton.on(CLICK, this._deleteContacts, this);
        }

        addButton = this.get('addButton');
        if (addButton) {
            addButton.on(CLICK, this._addContact, this);
        }


        this.get('list').delegate(
            CLICK, this._handleCheckboxClick, 'input', this);
            this.get('list').delegate(
                CLICK, this._viewContact, '.contact-record', this);

                Y.Global.on('one:stale', this._updateContent, this);
    },
    renderUI: function() {
        this._setUpNavigation();
    },
    syncUI: function() {},
    _handleHistoryChange: function(e) {

        var action = null,
        contact = null;
        if (e.changed.action) {
            action = e.changed.action.newVal;
        } else {
            action = this.get('history').get('action');
        }
        if (e.changed.contact) {
            contact = e.changed.contact.newVal;
        } else {
            contact = this.get('history').get('contact');
        }
        switch (action) {
            case 'view':
                this._loadViewContact(contact);
            break;
            case 'add':
                this._loadAddContactForm();
            break;
            case 'edit':
                this._loadEditContact(contact);
            break;
        }
    },
    _updateContent: function() {
        var list = this.get('list');
        Y.io('/contacts/_/contact_list', {
            on: {
                success: function(id, response) {
                    list.setContent(response.responseText);
                },
                failure: Y.bind(this._handleFailure, this)
            }
        });
    },
    _handleFailure: function() {
        var error = this.get('strings').error_msg;

        try {
            Y.one('#modal-contacts').remove();
        } catch (e) {}
        Y.Global.fire('one:idle');
        Y.Global.fire('one:error', { message: error });
    },
    _handleCheckboxClick: function(e) {
        var selectedCheckboxes = this.get('selectedCheckboxes'),
            checkbox = e.target,
            where_in_array;

        if (checkbox.get('checked')) {
            selectedCheckboxes.push(checkbox.get('value'));
            this.set('selectedCheckboxes', selectedCheckboxes);
        } else {
            where_in_array = selectedCheckboxes.indexOf(checkbox.get('value'));
            if(where_in_array !== -1){
                selectedCheckboxes.splice(where_in_array, 1);
                this.set('selectedCheckboxes', selectedCheckboxes);
            }
        }
    },
    _enableActions: function() {
        var del = this.get('deleteButton');

        if (del) {
            del.removeClass('delete-action-disabled');
        }
    },
    _disableActions: function() {
        var del = this.get('deleteButton');

        if (del) {
            del.addClass('delete-action-disabled');
        }
    },
    _selectAll: function(e) {
        e.preventDefault();
        var selectAll = e.target,
        selectedCheckboxes = [];
        this.get('list').all('input').each(function(element) {
            selectedCheckboxes.push(element.get('value'));
            element.set('checked', true);
        });
        this.set('selectedCheckboxes', selectedCheckboxes);

        selectAll.detach();
        selectAll.on(CLICK, this._deselectAll, this);
        selectAll.set('text', 'Deselect all');
    },
    _deselectAll: function(e) {
        e.preventDefault();
        this.get('list').all('input').each(function(element) {
            element.set('checked', false);
        });
        this.reset('selectedCheckboxes');

        var selectAll = e.target;
        selectAll.detach();
        selectAll.on(CLICK, this._selectAll, this);
        selectAll.set('text', 'Select all');
    },
    _viewContact: function(e) {
        var target_class = e.target.get('className'),
            current_row,
            contact_id;

        // Delete checkbox should still be clickable
        if(target_class !== 'contact-delete-checkbox') {
            e.halt();
            current_row = e.currentTarget;
            contact_id = current_row.getAttribute('id').replace(/^contact-(.*)/, '$1');
            this._loadContact(contact_id);
        }
    },
    _loadViewContact: function(contact_id) {
        var uri = '/contacts/api/snippet/' + contact_id,
            self = this;
        this._refreshContactView(uri, function(){
            Y.one('.edit-button').on(CLICK, self._editContact, self);
        });
    },
    _loadContact: function(contact_id) {
        if(contact_id !== undefined){
            this.get('history').add({
                action: 'view', contact: contact_id});
        } else {
            this.get('history').add({action: null, contact: null});
        }
    },
    _loadAddContactForm: function() {
        var uri = '/contacts/api/snippet/add/';
        this._refreshContactView(uri, Y.bind(this._prepareForm, this));
    },
    _addContact: function (e) {
        e.preventDefault();
        this.get('history').add({action: 'add', contact: null});
    },
    _loadEditContact: function(contact_id) {
        var uri = '/contacts/api/snippet/' + contact_id + '/edit/';
        this._refreshContactView(uri,
                                 Y.bind(this._prepareForm, this, contact_id));
    },
    _editContact: function(e) {
        e.preventDefault();
        var current_row = e.currentTarget,
            contact_id = current_row.getAttribute('id').replace(/^contact-(.*)/, '$1');
        this.get('history').add({action: 'edit', contact: contact_id});
    },
    _prepareForm: function(contact_id) {
        Y.all('.add-edit-controls').setStyle('display', 'block');
        Y.all('.save-button').on(CLICK, this._submitForm, this);
        Y.all('.cancel-button').on(CLICK, Y.bind(this._loadContact, this, contact_id));


        var addField = function(fieldName) {
            var section = Y.one('#dynamic-' + fieldName),
            count = section.all('ul').size(),
            templateField = Y.one('#' + fieldName + '-template'),
            newField = templateField.cloneNode(true),
            newSelectName,
            selectNodes,
            isFirstInput = true,
            inputNodes,
            inputToFocus,
            newInputName;

            selectNodes = newField.all('select').each(function(newSelect) {
                newSelectName = count + '-' + newSelect.get('name');
                newSelect.set('name', newSelectName);
                newSelect.set('id', newSelectName);
            });

            inputNodes = newField.all('input').each(function(newInput) {
                if (isFirstInput === true) {
                    inputToFocus = newInput;
                    isFirstInput = false;
                }
                newInputName = count + '-' + newInput.get('name');
                newInput.set('name', newInputName);
                newInput.set('id', newInputName);
            });

            newField.setStyle('display', 'block');
            newField.set('id', count);
            section.appendChild(newField);
            inputToFocus.focus();
        };

        Y.one('#add-phone-number').on('click', function() { addField('phone-numbers'); });
        Y.one('#add-email-address').on('click', function() { addField('email-addresses'); });
        Y.one('#add-im-address').on('click', function() { addField('im-addresses'); });
        Y.one('#add-url').on('click', function() { addField('urls'); });
        Y.one('#add-postal-address').on('click', function() { addField('postal-addresses'); });
    },
    _submitForm: function(e) {
        e.preventDefault();
        var uri = '/contacts/api/create_or_edit',
            cfg,
            request,
            onIOSuccess;

        onIOSuccess = function(request_id, o){
            var response = Y.JSON.parse(o.responseText);
            this._loadContact(response.contact_id);
            Y.Global.fire('one:stale');
        };

        cfg = {
            method: 'POST',
            data: {'csrfmiddlewaretoken': YUI.Env.CSRF_TOKEN},
            form: {id: 'contact-edit-form'},
            on: {
                success: Y.bind(onIOSuccess, this),
                failure: Y.bind(this._handleFailure, this)
            }
        };
        request = Y.io(uri, cfg);
    },
    _getCandidates: function() {
        var candidates = [];
        this.get('list').all('input').each(function(item) {
            if (item.get('checked')) {
                candidates.push(item.get('value'));
            }
        });
        return candidates;
    },
    _deleteContacts: function(e) {
        e.preventDefault();
        var candidates = this._getCandidates();
        if (!candidates.length) {
            return;
        }

        if (!window.confirm('Delete selected contacts?', 'Delete contacts')) {
            return;
        }

        Y.io('/contacts/api/bulk/delete', {
            method: 'POST',
            data: {
                'contacts': Y.JSON.stringify(candidates),
                'csrfmiddlewaretoken': YUI.Env.CSRF_TOKEN
            },
            on: {
                success: function() {
                    for (var i=0; i < candidates.length; i++) {
                        Y.one('#contact-' + candidates[i]).remove();
                    }
                    Y.one('#delete-selected').addClass('delete-action-disabled');
                    Y.Global.fire('one:stale');
                },
                error: function() {
                    Y.Global.fire('one:error');
                }
            }
        });
    },
    _refreshContactView: function(uri, onSuccess) {
        var contact = Y.one('#contact-details'),
            request;

        function onIOSuccess(request_id, o) {
            contact.setContent(o.responseText);
            if(onSuccess){
                onSuccess();
            }
            Y.Global.fire('one:idle');
            Y.Global.fire("contacts:contentUpdate");
        }
        // Remove in case there was an error before
        Y.Global.fire('one:busy');
        request = Y.io(uri, {
            on: {
                success: onIOSuccess,
                failure: Y.bind(this._handleFailure, this)
            }
        });
    },
    _showMergeOverlay: function(e) {
        e.preventDefault();
        Y.Global.fire('one:busy');
    },
    _setUpNavigation: function() {
        var currentClass = "current",
        listNode = Y.one("#list"),
        listNodeTop = listNode.getY(),
        lastPeopleUl = Y.one("#last-group"),
        posList = [],
        extraMargin,
        anim,
        animating,
        onEnd,
        onStart,
        timer;

        function extractClass(className, startMatch) {
            // Get the first class that matches.
            // Prevents the addition of a class breaking the nav.
            var aClasses = className.split(" "),
            i, j;

            for (i=0, j=aClasses.length; i<j; i++){
                if (aClasses[i].indexOf(startMatch) === 0) {
                    return aClasses[i];
                }
            }
            return false;
        }

        function calcListPositions() {
            // Pre-Calculate the positions of the headings.
            var groupNodes = Y.all("#list li.letter-group");
            groupNodes.each( function(n){
                var elTop =  parseInt((n.getY() - listNodeTop), 10) - 10 + listNode.get("scrollTop"),
                    elBottom = elTop + n.get("offsetHeight"),
                    h2 = n.one("h2"),
                    letter = h2.get("text"),
                    ident = h2.get("id");

                posList.push( {"top": elTop, "bottom": elBottom, "node": n, "letter": letter, "id": ident });
            });
        }

        function setUpContactsList() {
            // Contacts list gets a height based on the height of the main content div
            // overflow auto gives us the scrollbars.
            var mainNode = Y.one("#main"),
                mainHeight = mainNode.get("offsetHeight");

            listNode.setStyle("height", mainHeight+"px");
            listNode.setStyle("overflow", "auto");
            // boost how much space the last ul take up by adding margin.
            if(lastPeopleUl){
                extraMargin = parseInt(mainHeight - lastPeopleUl.get("offsetHeight"), 10);
                lastPeopleUl.setStyle("marginBottom", extraMargin+"px");
            }
        }

        function selectNavLetter(){
            var listNode = Y.one("#list"),
                scrollTop = listNode.get("scrollTop"),
                i, j, o;

            function getParentAndAddClass(node) {
                if (node) {
                    node.get("parentNode").addClass(currentClass);
                }
            }

            for (i=0, j=posList.length; i<j; i++) {
                o = posList[i];
                if (scrollTop >= o.top && scrollTop <= o.bottom){
                    Y.all("div.letters li." + currentClass).removeClass(currentClass);
                    Y.all("div.letters a."+ o.id).each(getParentAndAddClass);
                    break;
                }
            }

        }

        //Subscribe to a global custom event.
        Y.Global.on("contacts:contentUpdate", function() {
            setUpContactsList();
            calcListPositions();
        });

        // Use animations for scrolling to a
        // contact letter group.
        anim = new Y.Anim({
            node: listNode,
            duration: 1,
            easing: Y.Easing.easeOut
        });
        animating = false;
        onEnd = function() {
            animating = false;
            selectNavLetter();
        };
        onStart = function() {
            animating = true;
        };
        anim.on("start", onStart);
        anim.on("end", onEnd);

        function onClickNav(e) {
            var headingNode,
                scrollTop,
                letterClass;

            e.preventDefault();
            if (animating === false){
                letterClass = extractClass(e.currentTarget.get("className"), "cl_");
                if (letterClass) {
                    headingNode = Y.one("#" + letterClass);
                    scrollTop = listNode.get("scrollTop");
                    anim.set('to', { scroll: [scrollTop, scrollTop + (headingNode.getY() - listNodeTop)] });
                    anim.run();
                }
            }
        }
        // Set-up event delegation for watch for clicks on a elements
        Y.delegate("click", onClickNav, "#top-contact-nav", "a");
        Y.delegate("click", onClickNav, "#bottom-contact-nav", "a");

        // Throttled scroll-handling
        // This only works outside of the animation
        timer = null;

        listNode.on("scroll", function(){
            if (animating === false) {
                if (timer) {
                    clearTimeout(timer);
                }
                timer = setTimeout(selectNavLetter, 100);
            }
        });

        calcListPositions();
        setUpContactsList();
        selectNavLetter();

    }
}, {
    ATTRS: {
        addButton: {},
        contactDetails: {
            value: null
        },
        deleteButton: {},
        history: {
            value: null
        },
        list: {},
        main: {},
        selectAll: {},
        selectedCheckboxes: {
            value: []
        },
        strings: {
            value: {
                error_msg: "We're sorry. Something has gone wrong. Our engineers have been notified.",
                loading_msg: '<span class="loading-contact">Loading...</span>'
            }
        }
    },
    HTML_PARSER: {
        addButton: '.add-button',
        contactDetails: '#contact-details',
        deleteButton: '.delete-action',
        list: '#list',
        main: '#main',
        selectAll: '#select-all'
    }
});

Y.Manager = Manager;


}, '@VERSION@', {
    "requires": [
        "anim-base",
        "anim-color",
        "anim-curve",
        "anim-easing",
        "anim-node-plugin",
        "anim-scroll",
        "anim-xy",
        "base-base",
        "base-pluginhost",
        "base-build",
        "event-base",
        "event-delegate",
        "event-synthetic",
        "event-mousewheel",
        "event-mouseenter",
        "event-key",
        "event-focus",
        "event-resize",
        "event-hover",
        "event-outside",
        "history-base",
        "history-hash",
        "history-hash-ie",
        "history-html5",
        "io-base",
        "io-xdr",
        "io-form",
        "io-upload-iframe",
        "io-queue",
        "json-parse",
        "json-stringify",
        "node-base",
        "node-event-delegate",
        "node-pluginhost",
        "node-screen",
        "node-style",
        "contacts-merge",
        "overlay",
        "transition",
        "widget-base",
        "widget-htmlparser",
        "widget-uievents",
        "widget-skin"
    ]
});
