YUI.add('contacts-merge', function (Y, NAME) {

"use strict";
var CLICK = 'click',
    SmartMergeDialog;

SmartMergeDialog = Y.Base.create('smartMergeDialog', Y.Overlay, [], {
    initializer: function() {
        this._getMergeCandidates();
    },
    bindUI: function() {
        /* Listen for escape event. */
        Y.on('keydown', function(e) {
            if (e.keyCode === 27) {
                this.destroy();
                Y.Global.fire('one:idle');
            }
        }, window, this);
    },
    _getMergeCandidates: function() {
        Y.io('/contacts/_/smart_merge_contact_list', {
            timeout: 2*60*1000, // 2 minutes
            on: {
                success: Y.bind(this._populateContent, this),
                failure: Y.bind(this._handleFailure, this)
            }
        });
    },
    _handleCancel: function(e) {
        e.preventDefault();
        this.destroy();
    },
    _handleFailure: function() {
        Y.Global.fire('one:idle');
        Y.Global.fire('one:error');
        this.destroy();
    },
    _handleSave: function(e) {
        e.preventDefault();
        Y.Global.fire('one:busy');
        Y.io('/contacts/merge', {
            method: 'POST',
            data: {'csrfmiddlewaretoken': YUI.Env.CSRF_TOKEN},
            timeout: 2*60*1000, // 2 minutes
            form: {
                id: Y.one('#smart-merge-form')
            },
            on: {
                success: Y.bind(this._saveSuccess, this),
                failure: function() {
                }
            }
        });
    },
    _populateContent: function(id, response) {
        if (!this.get('cancelled')) {
            var content = Y.Node.create('<div class="contact-box">' +
                                        response.responseText + '</div>'),
            wrapperDiv,
            footer = Y.Node.create('<div class="control-footer"></div>'),
            submitButton = content.one('input[type="submit"]'),
            cancel,
            save,
            description,
            close;

            if (submitButton) {
                submitButton.remove();
                wrapperDiv = Y.Node.create(
                    '<div></div>'),
                    description = Y.Node.create(
                        '<p>' + this.get('strings').description + '</p>'),
                        save = Y.Node.create('<a class="clickable save-button">Merge</a>'),
                        cancel = Y.Node.create('<a class="clickable cancel-button">Cancel</a>');

                        wrapperDiv.appendChild(description);
                        wrapperDiv.appendChild(content);
                        this.set('bodyContent', wrapperDiv);

                        save.on(CLICK, this._handleSave, this);
                        cancel.on(CLICK, this._handleCancel, this);

                        footer.appendChild(save);
                        footer.appendChild(cancel);
                        this.set('footerContent', footer);

            } else {
                close = Y.Node.create(
                    '<a class="clickable save-button">OK</a>');
                    close.on(CLICK, this._handleCancel, this);

                    footer.appendChild(close);
                    this.set('bodyContent', content);
                    this.set('footerContent', footer);
            }
            Y.Global.fire('one:idle');
            this.render();
        }
    },
    _saveFailure: function() {
        Y.Global.fire('one:idle');
        this.destroy();
        Y.Global.fire(
            'one:error',
            this.get('strings').error_msg);
    },
    _saveSuccess: function() {
        Y.Global.fire('one:stale');
        Y.Global.fire('one:idle');
        this.destroy();
    }
}, {
    ATTRS: {
        headerContent: { value: '<h2>Merge contacts</h2>' },
        visible: { value: true },
        centered: { value: true },
        width: { value: '400px' },
        zIndex: { value: 1000 },

        saveButton: {},
        cancelButton: {},
        cancelled: {
            value: false
        },
        strings: {
            value: {
                description: "We found some contacts that look pretty similar. We can merge them together if you'd like.",
                error_msg: "We're sorry. Something has gone wrong. Our engineers have been notified."
            }
        }
    }
});
Y.SmartMergeDialog = SmartMergeDialog;



}, '@VERSION@', {
    "requires": [
        "base-base",
        "base-pluginhost",
        "base-build",
        "io-base",
        "io-xdr",
        "io-form",
        "io-upload-iframe",
        "io-queue",
        "overlay"
    ]
});
