YUI.add('feedback', function (Y, NAME) {

    "use strict";
    var BusyWidget,
        BrowserMessage,
        InfoWidget,
        ErrorWidget,
        BOUNDINGBOX = 'boundingBox',
        LoadingWidget = Y.Base.create('loading', Y.Widget, [
        Y.WidgetStdMod, Y.WidgetPosition, Y.WidgetStack,
        Y.WidgetPositionAlign, Y.WidgetPositionConstrain
    ], {
        bindUI: function() {
            Y.Global.before('io:start', this.beforeLoading, this);
            Y.Global.after('io:complete', this.afterComplete, this);
            this.before('visibleChange', this.beforeVisibleChange, this);
        },
        beforeLoading: function() {
            this.show();
        },
        afterComplete: function() {
            this.hide();
        },
        beforeVisibleChange: function(e) {
            if (e.newVal === true) {
                this.set('align', {
                    points: [
                        Y.WidgetPositionAlign.TL,
                        Y.WidgetPositionAlign.TL
                    ]
                });
            }
        }
    }, {
        ATTRS: {
            bodyContent: { value: 'Loading...' },
            height: { value: '20px' },
            visible: { value: false },
            width: { value: '70px' },
            zIndex: { value: 5 }
        }
    });

    BusyWidget = Y.Base.create('busy', Y.Overlay, [], {
        initializer: function() {
            var className = this.getClassName() + '-shim',
                shim = Y.Node.create('<div class="' + className + '"></div>');
            shim.setStyles({
                height: Y.config.doc.height,
                width: Y.config.doc.width
            });
            this.set('shim', shim);
        },
        bindUI: function() {
            Y.Global.on('one:busy', this.showBusy, this);
            Y.Global.on('one:idle', this.hideBusy, this);
        },
        align: function() {
            this.set('centered', true);
        },
        hideBusy: function() {
            var events = this.get('events');
            while (events.length > 0) {
                events.pop(0).detach();
            }
            this.hide();
            this.get('shim').remove();
            this.set('footerContent', '');
        },
        showBusy: function(e) {
            Y.one('body').prepend(this.get('shim'));
            var message = this.get('strings').defaultMessage,
                escapeHTML = true;

            if (e && e.message) {
                message = e.message;
            }
            if (e && e.escapeHTML === false){
                escapeHTML = false;
            }
            if (escapeHTML === true){
                message = Y.Escape.html(message);
            }

            this.set('footerContent', message);
            this.show();
            this.align();
            this.get('events').push(
                Y.on('scroll', Y.bind(this.align, this)));
        }
    }, {
        ATTRS: {
            bodyContent: { valueFn: function() {
                return '<div class="' + this.getClassName() + '-waiting"></div>';
            }},
            events: { value: [] },
            shim: { value: null },
            strings: { value: {
                defaultMessage: 'Loading. Please wait...'
            }},
            visible: { value: false },
            width: { value: '220px' },
            zIndex: { value: 1000 }
        }
    });

    BrowserMessage = Y.Base.create('message', Y.Widget, [Y.WidgetStdMod, Y.WidgetStack], {
        bindUI: function() {
            Y.Global.on('one:' + this.get('eventName'), this.showMessage, this);
        },
        align: function() {
            var boundingBox = this.get(BOUNDINGBOX),
                offset = parseInt(window.pageYOffset, 10);
            boundingBox.setStyle("top", offset);
        },
        showMessage: function(e) {
            var boundingBox = this.get(BOUNDINGBOX),
                message = (e && e.message) ? e.message : this.get('defaultMessage'),
                escapeHTML = true,
                ev;

            if (e && e.escapeHTML === false){
                escapeHTML = false;
            }
            if (escapeHTML === true){
                message = Y.Escape.html(message);
            }

            this.set('bodyContent', message);
            this.show();
            // We use postion:fixed for better perf elsewhere.
            // This is for mobile UAs which don't have solid position:fixed
            // implementations for various reasons.
            // See http://www.quirksmode.org/mobile/tableViewport.html#positionfixed
            if (Y.UA.mobile) {
                boundingBox.setStyle("position", "absolute");
                this.align();
                ev = Y.on('scroll', Y.bind(this.align, this));
                this.get('events').push(ev);
            }
            boundingBox.transition({
                duration: 1,
                opacity: 1
            }, Y.bind(this.timeout, this));
        },
        timeout: function() {
            setTimeout(Y.bind(this.hideError, this), 7000);
        },
        hideError: function() {
            if (Y.UA.mobile) {
                var events = this.get('events');
                while (events.length > 0) {
                    events.pop(0).detach();
                }
            }
            this.get(BOUNDINGBOX).transition({
                duration: 1,
                opacity: 0
            }, Y.bind(this.hide, this));
        }
    }, {
        ATTRS: {
            defaultMessage: { value: '' },
            eventName: { value: '' },
            visible: { value: false },
            zIndex: { value: 10 },
            events: { value: [] }
        }
    });

    InfoWidget = Y.Base.create('info', BrowserMessage, [], {
        bindUI: function() {
            this.after('hide', this.afterHide, this);
            Y.Global.on('one:' + this.get('eventName'), this.showMessage, this);
            Y.Global.on('one:error', this.hideError, this);
        }
    }, {
        ATTRS: {
            eventName: { value: 'info' }
        }
    });

    ErrorWidget = Y.Base.create('error', BrowserMessage, [], {}, {
        ATTRS: {
            defaultMessage: { value: 'An error has occurred. Our engineers have been notified and we will work to fix this.' },
            eventName: { value: 'error' }
        }
    });

    Y.initFeedback = function() {
        new LoadingWidget().render();
        new BusyWidget().render();
        new InfoWidget().render();
        new ErrorWidget().render();

        Y.one(Y.config.win).on('error', function() {
            Y.Global.fire('one:idle');
        });
    };



}, '@VERSION@', {
    "requires": [
        "base-base",
        "base-pluginhost",
        "base-build",
        "escape",
        "event-base",
        "event-delegate",
        "event-synthetic",
        "event-mousewheel",
        "event-mouseenter",
        "event-key",
        "event-focus",
        "event-resize",
        "event-hover",
        "event-outside",
        "overlay",
        "transition"
    ]
});
