YUI.add('global-navigation', function (Y, NAME) {

"use strict";
/**
 * GlobalNav
 * @constructor
 * @param {Array} ai list of url objects
 * @param {Number} depth how far into the ai array we go before wrapping
 * everything else in a sub menu, the zero based number of displayed items.
 **/
function GlobalNav(ai, depth) {
    ai = ai || [{
        url: 'www.ubuntu.com',
        title: 'Ubuntu'
    }, {
        url: "community.ubuntu.com",
        title: "Community"
    }, {
        url: "askubuntu.com",
        title: "Ask!"
    }, {
        url: "developer.ubuntu.com",
        title: "Developer"
    }, {
        url: "design.ubuntu.com",
        title: "Design"
    }, {
        url: "discourse.ubuntu.com",
        title: "Discourse"
    }, {
        url: "www.ubuntu.com/certification",
        title: "Hardware"
    }, {
        url: "shop.ubuntu.com",
        title: "Shop"
    }, {
        url: "apps.ubuntu.com",
        title: "Apps"
    }, {
        url: "help.ubuntu.com",
        title: "Help"
    }, {
        url: "ubuntuforums.org",
        title: "Forum"
    }, {
        url: "one.ubuntu.com",
        title: "Ubuntu One"
    }, {
        url: "www.launchpad.net",
        title: "Launchpad"
    }, {
        url: "juju.ubuntu.com",
        title: "Juju"
    }, {
        url: "maas.ubuntu.com",
        title: "MAAS"
    }, {
        url: "www.canonical.com",
        title: "Canonical"
    }];

    depth = depth || 7;

    this.location = document.URL.replace(/https?:\/\//i, "");

    var activeIndex,
    template,
    moreEl;


    activeIndex = this.getActiveUrl(ai);

    if (activeIndex) {
        // set active prop on link object which matches current URL
        ai[this.getActiveUrl(ai)].active = true;
    }


    template = Y.Template.Micro.revive(Y.U1.Templates['global-nav']);

    Y.one('body').prepend(template({
        length: depth,
        links: ai
    }));

    moreEl = Y.one('.u-nav-global-more');

    // if the user clicks outside the menu, make sure the more menu is closed
    Y.one(document).on('click', function () {
        moreEl.removeClass('u-nav-global-show');
    });

    // open the more menu
    moreEl.one('a').on('click', function (e) {
        e.preventDefault();
        e.stopPropagation();
        moreEl.toggleClass('u-nav-global-show');
    });

}

GlobalNav.prototype = {
    /**
     * getActiveUrl
     * Takes a list of urls and returns the index of the urk which best matches document.URL
     * @param {Array} links a list of link objects { url: foo, title: bar }
     * @return {Number}
     */
    getActiveUrl: function(links) {
        var re,
        urlList,
        activeUrl,
        index;

        re = new RegExp('^' + this.location, 'i');

        urlList = this._filterUrls(links, this.location);

        activeUrl = (urlList && urlList.length) ? this._reduceUrlList(urlList) : urlList[0];

        index = (activeUrl) ? activeUrl.index : false;

        return index;
    },

    /**
     * _filterUrls
     * filter a list of urls, returning only those containing the string
     * @param {Array} links an Array of url objects
     * @param {String} location
     * @return {Array}
     */

    _filterUrls: function(links, location) {
        return Y.Array.filter(links, function (item, index) {
            if (location.toLowerCase().indexOf(item.url) >= 0) {
                item.index = index;
                return true;
            }
        }, this);
    },

    /**
     * _reduceUrlList
     * reduce a list of urls down to a single object containing the longest url
     * @param {Array} urlList an Array of url objects
     * @return {Object}
     */

    _reduceUrlList: function(urlList) {
        return Y.Array.reduce(urlList, urlList[0], function (a, b) {
            return a.url.length > b.url.length ? a : b;
        }, this);
    }

};

Y.namespace("U1").GlobalNav = GlobalNav;


}, '0.1.0', {"requires": ["node", "template-micro", "array-extras", "global-navigation-templates"]});
