YUI.add('music-io', function (Y, NAME) {

"use strict";
/*global STREAMING_URL, STREAMING_PASS, STREAMING_USER */
var ERRATA = 'c=web&v=1.2&f=json',
    ONEERROR = 'one:error',
    EMPTY = '',
    AMPERSAND = '&',
    AMPERSANDSONGEQUALS = '&songId=',
    CREATEPLAYLISTVIEW = 'createPlaylist.view?',
    GETMUSICDIRECTORYVIEW = 'getMusicDirectory.view?id=',
    GETRANDOMSONGSVIEW = 'getRandomSongs.view?',
    SIZE='size',
    SRC = 'src',
    namespace = Y.namespace('Music');

namespace.DataSource = Y.Base.create('dataSource', Y.DataSource.IO, [], {
    /* This is perverse, I know, but it prevents DRY, so hooray. */
    _defDataFn: function(e) {
        var data = e.data,
        meta = e.meta,
        response = {
            results: (Y.Lang.isArray(data)) ? data: [data],
            meta: (meta) ? meta : {}
        };
        try {
            response.json = Y.JSON.parse(e.data.responseText)['subsonic-response'];
            if (response.json.error) {
                Y.Global.fire(ONEERROR, {
                    message: response.json.error.message
                });
                Y.Do.Halt('An error in the API caused a halt');
                return;
            }
        } catch(ex) {}
        this.fire('response', Y.mix({
            'response': response}, e));
    },
    getArtists: function(callback) {
        this.sendRequest({
            request: 'getIndexes.view?' + ERRATA,
            callback: {
                failure: function() { Y.Global.fire(ONEERROR); },
                success: callback
            }
        });
    },
    createPlaylist: function(name, songs, callback) {
        var songParams = EMPTY,
        songsLength = songs.length,
        i,
        params;

        for (i = 0; i < songsLength; i++) {
            songParams = songParams + AMPERSANDSONGEQUALS + songs[i];
        }
        params = 'name=' + name + songParams;
        this.sendRequest({
            request: CREATEPLAYLISTVIEW + params + AMPERSAND + ERRATA,
            callback: {
                success: callback,
                failure: function() { Y.Global.fire(ONEERROR); }
            }
        });
    },
    updatePlaylist: function(playlistId, songs, callback) {
        var songParams = EMPTY,
        songsLength = songs.length,
        i,
        params;

        for (i = 0; i < songsLength; i++) {
            songParams = songParams + AMPERSANDSONGEQUALS + songs[i];
        }
        params = 'playlistId=' + playlistId + songParams;
        this.sendRequest({
            request: CREATEPLAYLISTVIEW + params + AMPERSAND + ERRATA,
            callback: {
                success: callback,
                failure: function() { Y.Global.fire(ONEERROR); }
            }
        });
    },
    deletePlaylist: function(playlistId, callback) {
        this.sendRequest({
            request: 'deletePlaylist.view?id=' + playlistId + AMPERSAND + ERRATA,
            callback: {
                success: callback
            }
        });
    },
    getPlaylist: function(playlistId, callback) {
        this.sendRequest({
            request: 'getPlaylist.view?id=' + playlistId + AMPERSAND + ERRATA,
            callback: {
                success: callback
            }
        });
    },
    getSongs: function(callback) {
        this.sendRequest({
            request: 'getSongs.view?' + ERRATA,
            callback: {
                failure: function() { Y.Global.fire(ONEERROR); },
                success: callback
            }
        });
    },
    getRandomSongs: function(nResults, callback) {
        var url = null;
        if (!nResults) {
            url = GETRANDOMSONGSVIEW + ERRATA;
        } else {
            url = GETRANDOMSONGSVIEW + SIZE + "=" + nResults + AMPERSAND + ERRATA;
        }
        this.sendRequest({
            request: url,
            callback: {
                failure: function() { Y.Global.fire(ONEERROR); },
                success: callback
            }
        });
    },
    getAlbums: function(artistId, callback) {
        var url = null;
        if (!artistId) {
            url = 'getAlbums.view?' + ERRATA;
        } else {
            url = GETMUSICDIRECTORYVIEW + artistId + AMPERSAND + ERRATA;
        }
        this.sendRequest({
            request: url,
            callback: {
                failure: function() { Y.Global.fire(ONEERROR); },
                success: callback
            }
        });
    },
    getAlbum: function(albumId, callback) {
        this.sendRequest({
            request: GETMUSICDIRECTORYVIEW + albumId + AMPERSAND + ERRATA,
            callback: {
                failure: function() { Y.Global.fire(ONEERROR); },
                success: callback
            }
        });
    },
    getStreamingURI: function(song) {
        if (song === undefined) {
            return null;
        } else {
            return STREAMING_URL + '/rest/stream.view?id=' + song.id + '&u=' + STREAMING_USER + '&p=' + STREAMING_PASS + AMPERSAND + ERRATA;
        }
    },
    getAlbumArt: function(song, image) {
        var defaultURL = MEDIA_URL + 'img/default-album-art.png',
        url;

        image.set("alt", EMPTY);
        if (song.coverArt === undefined) {
            image.set(SRC, defaultURL);
            return;
        }
        url = 'getCoverArt.view?id=' + song.coverArt + AMPERSAND + ERRATA;
        image.set(SRC, this.get('source') + url);
    },
    search: function(term, callback) {
        this.sendRequest({
            request: 'search.view?any=' + term + AMPERSAND + ERRATA,
            callback: {
                success: callback
            }
        });
    }
}, {
    ATTRS: {
        plugins: [
            {fn: Y.Plugin.DataSourceCache, cfg: { max: 3 }}
        ],
        source: { value: '/api/music/1.0/' }
    }
});

namespace.PlaylistsDS = Y.Base.create('playlistsds', Y.DataSource.IO, [], {
    /* This is perverse, I know, but it prevents DRY, so hooray. */
    _defDataFn: function(e) {
        var data = e.data,
        meta = e.meta,
        response = {
            results: (Y.Lang.isArray(data)) ? data: [data],
            meta: (meta) ? meta : {}
        };
        try {
            response.json = Y.JSON.parse(e.data.responseText)['subsonic-response'];
            if (response.json.error) {
                Y.Global.fire(ONEERROR, {
                    message: response.json.error.message
                });
                Y.Do.Halt('An error in the API caused a halt');
                return;
            }
        } catch(ex) {}
        this.fire('response', Y.mix({
            'response': response}, e));
    },
    getPlaylists: function(callback) {
        this.sendRequest({
            request: 'getPlaylists.view?' + ERRATA,
            callback: {
                failure: function() { Y.Global.fire(ONEERROR); },
                success: callback
            }
        });
    }
}, {
    ATTRS: {
        plugins: [
            {fn: Y.Plugin.DataSourceCache, cfg: { max: 3 }}
        ],
        source: { value: '/api/music/1.0/' }
    }
});


}, '@VERSION@', {
    "requires": [
        "datasource-local",
        "datasource-io",
        "datasource-get",
        "datasource-function",
        "datasource-cache",
        "datasource-jsonschema",
        "datasource-xmlschema",
        "datasource-arrayschema",
        "datasource-textschema",
        "datasource-polling"
    ]
});
