YUI.add('music-views', function (Y, NAME) {

"use strict";
/*global GLOBAL_PLAYLIST_LIST:true, soundManager */
var BOUNDINGBOX = 'boundingBox',
    PARENTNODE = 'parentNode',
    CLICK = 'click',
    EVENTNEXT = 'music:next',
    EVENTPAUSE = 'music:pause',
    EVENTPLAY = 'music:play',
    EVENTPREVIOUS = 'music:previous',
    EVENTSTARTED = 'music:started',
    EVENTSTOP = 'music:stop',
    // Sound object events.
    EVSOUNDLOADING = 'music:loading',
    EVSOUNDPLAYING = 'music:playing',
    EVSOUNDPLAY = 'music:soundplay',
    EVSOUNDFINISH = 'music:soundfinish',
    EVSOUNDSTOP = 'music:soundstop',
    EVSOUNDPAUSE = 'music:soundpause',
    HISTORYRESET = {
        album: null,
        artist: null,
        playlist: null,
        search: null,
        songs: null,
        view: null
    },
    DEFAULT_ALBUM_ART = MEDIA_URL + 'img/music/album.png',
    DEFAULT_PLAYLIST_ART = MEDIA_URL + 'img/music/playlist.png',
    ZERO = '0',
    THIRTYSEVEN = '37',
    THIRTYNINE = '39',
    SLIDER = 'slider',
    PLAYBUTTON = 'playButton',
    PREVBUTTON = 'prevButton',
    STOPBUTTON = 'stopButton',
    NEXTBUTTON = 'nextButton',
    PROGRESSBARCONT = 'progressBarCont',
    KEY = 'key',
    MUTECONTROL = 'muteControl',
    REPEATCONTROL = 'repeatControl',
    SHUFFLECONTROL = 'shuffleControl',
    HISTORY = 'history',
    VIEWCHANGE = 'viewChange',
    PLAYLISTCHANGE = 'playlistChange',
    INDEX = 'index',
    DISABLED = 'disabled',
    ENABLED = 'enabled',
    MED = 'med',
    LOW = 'low',
    VIEW = 'view',
    HIDDEN = 'hidden',
    SONG = 'song',
    SONGS = SONG + 's',
    DATASOURCE = 'datasource',
    NOWPLAYING = 'nowplaying',
    CURRENT = 'current',
    HEIGHT = 'height',
    PLAYLIST = 'playlist',
    PLAYLISTS = PLAYLIST + 's',
    VISIBLE = 'visible',
    CHECKED = 'checked',
    TR = 'tr',
    EMPTY = '',
    ARTISTS = 'artists',
    ALBUMS = 'albums',
    WIDTH = 'width',
    SELECTEDSONGS = 'selectedSongs',
    VISIBLECHANGE = 'visibleChange',
    VALUE = 'value',
    MYSONGS = 'my-songs',
    SEARCH = 'search',
    PROGBARRIGHTARRCOUNT = 'progBarRightArrCount',
    PROGBARLEFTARRCOUNT = 'progBarLeftArrCount',
    REPEAT = 'repeat',
    SHUFFLE = 'shuffle',

    DOTACTIVESONGSVIEW = '.active-songs-view',
    DOTLEFT = '.left',
    DOTRIGHT = '.right',

    MUSICPLAYSONGATINDEX = 'music:playSongAtIndex',
    MUSICSETPLAYLIST = 'music:setPlaylist',
    MUSICUPDATESELECTED = 'music:updateSelected',
    MUSICADDSONGSTOPLAYLIST = 'music:addSongsToPlaylist',
    MUSICREMOVEFROMNOWPLAYING = 'music:removeFromNowPlaying',
    MUSICUPDATECOLPLAYBUTTON = 'music:updateColPlayButton',
    MUSICUPDATECURRENTTRACKROW = 'music:updateCurrentTrackRow',
    MUSICCHECKBOXESUPDATED = 'music:checkboxesUpdated',
    MUSICCHECKBOXCHANGED = 'music:checkboxChanged',
    MOVIESTARRX = /^aac|mp4|m4a$/,
    namespace = Y.namespace('Music'),
    ToggleView,
    SongView;

ToggleView = function() {
    Y.Do.before(this.beforeShow, this, 'show');
    Y.Do.after(this.afterHide, this, 'hide');
};
ToggleView.ATTRS = {events: {value: []}};
ToggleView.prototype = {
    afterHide: function() {
        var events = this.get('events');
        while (events.length) {
            events.pop().detach();
        }
    },
    beforeShow: function() {
        if (this.get("rendered") === true) {
            this.bindUI();
        }
    }
};

namespace.PlayerView = Y.Base.create('player', Y.Widget, [], {
    renderUI: function() {
        var slider = new Y.Music.Slider({
            valueChangeHandler: Y.bind(this.onVolumeChange, this),
            visible: false,
            zIndex: 5
        });
        slider.render();
        this.set(SLIDER, slider);
    },
    bindUI: function() {

        this.get(PLAYBUTTON).on(CLICK, this.emitPlay, this);
        this.get(PREVBUTTON).on(CLICK, this.emitPrevious, this);
        this.get(STOPBUTTON).on(CLICK, this.emitPause, this);
        this.get(NEXTBUTTON).on(CLICK, this.emitNext, this);
        this.get(PROGRESSBARCONT).on(CLICK, this.onProgressBarClick, this);
        this.get(PROGRESSBARCONT).on('blur', this.onProgressBarBlur, this);
        Y.on(KEY, this.onProgressBarKeyDown, this.get(PROGRESSBARCONT), 'down: 37, 39', this);
        Y.on(KEY, this.onProgressBarKeyUp, this.get(PROGRESSBARCONT), 'up: 37, 39', this);

        this.get(MUTECONTROL).on(CLICK, this.onMuteClick, this);
        this.get('volumeControl').on(CLICK, this.onVolumeClick, this);
        this.get(REPEATCONTROL).on(CLICK, this.onRepeatClick, this);
        this.get(SHUFFLECONTROL).on(CLICK, this.onShuffleClick, this);
        this.get(HISTORY).after(VIEWCHANGE, this.afterViewChange, this);

        this.after('indexChange', this.afterIndexChange, this);
        this.after(PLAYLISTCHANGE, this.afterPlaylistChange, this);
        // This is so nowPlaying is updated as soon as
        // something is added to the queue.
        this.after(PLAYLISTCHANGE, this.afterPlay, this);
        this.after('muteChange', this.afterMuteChange, this);
        this.after('repeatChange', this.afterRepeatChange, this);
        this.after('shuffleChange', this.afterShuffleChange, this);

        Y.Global.on(EVENTSTOP, this.onStop, this);
        Y.Global.on(EVENTPAUSE, this.onPause, this);
        Y.Global.on(EVENTSTOP, this.onVuMeterEnd, this);
        Y.Global.on(EVENTSTOP, this.resetTime, this);
        Y.Global.on(EVENTPAUSE, this.onVuMeterEnd, this);
        Y.Global.on(EVENTPLAY, this.onPlay, this);
        Y.Global.on(MUSICPLAYSONGATINDEX, this.onPlaySongAtIndex, this);
        Y.Global.on(EVENTNEXT, this.onNext, this);
        Y.Global.on(EVENTPREVIOUS, this.onPrevious, this);
        Y.Global.on(EVSOUNDPLAYING, Y.throttle(Y.bind(this.whilePlaying, this), 100), this);
        Y.Global.on(EVSOUNDLOADING, this.whileLoading, this);
        Y.Global.on(MUSICSETPLAYLIST, this.setPlaylist, this);
        Y.Global.on(EVSOUNDFINISH, this.continuePlaylist, this);
        Y.Global.on(MUSICUPDATESELECTED, this.onUpdateSelected, this);
        Y.Global.on(MUSICADDSONGSTOPLAYLIST, this.onAddSongsToPlaylist, this);
        Y.Global.on(MUSICREMOVEFROMNOWPLAYING, this.onRemoveFromNowPlaying, this);
        Y.Global.on(MUSICUPDATECOLPLAYBUTTON, this.onUpdateColPlayButton, this);
        Y.Global.on(MUSICUPDATECURRENTTRACKROW, this.updateCurrentTrackRow, this);
        Y.Global.after(EVENTPLAY, this.afterPlay, this);
        Y.Unity.onInit(Y.bind(this.bindUnity, this));
    },
    afterPlaylistChange: function(e) {
        var index = this.get(INDEX),
        playlist = e.newVal,
        nextButton = this.get(NEXTBUTTON),
        playButton = this.get(PLAYBUTTON),
        prevButton = this.get(PREVBUTTON),
        mp;

        if (playlist.length === 0) {
            this.set(INDEX, 0);
            if (!nextButton.hasClass(DISABLED)) {
                nextButton.addClass(DISABLED);
                nextButton.set(DISABLED, DISABLED);
            }
            if (!playButton.hasClass(DISABLED)) {
                playButton.addClass(DISABLED);
                playButton.set(DISABLED, DISABLED);
            }
            if (!prevButton.hasClass(DISABLED)) {
                prevButton.addClass(DISABLED);
                prevButton.set(DISABLED, DISABLED);
            }
        } else {
            playButton.removeClass(DISABLED);
            playButton.removeAttribute(DISABLED);
        }
        if (index === 0 || playlist.length === 0) {
            if (!prevButton.hasClass(DISABLED)) {
                prevButton.addClass(DISABLED);
                prevButton.set(DISABLED, DISABLED);
            }
        } else {
            prevButton.removeClass(DISABLED);
            prevButton.removeAttribute(DISABLED);
        }
        if (index + 1 >= playlist.length) {
            if (!nextButton.hasClass(NEXTBUTTON)) {
                nextButton.addClass(NEXTBUTTON);
            }
        } else {
            nextButton.removeClass(DISABLED);
            nextButton.removeAttribute(DISABLED);
        }
        if (Y.Unity.ready) {
            mp = Y.Unity.Unity.MediaPlayer;
            mp.setCanPlay(playlist.length !== 0);
            mp.setCanPause(playlist.length !== 0);
            mp.setCanGoPrevious(index !== 0 && playlist.length !== 0);
            mp.setCanGoNext(index + 1 < playlist.length);
        }
        Y.Global.fire(MUSICCHECKBOXESUPDATED);
    },
    onUpdateSelected: function(e) {
        var song = e.song,
        index = this.get(INDEX),
        history = this.get(HISTORY),
        view = history.get(VIEW),
        box = Y.one(".active-songs-view"),
        current;

        if (song && box) {
            if (view && view === NOWPLAYING) {
                current = box.one('[data-u1m-songid='+song.sID+'][data-u1m-npindex='+index+']');
            } else {
                current = box.one('[data-u1m-songid="'+song.sID+'"]');
            }
            box.all('.current').removeClass(CURRENT);

            if (current !== null) {
                current.addClass(CURRENT);
            }
            Y.Global.fire(MUSICUPDATECOLPLAYBUTTON, { song: song });
        }
    },
    onVuMeterEnd: function() {
        var vuMeter = Y.one(".active-songs-view .vu"),
            vuLeft, vuRight;
        if (vuMeter) {
            vuLeft = vuMeter.one(DOTLEFT);
            vuRight = vuMeter.one(DOTRIGHT);
            if (vuRight && vuLeft) {
                vuLeft.transition({
                    duration: 1,
                    easing: 'ease-in',
                    height: 0
                });
                vuRight.transition({
                    duration: 1,
                    easing: 'ease-in',
                    height: 0
                });
            }
        }
    },
    onUpdateColPlayButton: function(e) {
        var song = e && e.song || null,
        oNodeRow = Y.one(".active-songs-view .current");
        this.updateCurrentTrackRow({"node": oNodeRow, song: song});
    },
    updateCurrentTrackRow: function(e) {
        var song = e.song || Y.Music.lastSong,
        oNodeRow = e.node,
        box = Y.one(DOTACTIVESONGSVIEW),
        currentButton,
        currentVuMeter,
        button = null,
        vuMeter = this.get("vuMeter");

        if (box) {
            currentButton = box.one('button.hidden');
            currentVuMeter = box.one('.vu');
        }

        if (currentButton) {
            currentButton.removeClass(HIDDEN);
        }

        if (currentVuMeter) {
            currentVuMeter.remove();
        }

        if (oNodeRow !== null) {
            button = oNodeRow.one("button.play");
            button.get(PARENTNODE).appendChild(vuMeter);
            button.addClass(HIDDEN);
            oNodeRow.addClass(CURRENT);
        }

        if (song && vuMeter) {
            if (!song.isHTML5 && soundManager.flashVersion >= 9 && song.instanceOptions.usePeakData) {
                vuMeter.addClass("has-peak-data");
            } else {
                vuMeter.removeClass("has-peak-data");
                vuMeter.one(DOTLEFT).setStyle("height", 0);
                vuMeter.one(DOTRIGHT).setStyle("height", 0);
            }
        }
    },
    onProgressBarClick: function(e) {
        e.preventDefault();
        var x,
        target = e.currentTarget,
        song = Y.Music.lastSong,
        percOfTrack,
        newPosition,
        duration = this.getDuration();

        if (song){
            if (e.pageX || e.pageY) {
                x = e.pageX;
            }
            else {
                x = e.clientX + document.body.scrollLeft +
                    document.documentElement.scrollLeft;
            }
            x -= parseInt(target.getX(), 10);
            percOfTrack = parseInt((100 / parseInt(target.getComputedStyle(WIDTH), 10)) * x, 10);
            newPosition = parseInt((duration / 100 * percOfTrack), 10);
            if (newPosition) {
                song.setPosition(newPosition);
            }
        }
    },
    onProgressBarKeyDown: function(e){
        e.preventDefault();
        var song = Y.Music.lastSong,
        keyCode = e.keyCode,
        leftArrCount = this.get(PROGBARLEFTARRCOUNT),
        rightArrCount = this.get(PROGBARRIGHTARRCOUNT),
        leftMultiplier = (leftArrCount / 10),
        rightMultiplier = (rightArrCount / 10);

        leftMultiplier = leftMultiplier > 0 ? leftMultiplier : 1;
        rightMultiplier = rightMultiplier > 0 ? rightMultiplier : 1;

        if (song && leftArrCount === 0 || rightArrCount === 0) {
            song.pause();
        }
        // Right Arrow Key
        if (keyCode === THIRTYNINE) {
            song.setPosition(song.position + (100 * rightMultiplier));
            this.set(PROGBARRIGHTARRCOUNT, rightArrCount + 1);
            // Left Arrow key
        } else if (keyCode === THIRTYSEVEN){
            song.setPosition(song.position - (100 * leftMultiplier));
            this.set(PROGBARLEFTARRCOUNT, leftArrCount + 1);
        }
        this.whilePlaying();
    },
    onProgressBarKeyUp: function(e){
        e.preventDefault();
        var song = Y.Music.lastSong,
        keyCode = e.keyCode;

        if (song && song.paused) {
            song.togglePause();
        }
        // Right Arrow Key
        if (keyCode === THIRTYNINE) {
            this.set(PROGBARRIGHTARRCOUNT, 0);
            // Left Arrow key
        } else if (keyCode === THIRTYSEVEN){
            this.set(PROGBARLEFTARRCOUNT, 0);
        }
    },
    onProgressBarBlur: function() {
        var song = Y.Music.lastSong,
        leftArrCount = this.get(PROGBARLEFTARRCOUNT),
        rightArrCount = this.get(PROGBARRIGHTARRCOUNT);

        if (leftArrCount > 0 || rightArrCount > 0) {
            if (song && song.paused) {
                song.togglePause();
            }
            this.set(PROGBARLEFTARRCOUNT, 0);
            this.set(PROGBARRIGHTARRCOUNT, 0);
        }
    },
    updatePeakData: function () {
        var vuMeter = Y.one(".vu"),
        vuLeft, vuRight,
        song = Y.Music.lastSong;

        if (vuMeter) {
            vuLeft = vuMeter.one(DOTLEFT);
            vuRight = vuMeter.one(DOTRIGHT);
            if (vuRight && vuLeft) {
                vuLeft.setStyle(HEIGHT, Math.ceil(17*song.peakData.left)+'px');
                vuRight.setStyle(HEIGHT, Math.ceil(17*song.peakData.right)+'px');
            }
        }
    },
    getTime: function(nMSec, bAsString) {
        // convert milliseconds to mm:ss, return as object literal or string
        var nSec = Math.floor(nMSec/1000),
        min = Math.floor(nSec/60),
        sec = nSec-(min*60);
        return (bAsString?(min+':'+(sec<10?ZERO+sec:sec)):{'min':min,'sec':sec});
    },
    updateTime: function() {
        var totalTime = Y.one(".time-display .total"),
        currentTime = Y.one(".time-display .current"),
        song = Y.Music.lastSong,
        duration = this.getDuration();

        if (totalTime && currentTime) {
            totalTime.setContent(this.getTime(duration, true));
            currentTime.setContent(this.getTime(song.position, true));
        }
    },
    resetTime: function() {
        var blankTime = "-:--",
        totalTime = Y.one(".time-display .total"),
        currentTime = Y.one(".time-display .current");

        if (totalTime && currentTime) {
            totalTime.setContent(blankTime);
            currentTime.setContent(blankTime);
        }
    },
    onMuteClick: function(e) {
        e.preventDefault();
        var newVal = !this.get('mute');
        this.set('mute', newVal);
    },
    afterMuteChange: function(e) {
        var song = Y.Music.lastSong;
        if (e.newVal === true) {
            this.get(MUTECONTROL).addClass(ENABLED);
            if (song) { song.mute(); }
        } else {
            this.get(MUTECONTROL).removeClass(ENABLED);
            if (song) { song.unmute(); }
        }
    },
    onPlaySongAtIndex: function(e) {
        var index = e.index;
        this.set(INDEX, index);
        Y.Global.fire(EVSOUNDPLAY);
    },
    onShuffleClick: function(e) {
        e.preventDefault();
        var newVal = !this.get(SHUFFLE);
        this.set(SHUFFLE, newVal);
    },
    afterShuffleChange: function(e) {
        if (e.newVal === true) {
            this.get(SHUFFLECONTROL).addClass(ENABLED);
        } else {
            this.get(SHUFFLECONTROL).removeClass(ENABLED);
        }
        this.setPlaylistContext();
    },
    onRepeatClick: function(e) {
        e.preventDefault();
        var newVal = !this.get(REPEAT);
        this.set(REPEAT, newVal);
    },
    afterRepeatChange: function(e) {
        if (e.newVal === true) {
            this.get(REPEATCONTROL).addClass(ENABLED);
        } else {
            this.get(REPEATCONTROL).removeClass(ENABLED);
        }
        this.setPlaylistContext();
    },
    onVolumeClick: function(e) {
        var slider = this.get(SLIDER);
        e.halt();
        if (slider.get(VISIBLE)) {
            slider.hide();
        } else {
            slider.show();
            slider.set('align', {
                node: '.volume',
                points: [Y.WidgetPositionAlign.BC, Y.WidgetPositionAlign.TC]
            });
            Y.one('body').on(CLICK, function(e) {
                e.halt();
                if (!slider.get(BOUNDINGBOX).contains(e.target)) {
                    slider.hide();
                    e.currentTarget.detach();
                }
            });
        }
    },
    onVolumeChange: function(e) {
        /* The slider is actually reversed, so going up -> 0,
         * going down -> 100;
         */
                        var value = 100 - e.newVal,
                        volumeControl = this.get(MUTECONTROL),
                        song = Y.Music.lastSong ;

                        if (song) {
                            song.setVolume(value);
                        }
                        if (value >= 75) {
                            volumeControl.removeClass(ENABLED);
                            volumeControl.removeClass(LOW);
                            volumeControl.removeClass(MED);
                        } else if (value === 0) {
                            volumeControl.removeClass(MED);
                            volumeControl.removeClass(LOW);
                            volumeControl.addClass(ENABLED);
                        } else if (value <= 25) {
                            volumeControl.removeClass(ENABLED);
                            volumeControl.removeClass(MED);
                            volumeControl.addClass(LOW);
                        } else {
                            volumeControl.removeClass(ENABLED);
                            volumeControl.removeClass(LOW);
                            volumeControl.addClass(MED);
                        }
                    },
                    onRemoveFromNowPlaying: function(e){

                            var songIds = e.songIds,
                            length = songIds.length,
                            quantifier = (length === 1) ? SONG : SONGS,
                            playlist = this.get(PLAYLIST),
                            playlistLength = playlist.length,
                            deletedCurrent = false,
                            index = this.get(INDEX),
                            song = null,
                            message,
                            wasPlaying = false,
                            i, k;

                            if (length === playlistLength) {
                                Y.Global.fire("music:clearNowPlaying");
                                return;
                            }

                            message = EMPTY;
                            if (length === 0) {
                                /* IE is apparently weird about array length of EventFacade
                                 * items.
                                 */
        message = 'Songs removed from queue';
    } else {
        message = length + ' ' + quantifier + ' removed from the queue.';
    }

    for (i = 0; i < playlistLength; i++) {
        for (k = 0; k < length; k++) {
            if (playlist[i].id === songIds[k]) {
                if (i === this.get(INDEX)){
                    deletedCurrent = true;
                }
                playlist.splice(i, 1);
                i--;
                playlistLength = playlist.length;

                songIds.splice(k, 1);
                k--;
                length = songIds.length;
                break;
            }
        }
    }

    this.set(PLAYLIST, playlist);

    if (deletedCurrent === true) {
        song = Y.Music.lastSong;
        if (!song.paused) {
            wasPlaying = true;
        }
        Y.Global.fire(EVENTSTOP);
    }

    if (index === playlistLength) {
        this.set(INDEX, index - 1);
    } else {
        this.set(INDEX, index);
    }

    if (wasPlaying){
        Y.Global.fire(EVENTPLAY);
    }

    Y.Global.fire('music:nowPlayingButtonsUpdated');
    Y.Global.fire('one:info', {'message': message});
},
onAddSongsToPlaylist: function(e) {
    var songs = e.songs,
    length = songs.length,
    quantifier = (length === 1) ? SONG : SONGS,
    injectSong = e.injectSong || false,
    playNow = e.playNow || false,
    playlist = this.get(PLAYLIST),
    playlistLength = playlist.length,
    currentIndex,
    i, message;

    currentIndex = this.get(INDEX);
    if (injectSong && playlistLength > 0) {
        for (i = length-1; i >= 0; i--) {
            playlist.splice(currentIndex+1, 0, songs[i]);
        }
        this.set(PLAYLIST, playlist);
        this.set(INDEX, currentIndex + 1);
    } else {
        this.set(PLAYLIST, playlist.concat(songs));
    }
    message = EMPTY;
    if (length === 0) {
        /* IE is apparently weird about array length of EventFacade
         * items.
         */
        message = 'Songs added to queue';
    } else {
        message = length + ' ' + quantifier + ' added to the queue.';
    }
    if (playNow === true) {
        Y.Global.fire(EVENTPLAY);
    }
    Y.Global.fire('one:info', {'message': message});
},

setPlaylist: function(e) {
    Y.Global.fire(EVENTSTOP);
    this.set(INDEX, 0);
    this.set(PLAYLIST, e.playlist);
},

afterViewChange: function(e) {
    if (e.newVal === NOWPLAYING) {
        Y.Global.fire('music:nowPlaying', {
            songs: this.get(PLAYLIST) });
    }
},

afterPlay: function() {
               var index = this.get(INDEX),
               playlist = this.get(PLAYLIST),
               current = playlist[index],
               length = playlist.length,
               albumArtImage = Y.one(".albumart img"),
               albumArtImageGenerated = false,
               albumArtContainer = null,
               mp,
               trackInfo;

               // Generate the image elem if it doesn't already
               // exist.
               if (!albumArtImage) {
                   albumArtImage = Y.Node.create("<img>");
                   albumArtImageGenerated = true;
               }
               if (current === undefined) {
                   return;
               }
               Y.all(".player .hidden").removeClass(HIDDEN);
               Y.one(".player .start-here").addClass(HIDDEN);
               this.get('album').setContent(Y.Escape.html(current.album));
               this.get('artist').setContent(Y.Escape.html(current.artist));
               this.get('title').setContent(Y.Escape.html(current.title));
               this.setPlaylistContext();
               this.get(DATASOURCE).getAlbumArt(current, albumArtImage);

               if (albumArtImageGenerated === true) {
                   albumArtContainer = Y.one(".albumart");
                   if (albumArtContainer) {
                       albumArtContainer.appendChild(albumArtImage);
                   }
               }

               if (Y.Unity.ready) {
                   mp = Y.Unity.Unity.MediaPlayer;
                   trackInfo = {
                       title: current.title,
                       artist: current.artist,
                       album: current.album,
                       artLocation: albumArtImage.get('src')
                   };
                   mp.setTrack(trackInfo);
                   // And show a notification
                   Y.Unity.Unity.Notification.showNotification(
                       trackInfo.title,
                       "by " + trackInfo.artist + " from " + trackInfo.album,
                       trackInfo.artLocation);
               }

               this.get('header').setContent(
                   'Now playing ' + (index + 1) + ' of ' + length + ' songs');
},
setPlaylistContext: function() {
    var index = this.get(INDEX),
    repeat = this.get(REPEAT),
    shuffle = this.get(SHUFFLE),
    playlist = this.get(PLAYLIST),
    previous = playlist[index - 1],
    previousText = previous ? 'Previous: ' + Y.Escape.html(previous.title) + ' - ' + Y.Escape.html(previous.artist) : EMPTY,
    next = playlist[index + 1],
    nextText = next ? 'Next: ' + Y.Escape.html(next.title) + ' - ' + Y.Escape.html(next.artist): EMPTY,
    previousNode,
    nextNode;

    if (shuffle === false && repeat === false) {
        previousNode = this.get('previous');
        if (previous) {
            previousNode.setContent(previousText);
            previousNode.setStyle('display', 'block');
        } else {
            previousNode.setStyle('display', 'none');
        }
        nextNode = this.get('next');
        if (next) {
            nextNode.setContent(nextText);
            nextNode.setStyle('display', 'block');
        } else {
            nextNode.setStyle('display', 'none');
        }
    } else {
        this.get('previous').setContent(null);
        this.get('next').setContent(null);
    }
},
afterIndexChange: function(e) {
    var index = e.newVal,
    playlist = this.get(PLAYLIST),
    nextButton = this.get(NEXTBUTTON),
    prevButton = this.get(PREVBUTTON),
    mp;

    if (index === 0) {
        if (!prevButton.hasClass(DISABLED)) {
            prevButton.addClass(DISABLED);
            prevButton.set(DISABLED, DISABLED);
        }
    } else {
        prevButton.removeClass(DISABLED);
        prevButton.removeAttribute(DISABLED);
    }
    if (index + 1 >= playlist.length) {
        if (!nextButton.hasClass(DISABLED)) {
            nextButton.addClass(DISABLED);
            nextButton.set(DISABLED, DISABLED);
        }
    } else {
        nextButton.removeClass(DISABLED);
        nextButton.removeAttribute(DISABLED);
    }
    if (Y.Unity.ready) {
        mp = Y.Unity.Unity.MediaPlayer;
        mp.setCanGoPrevious(index !== 0);
        mp.setCanGoNext(index + 1 < playlist.length);
    }

    this.get("bytesLoadedBar").setStyle(WIDTH, 0);
    Y.Global.fire(EVENTPLAY);
},
onPause: function() {
    Y.Music.lastSong.pause();
    this.get(PLAYBUTTON).show();
    this.get(STOPBUTTON).hide();
    if (Y.Unity.ready) {
        var mp = Y.Unity.Unity.MediaPlayer;
        mp.setPlaybackState(mp.PlaybackState.PAUSED);
    }
},
onStop: function() {
    var lastSong = Y.Music.lastSong,
    mp;

    this.get("bytesLoadedBar").setStyle(WIDTH, 0);
    if (lastSong) {
        lastSong.destruct();
        Y.Music.lastSong = null;
    }
    this.get(PLAYBUTTON).show();
    this.get(STOPBUTTON).hide();
    this.get('progressBar').setStyle(WIDTH, ZERO);
    if (Y.Unity.ready) {
        mp = Y.Unity.Unity.MediaPlayer;
        mp.setPlaybackState(mp.PlaybackState.PAUSED);
    }
},

onPlay: function() {
    var index = this.get(INDEX),
    playlist = this.get(PLAYLIST),
    songObj = playlist[index],
    newSong = soundManager.getSoundById(songObj.id),
    lastSong = Y.Music.lastSong,
    songURI = this.get(DATASOURCE).getStreamingURI(songObj),
    mute = this.get('mute'),
    volume = 100 - this.get(SLIDER).getValue(),
    nAPIDurationMilSecs,
    soundData,
    mp;

    if (playlist.length === 0) {
        Y.Global.fire('music:emptyPlaylist');
        Y.Music.lastSong = null;
        return;
    }

    if (songURI === null) {
        return;
    }

    if (newSong) {
        if (newSong && lastSong && newSong === lastSong) {
            // If readyState not failed/error
            if (newSong.readyState !== 2) {
                if (!newSong.paused) {
                    lastSong.stop();
                }
                newSong.play({ 'volume': volume });
            }
        } else {
            // Stop the previous song
            if (lastSong) {
                lastSong.stop();
            }
            // Start the new one
            newSong.play();
        }
    } else {
        this.get("bytesLoadedBar").setStyle(WIDTH, 0);
        if (lastSong) {
            lastSong.stop();
        }

        soundData = {
            id: songObj.id,
            url: songURI,
            onfinish: function() {
                Y.Global.fire(EVSOUNDFINISH);
            },
            onpause: function() {
                Y.Global.fire(EVSOUNDPAUSE);
            },
            onplay: function() {
                Y.Global.fire(EVSOUNDPLAY);
            },
            onstop: function() {
                Y.Global.fire(EVSOUNDSTOP);
            },
            whileplaying: function() {
                Y.Global.fire(EVSOUNDPLAYING);
            },
            whileloading: function() {
                Y.Global.fire(EVSOUNDLOADING);
            },
            usePeakData: true,
            multiShot: false,
            isMovieStar: MOVIESTARRX.test(songObj.suffix)
        };

        // Inform SM2 about ogg fixes LP: 1002481, 921841
        if (songObj.suffix === "ogg") {
            soundData.type = "audio/ogg";
        }

        newSong = soundManager.createSound(soundData);
        newSong.setVolume(volume);
        newSong.onposition(1000, function() {
            Y.fire('loaded');
        });
        if (mute) {
            newSong.mute();
        }
        newSong.play();
        newSong.whilePlaying = function() {
            Y.Global.fire(EVSOUNDPLAYING);
        };
        Y.Global.fire(EVENTSTARTED, {song: songObj});
    }

    this.get(STOPBUTTON).show();
    this.get(PLAYBUTTON).hide();

    nAPIDurationMilSecs = songObj.duration;
    newSong.nAPIDurationMilSecs = (nAPIDurationMilSecs) ? nAPIDurationMilSecs * 1000 : 0;
    Y.Music.lastSong = newSong;
    Y.Global.fire(MUSICUPDATESELECTED, { song: newSong });

    if (Y.Unity.ready) {
        mp = Y.Unity.Unity.MediaPlayer;
        mp.setPlaybackState(mp.PlaybackState.PLAYING);
    }
},
onPrevious: function() {
    var song = Y.Music.lastSong,
    index;

    if (song.position > 5000 && !song.paused) {
        Y.Global.fire(EVENTSTOP);
        Y.Global.fire(EVENTPLAY);
    } else {
        index = this.get(INDEX);
        if (index <= 0) { return; }
        index--;
        this.set(INDEX, index);
    }
},
onNext: function() {
    var index = this.get(INDEX),
    newIndex,
    shuffle = this.get(SHUFFLE),
    length = this.get(PLAYLIST).length;
    if (shuffle === true) {
        if (length <= 1) { return; }
        newIndex = null;
        do {
            newIndex = Math.floor(Math.random() * length);
        } while (index === newIndex);
        index = newIndex;
    } else if (index+1 >= length) {
        return;
    } else {
        index++;
    }
    this.set(INDEX, index);
},
continuePlaylist: function() {
    Y.Global.fire(EVENTSTOP);
    var index = this.get(INDEX),
    repeat = this.get(REPEAT),
    length = this.get(PLAYLIST).length;
    if (repeat) {
        Y.Global.fire(EVENTPLAY);
    } else {
        Y.Global.fire(EVENTNEXT);
        if (index+1 < length) {
            Y.Global.fire(EVENTPLAY);
        }
    }
},
getDuration: function() {
    var song = Y.Music.lastSong,
    duration = song.nAPIDurationMilSecs,
    songDuration = song.duration,
    songDurationEstimate = song.durationEstimate,
    songLoaded = song.loaded;

    if (!duration && !songLoaded && songDurationEstimate) {
        duration = songDurationEstimate;
    }

    if (songLoaded && songDuration) {
        duration = songDuration;
    }

    return duration;
},
whilePlaying: function() {
    var song = Y.Music.lastSong,
    progressBar = this.get('progressBar'),
    duration = this.getDuration(),
    width = Math.floor((100 * song.position) / duration) + 1;

    progressBar.setStyle(WIDTH, width + '%');
    if (soundManager.flashVersion >= 9 && song.instanceOptions.usePeakData) {
        this.updatePeakData();
    }
    this.updateTime();
},
whileLoading: function() {
    var song = Y.Music.lastSong,
    bytesLoadedBar = this.get('bytesLoadedBar'),
    maxWidth = Math.ceil((song.bytesLoaded/song.bytesTotal)*100),
    width = maxWidth > 100 ? 100 : maxWidth;
    bytesLoadedBar.setStyle(WIDTH, width + '%');
},
emitNext: function(e) {
    e.halt();
    if (e.target.hasClass(DISABLED)) { return; }
    Y.Global.fire(EVENTNEXT);
},
emitPlay: function(e) {
    e.halt();
    if (e.target.hasClass(DISABLED)) { return; }
    Y.Global.fire(EVENTPLAY);
},
emitPrevious: function(e) {
    e.halt();
    if (e.target.hasClass(DISABLED)) { return; }
    Y.Global.fire(EVENTPREVIOUS);
},
emitStop: function(e) { e.halt(); Y.Global.fire(EVENTSTOP); },
emitPause: function(e) { e.halt(); Y.Global.fire(EVENTPAUSE); },
bindUnity: function() {
    Y.log("Configuring Unity integration");
    var mp = Y.Unity.Unity.MediaPlayer;
    mp.init("Ubuntu One Music");
    mp.setCanPlay(true);
    mp.setCanPause(true);
    mp.setCanGoNext(true);
    mp.setCanGoPrevious(true);

    mp.onPlayPause(Y.bind(function() {
        Y.log("Sound Menu: play/pause");
        if (this.get(PLAYBUTTON).getStyle('display') !== "none") {
            Y.Global.fire(EVENTPLAY);
        } else {
            Y.Global.fire(EVENTPAUSE);
        }
    }, this));
    mp.onNext(function() {
        Y.log("Sound Menu: next track");
        Y.Global.fire(EVENTNEXT);
    });
    mp.onPrevious(function() {
        Y.log("Sound Menu: previous track");
        Y.Global.fire(EVENTPREVIOUS);
    });
}
}, {
    ATTRS: {
        CONTENT_TEMPLATE: { value: null },
        album: { value: null },
        albumArt: { value: null },
        artist: { value: null },
        datasource: { value: null },
        header: { value: null },
        history: { value: null },
        next: { value: null },
        previous: { value: null },
        title: { value: null },
        mute: { value: false },
        shuffle: { value: false },
        repeat: { value: false },
        muteControl: { value: null },
        volumeControl: { value: null },
        repeatControl: { value: null },
        shuffleControl: { value: null },
        index: { value: 0 },
        nextButton: { value: null },
        vuMeter: { value: Y.Node.create(Y.one("#tpl-vu-meter").getContent()) },
        playlist: { value: [] },
        playButton: { value: null },
        prevButton: { value: null },
        progressBar: { value: null },
        progressBarCont: { value: null },
        progBarLeftArrCount: { value: 0 },
        progBarRightArrCount: { value: 0 },
        bytesLoadedBar: { value: null },
        stopButton: { value: null },
        timeDisplay: { value: null },
        volume: { value: null },
        volumeIndicator: { value: null },
        volumeNode: { value: null }
    },
    HTML_PARSER: {
        album: '.album',
        artist: '.artist',
        header: '.header',
        next: '.next-song',
        previous: '.previous-song',
        title: '.title',

        muteControl: '#mute',
        volumeControl: '#volume',
        repeatControl: '#repeat',
        shuffleControl: '#shuffle',
        nextButton: '#next',
        playButton: '#play',
        prevButton: '#prev',
        progressBar: '.progress',
        progressBarCont: '.progress-bar',
        bytesLoadedBar: '.bytes-loaded',
        stopButton: '#stop',
        timeDisplay: '.time-display span',
        volumeIndicator: '.volume-indicator',
        volumeNode: '.volume-slider'
    }
});

namespace.AlbumListView = Y.Base.create('albumView', Y.Widget, [ToggleView], {
    initializer: function() {
        this.get(HISTORY).after(VIEWCHANGE, this.afterViewChange, this);
        this.render();
    },
    bindUI: function() {
        var boundingBox = this.get('inner'),
        events = this.get('events');
        events.push(boundingBox.delegate(CLICK, this.onAlbumClick, '.album-listing', this));
        events.push(boundingBox.delegate(CLICK, this.onPlayAllClick, '.album-listing button', this));

        events.push(this.before(VISIBLECHANGE, this.beforeVisibleChange, this));
        events.push(this.after('albumsChange', this.afterAlbumsChange, this));
    },
    syncUI: function() {
        var view = this.get(HISTORY).get(VIEW);
        if (view === ALBUMS) {
            this.show();
        } else {
            this.get('inner').all('.album-listing').remove();
            this.hide();
        }
    },
    onAlbumClick: function(e) {
        e.preventDefault();
        if (e.target.get('nodeName').toLowerCase() === 'button' || e.target.ancestor("button") !== null) {
            return;
        }
        var target = e.currentTarget,
        albumId = target.one('a').getAttribute('data-u1m-albumid'),
        history = this.get(HISTORY);
        history.add(Y.mix({
            album: albumId,
            view: SONGS
        }, HISTORYRESET));
    },
    onPlayAllClick: function(e) {
        var albumId = e.target.ancestor(".album-listing").one('a').getAttribute('data-u1m-albumid'),
        datasource = this.get(DATASOURCE);
        datasource.getAlbum(albumId, function(e) {
            if (e.response.json.directory && e.response.json.directory.child) {
                var songs = e.response.json.directory.child;
                Y.Global.fire(MUSICADDSONGSTOPLAYLIST, {
                    songs: songs,
                    'injectSong': true,
                    'playNow': true
                });
            }
        });
    },
    afterAlbumsChange: function() {
        var albums = this.get(ALBUMS),
        album,
        count = albums.length,
        box = this.get('inner'),
        imageGroup = new Y.U1.ImgLoadGroup({
            foldDistance: 30,
            foldCheckDelay: 500
        }),
        escapeHtml = function(k,v){
            return Y.Escape.html(v);
        },
        i,
        bbViews,
        album_art,
        albumNode,
        temp_vars,
        img_id,
        placeholder,
        link;

        this.set("imageGroup", imageGroup);

        box.setContent(EMPTY);
        for (i=0; i<count; i++) {
            album = albums[i],
            album_art = '/api/music/v2/albums/' + album.u1CoverId + '/art/',
            img_id = Y.guid(),
            albumNode = Y.Node.create(Y.substitute(Y.one("#tpl-album-listing").getContent(), album, escapeHtml)),
            temp_vars = { "img_id": img_id, "default_art_url": DEFAULT_ALBUM_ART },
            placeholder = Y.Node.create(Y.substitute(Y.one("#tpl-art").getContent(), temp_vars, escapeHtml)),
            link = albumNode.one('[data-u1m-albumid="'+album.id+'"]');

            if (album.has_cover_art === true) {
                imageGroup.registerImage({
                    domId: img_id,
                    srcUrl: album_art
                });
            }

            albumNode.insertBefore(placeholder, link);
            box.appendChild(albumNode);
        }
        // This forces a redraw in IE8.
        if (Y.UA.ie === 8) {
            bbViews = Y.one(".views");
            if (bbViews) {
                bbViews.toggleClass("redraw");
            }
        }

    },
    beforeVisibleChange: function(e) {
        if (e.newVal === true) {
            var artist = this.get(HISTORY).get('artist');
            this.get(DATASOURCE).getAlbums(artist, Y.bind(this.onIOSuccess, this));
        }
    },
    onIOSuccess: function(e) {
        var json = e.response.json,
        albums = [],
        albumList,
        albumListLength,
        listLength,
        i, j,
        indexes,
        indexesLength;

        if (json.indexes) {
            this.get('header').setContent('All albums');
            indexes = json.indexes.index,
            indexesLength = indexes.length;
            for (i=0; i<indexesLength; i++) {
                albumList = indexes[i].album;
                listLength = albumList.length;
                for (j=0; j<listLength; j++) {
                    albums.push(albumList[j]);
                }
            }
        } else {
            if (json.directory && json.directory.child) {
                albumList = json.directory.child;
                albumListLength = albumList.length;
                this.get('header').setContent(Y.Escape.html(albumList[0].artist) + ' albums');
                for (i=0; i<albumListLength; i++) {
                    albums.push(albumList[i]);
                }
            }
        }
        this.set(ALBUMS, albums);
    },
    afterViewChange: function(e) {
        if (e.newVal === ALBUMS) {
            this.show();
        } else {
            var imageGroup = this.get("imageGroup");
            if (imageGroup) {
                imageGroup.destroy();
            }
            this.get('inner').all('.album-listing').remove();
            this.hide();
        }
    }
}, {
    ATTRS: {
        CONTENT_TEMPLATE: { value: null },
        albums: { value: [] },
        datasource: { value: null },
        history: { value: null },
        header: { value: null },
        inner: { value: null },
        visible: { value: false },
        imageGroup: { value: false }
    },
    HTML_PARSER: {
        header: '.view-header',
        inner: '.view-inner'
    }
});

namespace.ArtistListView = Y.Base.create('artistsView', Y.Widget, [ToggleView], {
    initializer: function() {
        this.get(HISTORY).on(VIEWCHANGE, this.onViewChange, this);
        this.render();
    },
    bindUI: function() {
        var events = this.get('events');

        events.push(this.get('inner').delegate(CLICK, this.onArtistClick, '.artist-listing', this));
        events.push(this.before(VISIBLECHANGE, this.beforeVisibleChange, this));
        events.push(this.after('artistsChange', this.afterArtistsChange, this));
    },
    syncUI: function() {
        var view = this.get(HISTORY).get(VIEW);
        if (view === ARTISTS) {
            this.show();
        } else {
            this.get('inner').all('.artist-listing').remove();
            this.hide();
        }
    },
    beforeVisibleChange: function(e) {
        if (e.newVal === true) {
            this.get(DATASOURCE).getArtists(Y.bind(this.onIOSuccess, this));
        }
    },
    afterArtistsChange: function() {
        var artists = this.get(ARTISTS),
        artist,
        count = artists.length,
        box = this.get('inner'),
        imageGroup = new Y.U1.ImgLoadGroup({
            foldDistance: 30,
            foldCheckDelay: 500
        }),
        escapeHtml = function(k,v){
            return Y.Escape.html(v);
        },
        i,
        bbViews,
        artistNode,
        temp_vars,
        img_id,
        placeholder,
        link,
        artist_art;

        this.set("imageGroup", imageGroup);

        box.setContent(EMPTY);
        for (i=0; i<count; i++) {
            artist = artists[i],
            artist_art = '/api/music/v2/artists/' + artist.u1CoverId + '/art/',
            img_id = Y.guid(),
            artistNode = Y.Node.create(Y.substitute(Y.one("#tpl-artist-listing").getContent(), artist, escapeHtml)),
            temp_vars = { "img_id": img_id, "default_art_url": DEFAULT_ALBUM_ART },
            placeholder = Y.Node.create(Y.substitute(Y.one("#tpl-art-artist").getContent(), temp_vars, escapeHtml)),
            link = artistNode.one('[data-u1m-artistid="'+artist.id+'"]');

            if (artist.has_cover_art === true) {
                imageGroup.registerImage({
                    domId: img_id,
                    srcUrl: artist_art
                });
            }

            artistNode.insertBefore(placeholder, link);
            box.appendChild(artistNode);
        }
        // This forces a redraw in IE8.
        if (Y.UA.ie === 8) {
            bbViews = Y.one(".views");
            if (bbViews) {
                bbViews.toggleClass("redraw");
            }
        }
    },
    onIOSuccess: function(e) {

        var indexes,
            indexesLength,
            artists = [],
            artistList,
            listLength,
            i, j;

        if (e.response.json && e.response.json.indexes) {
            indexes = e.response.json.indexes.index;
            indexesLength = indexes.length;
        }

        for (i=0; i<indexesLength; i++) {
            artistList = indexes[i].artist,
            listLength = artistList.length;
            for (j=0; j<listLength; j++) {
                artists.push(artistList[j]);
            }
        }
        this.set(ARTISTS, artists);
    },
    onViewChange: function(e) {
        var newView = e.newVal,
            imageGroup;

        if (newView === ARTISTS) {
            this.show();
        } else {
            imageGroup = this.get("imageGroup");
            if (imageGroup) {
                imageGroup.destroy();
            }
            this.get(BOUNDINGBOX).all('.artist-listing').remove();
            this.hide();
        }
    },
    onShowArtists: function(e) {
        var artists = e.artists,
        count = artists.length,
        box = this.get(BOUNDINGBOX),
        escapeHtml = function(k,v){
            return Y.Escape.html(v);
        },
        i;

        for (i=0; i<count; i++) {
            box.appendChild(Y.Node.create(
                Y.substitute(Y.one("#tpl-artist-listing").getContent(), artists[i], escapeHtml)));
        }
    },
    onArtistClick: function(e) {
        e.preventDefault();
        var target = e.currentTarget,
        artistId = target.one('a').getAttribute('data-u1m-artistid'),
        history = this.get(HISTORY);
        history.add(Y.mix({
            artist: artistId,
            view: ALBUMS
        }, HISTORYRESET));
    }
}, {
    ATTRS: {
        CONTENT_TEMPLATE: { value: null },
        artists: { value: [] },
        datasource: { value: null },
        history: { value: null },
        header: { value: null },
        inner: { value: null },
        imageGroup: { value: null }
    },
    HTML_PARSER: {
        header: '.view-header',
        inner: '.view-inner'
    }
});

namespace.MenuView = Y.Base.create('menu', Y.Widget, [], {
    renderUI: function() {
        this.get('playlistsds').getPlaylists(Y.bind(this.onIOSuccess, this));
    },
    bindUI: function() {
        this.get('searchButton').on(CLICK, this.onSearchClick, this);
        this.get('searchField').on('keypress', this.onSearchClick, this);
        this.get('albumsMenuItem').on(CLICK, this.onAlbumsClick, this);
        this.get('artistsMenuItem').on(CLICK, this.onArtistsClick, this);
        this.get('songsMenuItem').on(CLICK, this.onSongsClick, this);

        var history = this.get(HISTORY),
        playlists;

        history.on(VIEWCHANGE, this.syncUI, this);
        history.on(PLAYLISTCHANGE, this.syncUI, this);

        playlists = this.get(PLAYLISTS);
        playlists.delegate(CLICK, this.onPlaylistClick, '.playlist-listing', this);
        this.get('nowPlaying').on(CLICK, this.onNowPlayingClick, this);

        Y.Global.on('music:playlistsStale', this.renderUI, this);
    },
    syncUI: function() {
        Y.all(".u1-sidebar-menu .active").removeClass("active");
        var view = this.get(HISTORY).get(VIEW),
            playlist,
            playlistNode,
            menuItem;

        switch (view) {
            case ARTISTS:
                this.updateActive(ARTISTS);
            break;
            case NOWPLAYING:
                this.updateActive(NOWPLAYING);
            break;
            case ALBUMS:
                this.updateActive(ALBUMS);
            break;
            case MYSONGS:
                this.updateActive(MYSONGS);
            break;
            case PLAYLIST:
                playlist = this.get(HISTORY).get(PLAYLIST),
                playlistNode = Y.one('#menu #' + playlist);
                this.updateActive(playlistNode);
            break;
            case PLAYLISTS:
                menuItem = Y.one('#menu .more') || null;
                if (menuItem) {
                    menuItem.addClass('active');
                }
            break;
            default:
                break;
        }
    },
    updateActive: function(classOrNode) {
        var menuItem = Y.one("#menu ." + classOrNode) || classOrNode;
        if (Y.Lang.isObject(menuItem)) {
            menuItem.addClass("active");
        }
    },
    onSearchClick: function(e) {
        if (e.keyCode && e.keyCode !== 13) { return; }
        e.preventDefault();
        var term = this.get('searchField').get(VALUE),
        history = this.get(HISTORY);
        if (term === "") {
            Y.Global.fire('one:error', {
                message: 'Please enter a search term'
            });
            return;
        }
        history.add(Y.mix({
            search: term,
            view: SEARCH
        }, HISTORYRESET));
    },
    onIOSuccess: function(e) {
        var playlists = e.response.json.playlists.playlist,
        playlistsLength = playlists ? playlists.length : 0,
        playlistTemplate = Y.one('#playlist-listing').getContent(),
        playlistList = Y.one('.playlists ul'),
        escapeHtml = function(k,v){
            return Y.Escape.html(v);
        },
        i,
        playlist,
        node;

        if (playlistList) {
            playlistList.all("li:not(#shuffle-all)").remove();
        } else {
            playlistList = Y.Node.create('<ul></ul>');
        }
        if (playlistsLength > 0){
            GLOBAL_PLAYLIST_LIST = playlists;
            if (playlistsLength > 10) {
                playlists = playlists.slice(0, 10);
                playlistsLength = 10;
            }
            for (i=0; i<playlistsLength; i++) {
                playlist = playlists[i],
                node = Y.Node.create(
                    Y.substitute(playlistTemplate, playlist, escapeHtml));
                    playlistList.append(node);
            }
            if (playlistsLength >= 1) {
                playlistList.append(
                    Y.Node.create(Y.one('#more-playlists').getContent()));
            }
            Y.one('.playlists').append(playlistList);
        } else {
            GLOBAL_PLAYLIST_LIST = [];
        }
        this.syncUI();
    },
    onAlbumsClick: function(e) {
        e.preventDefault();
        var history = this.get(HISTORY);
        history.add(Y.mix({
            view: ALBUMS
        }, HISTORYRESET));
    },
    onArtistsClick: function(e) {
        e.preventDefault();
        this.get(HISTORY).add(Y.mix({
            view: ARTISTS
        }, HISTORYRESET));
    },
    onNowPlayingClick: function(e) {
        e.preventDefault();
        this.get(HISTORY).add(Y.mix({
            view: NOWPLAYING
        }, HISTORYRESET));
    },
    onPlaylistClick: function(e) {
        e.preventDefault();
        var parentNode = e.target.get(PARENTNODE);
        if (parentNode.hasClass('more')) {
            // 'More' link was clicked
            this.get(HISTORY).add(Y.mix({
                view: PLAYLISTS
            }, HISTORYRESET));
        } else {
            // Playlist was clicked
            this.get(HISTORY).add(Y.mix({
                playlist: parentNode.get('id'),
                view: PLAYLIST
            }, HISTORYRESET));
        }
    },
    onSongsClick: function(e) {
        e.preventDefault();
        this.get(HISTORY).add(Y.mix({
            view: MYSONGS
        }, HISTORYRESET));
        this.updateActive(MYSONGS);
    }
}, {
    ATTRS: {
        CONTENT_TEMPLATE: { value: null },
        albumsMenuItem: { value: null },
        artistsMenuItem: { value: null },
        datasource: { value: null },
        playlistsds: { value: null },
        history: { value: null },
        nowPlaying: { value: null },
        playlists: { value: null },
        searchField: { value: null },
        searchButton: { value: null },
        songsMenuItem: { value: null }
    },
    HTML_PARSER: {
        albumsMenuItem: '.albums',
        artistsMenuItem: '.artists',
        nowPlaying: '.nowplaying',
        playlists: '.playlists',
        searchField: '.u1-search-input',
        searchButton: '.u1-search-button',
        songsMenuItem: '.my-songs'
    }
});

SongView = Y.Base.create('songView', Y.Widget, [ToggleView], {
    initializer: function() {
        this.after('songsChange', this.afterSongsChange, this);
        this.get(HISTORY).after(VIEWCHANGE, this.afterViewChange, this);
    },
    bindUI: function() {
        var events = this.get('events'),
        selectAll = this.get('selectAll');
        if (selectAll) {
            events.push(selectAll.after(CLICK, this.afterSelectAllClick, this));
            events.push(selectAll.after('key', this.afterSelectAllClick, 'down: 32', this));
        }
        events.push(this.before(VISIBLECHANGE, this.beforeVisibleChange, this));
        events.push(this.before(VISIBLECHANGE, this.beforeVisibleChangeSongView, this));
        events.push(this.after(VISIBLECHANGE, this.afterVisibleChangeSongView, this));
        this.trackedSongs = new Y.Array([]);
        events.push(this.get(BOUNDINGBOX).delegate(CLICK, this.onRowClick, 'tbody tr', this));
        events.push(this.get(BOUNDINGBOX).delegate('key', this.onRowClick, 'tbody input[type=checkbox]', 'down:32', this));
        events.push(Y.Global.on(MUSICCHECKBOXCHANGED, this.onCheckboxChange, this));
        events.push(Y.Global.on(MUSICCHECKBOXESUPDATED, this.onCheckboxesUpdated, this));
    },
    syncUI: function() {
        if (this.get(HISTORY).get(VIEW) === this.get('viewName')) {
            this.show();
        } else {
            this.hide();
        }
    },
    onCheckboxChange: function(e) {
        var input = e.input,
        bChecked = input.get("checked"),
        index = null,
        songData,
        oSongFromCache = null,
        tr = input.ancestor(TR),
        trackedSongsLen = this.trackedSongs.length,
        i;

        if (tr) {
            if (bChecked === true) {
                tr.addClass(CHECKED);
            } else {
                tr.removeClass(CHECKED);
            }
            songData = tr.getData("song");
        }
        if (songData) {
            for (i=0; i<trackedSongsLen; i++) {
                if (this.trackedSongs[i].id === songData.id) {
                    oSongFromCache = this.trackedSongs[i];
                    index = i;
                    break;
                }
            }
            // Add songData if not already present
            if (bChecked === true) {
                if (oSongFromCache === null) {
                    this.trackedSongs.push(songData);
                }
                // Remove it
            } else if (bChecked === false) {
                if (oSongFromCache !== null && index !== null) {
                    this.trackedSongs.splice(index, 1);
                }
            }
        }
        Y.Global.fire(MUSICCHECKBOXESUPDATED);
    },
    onRowClick: function(e) {

        var target = e.target,
        currentTarget = e.currentTarget,
        input = currentTarget.one('input[type=checkbox]'),
        button = currentTarget.one('button.play'),
        tr,
        trs,
        index,
        view,
        song,
        history;

        if (target === button || button.contains(target) === true) {
            e.halt();
            trs = Y.all(".active-songs-view tbody tr");
            index = trs.indexOf(currentTarget);
            history = this.get(HISTORY);
            view = history.get(VIEW);

            if (view === NOWPLAYING || (view === PLAYLIST && this.get("isShufflePlaylist") === true)) {
                Y.Global.fire(MUSICPLAYSONGATINDEX, { index: index });
            } else {
                song = this.get(SONGS)[index];
                // Get the song object from data added to the tr
                // in the case of infinite scrolled data.
                tr = target.ancestor(TR);
                if (!song && tr) {
                    song = tr.getData(SONG);
                }

                Y.Global.fire(MUSICADDSONGSTOPLAYLIST,
                              { songs: [song],
                                  injectSong: true,
                                  playNow: true });
            }
            return;
        }

        if (input && target !== input) {
            tr = target.ancestor(TR);
            if (tr) {
                this.setCheckBox(input, !input.get(CHECKED));
            }
        } else if (input) {
            // Fire for natural checking of the checkbox
            Y.Global.fire(MUSICCHECKBOXCHANGED, { input: input });
        }

    },
    setCheckBox: function(ele, value) {
        var curVal = ele.get("checked");
        ele.set(CHECKED, value);
        Y.Global.fire(MUSICCHECKBOXCHANGED, { input: ele, prevVal: curVal, newVal: value });
    },
    clearAllCheckboxes: function() {
        if (this.trackedSongs) {
            this.trackedSongs = new Y.Array([]);
        }
        var selectAll = this.get("selectAll");
        if(selectAll) {
            selectAll.set(CHECKED, false);
        }
        this.get(SELECTEDSONGS).each(function(ele) {
            this.setCheckBox(ele, false);
        }, this);
        Y.Global.fire(MUSICCHECKBOXESUPDATED);
    },
    resetSelectAll: function() {
        var selectAll = this.get('selectAll');
        if (selectAll) {
            selectAll.set(CHECKED, false);
        }
    },
    afterSelectAllClick: function(e) {
        var value = e.target.get(CHECKED);
        this.get(BOUNDINGBOX).all('.song-listing input').each(function(ele) {
            this.setCheckBox(ele, value);
            if (value === true){
                ele.ancestor(TR).addClass(CHECKED);
            } else {
                ele.ancestor(TR).removeClass(CHECKED);
            }
        }, this);
        Y.Global.fire(MUSICCHECKBOXESUPDATED);
    },
    afterSongsChange: function(e) {
        var songs = e.newVal,
        playingSong = Y.Music.lastSong,
        view = this.get(HISTORY).get(VIEW),
        count = songs.length,
        tbody = this.get('table').one('.body'),
        listing = null,
        playButtonTh = Y.one(".active-songs-view th.play-button"),
        theadRow,
        escapeHtml = function(k,v){
            return Y.Escape.html(v);
        },
        noSongsMsg = tbody.one(".no-songs"),
        i,
        song,
        content,
        bbViews;

        this.get(BOUNDINGBOX).all('.song-listing').remove();

        if (count === 0) {
            if (noSongsMsg) {
                listing = noSongsMsg;
            } else {
                listing = Y.Node.create(Y.one("#tpl-no-songs").getContent());
            }
            tbody.appendChild(listing);
        } else if (count > 0 && noSongsMsg) {
            noSongsMsg.remove();
        }

        if (!playButtonTh){
            playButtonTh = Y.Node.create(Y.one("#tpl-play-button-th").getContent());
            theadRow = Y.one(".active-songs-view thead tr");
            if (theadRow) {
                theadRow.prepend(playButtonTh);
            }
        }

        if (view === "playlist" && this.get("isShufflePlaylist") === true) {
            view = "shuffle";
            this.get(BOUNDINGBOX).addClass("shuffled");
        } else {
            this.get(BOUNDINGBOX).removeClass("shuffled");
        }

        for (i=0; i<count; i++) {
            song = songs[i],
            content = Y.mix(song, {index: i});

            if (view === 'search') {
                view = 'songs';
            }
            listing = Y.Node.create(Y.substitute(Y.one('#'+view+'-song-listing').getContent(), song, escapeHtml));
            // Stash the actual trackid + index on the DOM node using HTML5 custom data attrs.
            listing.setAttribute("data-u1m-songid", song.id);
            // If we are on the now playing view we need to track the index of the track
            // to compare with the the now playing index.
            if (view === NOWPLAYING) {
                // The concatanation is to co-erce data to a string for IE's benefit.
                listing.setAttribute("data-u1m-npindex", EMPTY+i);
                // If view !== nowplaying then we currently don't show duplicate tracks.
                // Should this change then extra tracking data will be required.
            } else if (playingSong !== null && song.id === playingSong.sID) {
                listing.addClass(CURRENT);
            }
            tbody.appendChild(listing);
        }
        // Now fire the event to update the selected track for the nowplaying view
        // using the data + the currently playing track index.
        Y.Global.fire(MUSICUPDATESELECTED, { song: playingSong });
        // Fire event to update the col playbutton.
        Y.Global.fire(MUSICUPDATECOLPLAYBUTTON);
        // This forces a redraw in IE8.
        if (Y.UA.ie === 8) {
            bbViews = Y.one(".views");
            if (bbViews) {
                bbViews.toggleClass("redraw");
            }
        }
    },
    afterVisibleChangeSongView: function() {
        var playingSong = Y.Music.lastSong;
        Y.Global.fire(MUSICUPDATESELECTED, { song: playingSong });
    },
    beforeVisibleChangeSongView: function(e) {
        if (e.newVal === false ) {
            this.resetSelectAll();
            this.get(BOUNDINGBOX).all('.song-listing').remove();
        } else {
            Y.all(DOTACTIVESONGSVIEW).removeClass("active-songs-view");
            this.get(BOUNDINGBOX).addClass("active-songs-view");
            Y.Global.fire(MUSICCHECKBOXESUPDATED);
        }
    },
    afterViewChange: function(e) {
        if (e.newVal === this.get('viewName')) {
            if (this.get(VISIBLE) === false) {
                this.show();
            }
        } else {
            this.hide();
        }
    },
    onCheckboxesUpdated: function() {
        var subMenu = this.get(BOUNDINGBOX).one(".submenu"),
        numChecked = this.get(SELECTEDSONGS).size(),
        nSongsTemplate = Y.one("#tpl-n-songs").getContent(),
        templVars = {"songs": "song", "num": 1 },
        numCheckedNodeList = Y.all(".n-songs"),
        tmp;


        // Improve checking size to better take into account the my-songs view which
        // doesn't always have rendered checked boxes.
        if (numChecked === 0 && this.trackedSongs && this.trackedSongs.length > 0) {
            numChecked = this.trackedSongs.length;
        }
        if (numChecked > 0) {
            subMenu.all('.chk[disabled]').each( function(ele){
                ele.removeClass(DISABLED);
                ele.removeAttribute(DISABLED);
            });
            numCheckedNodeList.each( function(ele) {
                if (numChecked === 1) {
                    tmp = Y.substitute(nSongsTemplate, templVars); // safe
                } else {
                    templVars.songs = "songs";
                    templVars.num = numChecked;
                    tmp = Y.substitute(nSongsTemplate, templVars); // safe
                }
                ele.setContent(tmp);

            });
        } else {
            subMenu.all('.chk:not([disabled=disabled])').each( function(ele){
                ele.addClass(DISABLED);
                ele.set(DISABLED, DISABLED);
            });
            numCheckedNodeList.each( function(ele) {
                templVars.songs = "songs";
                templVars.num = "";
                tmp = Y.substitute(nSongsTemplate, templVars); // safe
                ele.setContent(tmp);
            });
        }
    },
    onPlaylistCreate: function() {
        Y.Global.fire('music:playlistsStale');
    },
    getSelectedTracks: function(){
        var songs = this.get(SONGS),
        songsLength = songs.length,
        selected = [],
        selectedSongs = this.get(SELECTEDSONGS),
        i, j;

        if (selectedSongs.size() === 0 && this.trackedSongs.length === 0) {
            return [];
        }
        selectedSongs.each(function(ele) {
            var id = ele.get(VALUE),
            i;
            // For when songs is populated
            for (i=0; i<songsLength; i++) {
                if (songs[i].id === id) {
                    selected.push(songs[i]);
                }
            }
        });
        // For newer views using the dynamic table widget we use the songs that are tracked
        // as checkboxes are checked and unchecked.
        if (this.trackedSongs && this.trackedSongs.length > 0) {
            selected = [];
            for (i=0, j=this.trackedSongs.length; i<j; i++) {
                selected.push(this.trackedSongs[i]);
            }
        }
        return selected;
    },
    onAddSongsClick: function(e) {
        e.preventDefault();
        var selected = this.getSelectedTracks();
        if (selected.length === 0) {
            return;
        }
        Y.Global.fire(MUSICADDSONGSTOPLAYLIST, { songs: selected });
        this.clearAllCheckboxes();
    },
    onAddSongsToNewClick: function(e) {
        e.preventDefault();
        var songs = [],
        selectedSongs = this.get(SELECTEDSONGS),
        createFn,
        addFn,
        dialog;

        if (selectedSongs.size() === 0 && this.trackedSongs.length === 0) {
            return;
        }

        createFn = function(name) {
            if (name === null || name === EMPTY) {
                return;
            }
            this.get(SELECTEDSONGS).each(function(ele) {
                songs.push(ele.get(VALUE));
            });
            if (songs.length === 0) { return; }
            this.get(DATASOURCE).createPlaylist(
                name, songs, Y.bind(this.onPlaylistCreate, this));
                this.clearAllCheckboxes();
        },
        addFn = function(playlistId) {
            var datasource = this.get(DATASOURCE),
            self = this,
            getCallback = function(e) {
                var newSongList = [],
                oldSongs = e.response.json.playlist.entry,
                oldSongsLength = oldSongs.length,
                i;

                for (i=0; i<oldSongsLength; i++) {
                    newSongList.push(oldSongs[i].id);
                }
                selectedSongs.each(function(ele) {
                    newSongList.push(ele.get(VALUE));
                });
                datasource.updatePlaylist(
                    playlistId, newSongList, function() {});
                    self.clearAllCheckboxes();
            };
            datasource.getPlaylist(playlistId, getCallback);
        },
        dialog = new Y.Music.PlaylistDialog({
            createFn: Y.bind(createFn, this),
            addFn: Y.bind(addFn, this),
            focusNode: e.target,
            visible: true
        });
        dialog.render();
    },
    onPlaySongsClick: function(e) {
        e.preventDefault();
        var selected = this.getSelectedTracks();
        if (selected.length === 0) {
            selected = Y.clone(this.get(SONGS), true);
        }
        Y.Global.fire(MUSICSETPLAYLIST, { playlist: selected });
        Y.Global.fire(EVENTPLAY);
        this.clearAllCheckboxes();
    }
}, {
    ATTRS: {
        datasource: { value: null },
        datasourceV2: { value: null },
        history: { value: null },
        isShufflePlaylist: { value: false },
        selectAll: { value: null },
        selectedSongs: {
            getter: function() {
                return Y.all('tbody input:checked');
            }
        },
        songs: { value: [] },
        submenu: { value: null },
        table: { value: null },
        unselectedSongs: {
            getter: function() {
                var songs = [];
                Y.all('tbody input').each(function(ele) {
                    if (!ele.get(CHECKED)) {
                        songs.push(ele);
                    }
                });
                return songs;
            }
        },
        header: { value: null },
        viewName: { value: null }
    },
    HTML_PARSER: {
        selectAll: '.selectall',
        submenu: '.submenu',
        header: '.view-header',
        table: 'table'
    }
});

namespace.NowPlayingListView = Y.Base.create('nowplayingView', SongView, [], {
    initializer: function() {
        Y.Global.after('music:nowPlaying', this.afterNowPlaying, this);
        this.render();
    },
    bindUI: function() {
        Y.bind(this.constructor.superclass.bindUI, this)();

        var events = this.get('events');
        events.push(this.get('clearNowPlayingButton').on(
            CLICK, this.onClearNowPlayingClick, this));
            events.push(this.get('removeFromNowPlayingButton').on(
                CLICK, this.onRemoveFromNowPlayingClick, this));
                events.push(this.get('saveNowPlayingButton').on(
                    CLICK, this.onSaveNowPlayingClick, this));

    },
    onClearNowPlayingClick: function(e) {
        e.preventDefault();
        Y.Global.fire(MUSICSETPLAYLIST, { playlist: [] });
        this.set(SONGS, []);
        Y.Global.fire(EVENTSTOP);

        // Clear now playing album art etc
        Y.one(".player .current-song").addClass(HIDDEN);
        Y.one(".player .next-song").addClass(HIDDEN);
        Y.one(".player .previous-song").addClass(HIDDEN);
        Y.one(".player .start-here").removeClass(HIDDEN);

        // Update title.
        Y.one(".player h2").setContent("Ubuntu One Music Player");
        Y.Global.fire('music:nowPlayingButtonsUpdated');
    },
    onSaveNowPlayingClick: function(e) {
        e.preventDefault();
        var songlist = this.get(SONGS),
        songs = [],
        i,
        createFn,
        dialog;

        if (songlist.length === 0 && this.trackedSongs.length === 0) {
            return;
        }
        for (i=0; i<songlist.length; i++) {
            songs.push(songlist[i].id);
        }

        createFn = function(name) {
            if (name === null || name === EMPTY) { return; }
            this.get(DATASOURCE).createPlaylist(
                name, songs, Y.bind(this.onPlaylistCreate, this));
                this.clearAllCheckboxes();
        },
        dialog = new Y.Music.PlaylistDialog({
            addFn: null,
            createFn: Y.bind(createFn, this),
            focusNode: e.target,
            visible: true
        });
        dialog.render();
    },
    onRemoveFromNowPlayingClick: function(e) {
        e.preventDefault();
        var selected = this.get(SELECTEDSONGS),
        length = selected.size(),
        songIds = [];

        if (length === 0) {
            return;
        }
        selected.each(function(elm){
            songIds.push(elm.get(VALUE));
            elm.ancestor(TR).remove();
        });
        Y.Global.fire(MUSICREMOVEFROMNOWPLAYING, { songIds: songIds });
        Y.Global.fire(MUSICCHECKBOXESUPDATED);
    },
    afterNowPlaying: function(e) {
        this.set(SONGS, e.songs);
    }
}, {
    ATTRS: {
        clearNowPlayingButton: { value: null },
        saveNowPlayingButton: { value: null },
        removeFromNowPlayingButton: { value: null },
        viewName: { value: NOWPLAYING }
    },
    HTML_PARSER: {
        clearNowPlayingButton: '.submenu .clear',
        saveNowPlayingButton: '.submenu .save',
        removeFromNowPlayingButton: '.submenu .delete'
    }
});

namespace.PlaylistListView = Y.Base.create('playlistView', SongView, [], {
    // Track the current playListId in flight
    currentPlaylistId: null,
    initializer: function() {
        this.render();
    },
    bindUI: function() {
        Y.bind(this.constructor.superclass.bindUI, this)();

        var events = this.get('events');
        events.push(this.get(HISTORY).after(PLAYLISTCHANGE, this.afterPlaylistChange, this));

        events.push(this.get('queuePlaylistButton').on(CLICK, this.onQueuePlaylistClick, this));
        events.push(this.get('removeFromPlaylistButton').on(CLICK, this.onRemoveFromPlaylistClick, this));
        events.push(this.get('addSongsButton').on(CLICK, this.onAddSongsClick, this));
        events.push(this.get('deletePlaylistButton').on(CLICK, this.onDeletePlaylistClick, this));
    },
    getPlaylist: function(playlistId) {
        if (playlistId === "shuffle-all") {
            this.set("isShufflePlaylist", true);
            this.get(DATASOURCE).getRandomSongs(200, Y.rbind(this.onIOSuccess, this, "random", playlistId));
        } else {
            this.set("isShufflePlaylist", false);
            this.get(DATASOURCE).getPlaylist(playlistId, Y.rbind(this.onIOSuccess, this, "playlist", playlistId));
        }
    },
    afterPlaylistChange: function(e) {
        if (this.get(VISIBLE)) {
            if (e.newVal !== this.currentPlaylistId) {
                this.getPlaylist(e.newVal);
                this.currentPlaylistId = e.newVal;
            }
        }
    },
    beforeVisibleChange: function(e) {
        if (e.newVal === true) {
            var playlist = this.get(HISTORY).get(PLAYLIST);
            if (e.newVal !== this.currentPlaylistId) {
                this.currentPlaylistId = playlist;
                this.getPlaylist(playlist);
            }
        } else {
            this.set("isShufflePlaylist", false);
        }
    },
    onIOSuccess: function(e, type) {
        type = type || "playlist";
        var songs;
        if (type === "random") {
            this.get('header').setContent("Shuffle All Songs");

            if (e.response.json.randomSongs && e.response.json.randomSongs.song) {
                songs = e.response.json.randomSongs.song;
                this.set(SONGS, songs);
                this.get(BOUNDINGBOX).one(".submenu").hide();
                if (songs.length === 0) {
                    return;
                }
                Y.Global.fire(MUSICSETPLAYLIST, { playlist: songs });
                Y.Global.fire(EVENTPLAY);
            }
        } else {
            this.get('header').setContent(Y.Escape.html(e.response.json.playlist.name));
            this.set(SONGS, e.response.json.playlist.entry);
            this.get(BOUNDINGBOX).one(".submenu").show();
        }
    },
    onQueuePlaylistClick: function(e) {
        e.preventDefault();
        var songs = this.get(SONGS);
        if (songs.length === 0) { return; }
        Y.Global.fire(MUSICSETPLAYLIST, { playlist: Y.clone(songs, true) });
        Y.Global.fire(EVENTPLAY);
        this.clearAllCheckboxes();
    },
    onRemoveFromPlaylistClick: function(e) {
        e.preventDefault();
        var songs = [],
        unselectedSongs = this.get('unselectedSongs'),
        length = unselectedSongs.length,
        playlistId = this.get(HISTORY).get(PLAYLIST),
        i;

        for (i=0; i<length; i++) {
            songs.push(unselectedSongs[i].get(VALUE));
        }
        this.get(DATASOURCE).updatePlaylist(
            playlistId, songs, Y.bind(this.onPlaylistUpdate, this));
    },
    onPlaylistUpdate: function() {
        this.get(SELECTEDSONGS).each(function(ele) {
            ele.ancestor("tr").remove();
        });
        Y.Global.fire('music:playlistsStale');
    },
    onDeletePlaylistClick: function(e) {
        e.preventDefault();
        var history = this.get(HISTORY),
        id = history.get(PLAYLIST),
        datasource = this.get(DATASOURCE),
        confirmDialog = new Y.Music.YesNoDialog({
            message: 'Are you sure you want to delete this playlist?',
            yesFn: function() {
                datasource.deletePlaylist(id, function() {
                    Y.Global.fire('music:playlistsStale');
                    history.add(
                        Y.mix({ view: ARTISTS }, HISTORYRESET));
                        confirmDialog.destroy();
                });
            },
            noFn: function() { confirmDialog.destroy(); }
        });
        confirmDialog.render();
    }
}, {
    ATTRS: {
        queuePlaylistButton: { value: null },
        removeFromPlaylistButton: { value: null },
        deletePlaylistButton: { value: null },
        addSongsButton: { value: null },
        viewName: { value: PLAYLIST }
    },
    HTML_PARSER: {
        queuePlaylistButton: '.submenu .play',
        removeFromPlaylistButton: '.submenu .delete',
        deletePlaylistButton: '.delete-playlist',
        addSongsButton: '.submenu .add'
    }
});

namespace.SongListView = Y.Base.create('songView', SongView, [], {
    initializer: function() {
        this.render();
    },
    bindUI: function() {
        Y.bind(this.constructor.superclass.bindUI, this)();
        var events = this.get('events');

        events.push(this.get(HISTORY).after('searchChange', this.afterSearchChange, this));

        events.push(this.get('addSongsButton').on(CLICK, this.onAddSongsClick, this));
        events.push(this.get('addSongsToNewButton').on(CLICK, this.onAddSongsToNewClick, this));
        events.push(this.get('playSongsButton').on(CLICK, this.onPlaySongsClick, this));
    },
    syncUI: function() {
        var view = this.get(HISTORY).get(VIEW);
        if (view === SONGS || view === SEARCH) {
            this.show();
        } else {
            this.hide();
        }
    },
    afterSearchChange: function(e) {
        if (e.newVal === null ) { return; }
        if (this.get(VISIBLE)) {
            this.get(DATASOURCE).search(e.newVal, Y.bind(this.onIOSuccess, this));
        } else {
            this.show();
        }
    },
    beforeVisibleChange: function(e) {
        var history,
        term,
        album,
        view;

        if (e.newVal === true) {
            history = this.get(HISTORY),
            view = history.get(VIEW);
            if (view === SEARCH) {
                term = history.get(SEARCH);
                this.get(DATASOURCE).search(term, Y.bind(this.onIOSuccess, this));
            } else {
                album = history.get('album');
                this.get(DATASOURCE).getAlbum(
                    album, Y.bind(this.onIOSuccess, this));
            }
        } else {
            this.resetSelectAll();
            this.get(BOUNDINGBOX).all('.song-listing').remove();
        }
    },
    onIOSuccess: function(e) {
        var view = this.get(HISTORY).get(VIEW),
        songs,
        searchTerm,
        song,
        i;

        if (view === SEARCH) {
            searchTerm = this.get(HISTORY).get('search');
            this.get('header').setContent('Songs matching "' + Y.Escape.html(searchTerm) + '"');
            songs = e.response.json.searchResult.match;
        } else {
            if (e.response.json.directory && e.response.json.directory.child) {
                songs = e.response.json.directory.child;
                this.get('header').setContent(
                    Y.Escape.html(songs[0].album) + ' by ' + Y.Escape.html(songs[0].artist));
            }
        }
        for (i=0; i<songs.length; i++) {
            song = songs[i];
            if (song.track === ZERO) {
                song.track = EMPTY;
            }
        }
        this.set(SONGS, songs);
    },
    afterViewChange: function(e) {
        var view = e.newVal;
        if (view === SONGS || view === SEARCH) {
            this.show();
        } else {
            this.resetSelectAll();
            this.hide();
        }
        Y.Global.fire(MUSICCHECKBOXESUPDATED);
    }
}, {
    ATTRS: {
        addSongsButton: { value: null },
        addSongsToNewButton: { value: null },
        playSongsButton: { value: null }
    },
    HTML_PARSER: {
        addSongsButton: '.submenu .add',
        addSongsToNewButton: '.submenu .addnew',
        playSongsButton: '.submenu .play'
    }
});

namespace.MySongListView = Y.Base.create('mySongView', SongView, [], {
    initializer: function() {
        this.render();
    },
    renderUI: function() {
        this.nRowPlayingCacheIndex = null;
        var datasourceV2 = this.get("datasourceV2");
        this.tableScroll = new Y.u1.TableScroll({
            oNodeContainer: "#my-song-view .tablescroll",
            oDataSource: datasourceV2,
            nRecordLimit: 100,
            nRowHeight: 34,
            sDataSourceMethod: "getSongs",
            fRecordTotalSchema: function(o) {
                return o.response.meta.total;
            },
            fRecordListSchema: function(o) {
                return o.response.json.songs;
            },
            bDebugMode: false,
            aColumnSpec: [
                // Width is a fixed width in px.
                // flexwidth is a percentage based width based on the
                // space remaining after the px based widths have been accounted for.
                // TODO: name isn't used currently should bin it if it's superfluos.
                { "name": "play", "width": 40 },
                { "name": "chk", "width": 35 },
                { "name": "title", "flexwidth": "33.3", "ellipsis": true },
                { "name": "artist", "flexwidth": "33.3", "ellipsis": true },
                { "name": "album", "flexwidth": "33.3", "ellipsis": true }
            ],
            oRowSpec: {
                "preDataTemplate": '#song-list-item-predata',
                "postDataTemplate": '#song-list-item-postdata'
            },
            // The rowData is the 1st arg
            // The row number is the 2nd arg
            // The instance is the 3rd arg
            fCustomRowRenderer: Y.rbind(function(rowData, nRow, context){
                var lastSong = Y.Music.lastSong,
                node = Y.Node.create(Y.substitute(context.get("sRowPostDataTemplate"), rowData, function(k,v){return Y.Escape.html(v);})),
                songChecked,
                songId = node.getAttribute("data-u1m-songid");

                node.setData(SONG, rowData);
                if (lastSong && songId === lastSong.sID) {
                    this.nRowPlayingCacheIndex = nRow;
                    Y.Global.fire(MUSICUPDATECURRENTTRACKROW, {"node": node, "song": lastSong});
                }
                // Looked for checked tracks based on the data we are tracking.
                if (this.trackedSongs && this.trackedSongs.length > 0) {
                    songChecked = Y.Array.find(this.trackedSongs, function(songData) {
                        if (songData.id === this.songId) {
                            return true;
                        }
                    }, {"songId": songId});
                    if (songChecked !== null) {
                        this.setCheckBox(node.one('input[type=checkbox]'), true);
                        Y.Global.fire(MUSICCHECKBOXESUPDATED);
                    }
                }
                return node;
            }, this)
        });
        // If the current view is MYSONGS at the point of render
        // so we're on this view via history being set
        // update the scroll widget which will render if
        // it's not already rendered.
        if (this.get(HISTORY).get(VIEW) === MYSONGS) {
            this.updateScrollView();
        }
    },
    bindUI: function() {
        Y.bind(this.constructor.superclass.bindUI, this)();
        var events = this.get('events');

        events.push(this.get('addSongsButton').on(CLICK, this.onAddSongsClick, this));
        events.push(this.get('addSongsToNewButton').on(CLICK, this.onAddSongsToNewClick, this));
        events.push(this.get('playSongsButton').on(CLICK, this.onPlaySongsClick, this));

        events.push(this.after(VISIBLECHANGE, this.afterVisibleChange, this));
        events.push(this.tableScroll.after("visibleChange", this.updateScrollView, this));
    },
    updateScrollView: function() {
        if (this.tableScroll.get("rendered") === true) {
            // Dump the cache so we re-evaluate what's currently playing.
            this.tableScroll.clearDataCache();
            this.tableScroll.clearRows();
            this.tableScroll.syncUI();
        } else {
            this.tableScroll.render();
        }
    },
    afterVisibleChange: function(e) {
        if (e.newVal === true ) {
            this.tableScroll.set("visible", true);
        } else {
            this.tableScroll.set("visible", false);
        }
    },
    afterViewChange: function(e) {
        var view = e.newVal;
        if (view === MYSONGS) {
            this.show();
        } else {
            this.clearAllCheckboxes();
            this.hide();
        }
    }
}, {
    ATTRS: {
        addSongsButton: { value: null },
        addSongsToNewButton: { value: null },
        datasourceV2: { value: null },
        playSongsButton: { value: null },
        viewName: { value: MYSONGS }
    },
    HTML_PARSER: {
        addSongsButton: '.submenu .add',
        addSongsToNewButton: '.submenu .addnew',
        playSongsButton: '.submenu .play'
    }
});

namespace.PlaylistGridView = Y.Base.create('PlaylistGridView', Y.Widget, [ToggleView], {
    initializer: function() {
        this.get(HISTORY).after(VIEWCHANGE, this.afterViewChange, this);
        this.render();
    },
    bindUI: function() {
        var boundingBox = this.get('inner'),
        events = this.get('events');
        events.push(boundingBox.delegate(CLICK, this.onPlaylistClick, '.playlist-grid-listing', this));
        events.push(boundingBox.delegate(CLICK, this.onPlayAllClick, '.playlist-grid-listing button', this));
        events.push(this.get(HISTORY).after(VIEWCHANGE, this.afterViewChange, this));

        events.push(this.before(VISIBLECHANGE, this.beforeVisibleChange, this));
        events.push(this.after('playlistsChange', this.afterPlaylistsChange, this));
    },
    syncUI: function() {
        var view = this.get(HISTORY).get(VIEW);
        if (view === PLAYLISTS) {
            this.show();
        } else {
            this.get('inner').all('.playlist-listing').remove();
            this.hide();
        }
    },
    onPlaylistClick: function(e) {
        e.preventDefault();
        if (e.target.get('nodeName').toLowerCase() === 'button' || e.target.ancestor("button") !== null) {
            return;
        }
        var target = e.currentTarget,
        playlistId = target.one('a').getAttribute('data-u1m-playlistid'),
        history = this.get(HISTORY);
        history.add(Y.mix({
            playlist: playlistId,
            view: PLAYLIST
        }, HISTORYRESET));
    },
    onPlayAllClick: function(e) {
        var playlistId = e.target.ancestor(".playlist-grid-listing").one('a').getAttribute('data-u1m-playlistid'),
        datasource = this.get(DATASOURCE);
        datasource.getPlaylist(playlistId, function(e) {
            var songs = e.response.json.playlist.entry;
            Y.Global.fire(MUSICADDSONGSTOPLAYLIST, {
                songs: songs,
                'injectSong': true,
                'playNow': true
            });
        });
    },
    onPlaylistLoaded: function(e) {
        var songs = e.response.json.playlist.entry;
        if (songs.length === 0) { return; }
        Y.Global.fire(MUSICSETPLAYLIST, { playlist: songs });
        Y.Global.fire(EVENTPLAY);
    },
    onIOSuccess: function(e) {
        this.set(PLAYLISTS, e.response.json.playlists.playlist);
    },
    afterPlaylistsChange: function() {
        var playlists = this.get(PLAYLISTS),
        count = playlists.length,
        box = this.get('inner'),
        escapeHtml = function(k,v){
            return Y.Escape.html(v);
        },
        i,
        playlist,
        temp_vars,
        img_id,
        playlistNode,
        placeholder,
        link;

        box.setContent(EMPTY);
        for (i=0; i<count; i++) {
            playlist = playlists[i],
            img_id = Y.guid(),
            playlistNode = Y.Node.create(Y.substitute(Y.one("#playlist-grid-listing").getContent(), playlist, escapeHtml)),
            temp_vars = { "img_id": img_id, "default_art_url": DEFAULT_PLAYLIST_ART },
            placeholder = Y.Node.create(Y.substitute(Y.one("#tpl-art").getContent(), temp_vars, escapeHtml)),
            link = playlistNode.one('[data-u1m-playlistid="'+playlist.id+'"]');

            playlistNode.insertBefore(placeholder, link);
            box.appendChild(playlistNode);
        }
    },
    beforeVisibleChange: function(e) {
        if (e.newVal === true) {
            this.get('playlistsds').getPlaylists(Y.bind(this.onIOSuccess, this));
        }
    },
    afterViewChange: function(e) {
        if (e.newVal === PLAYLISTS) {
            this.show();
        } else {
            this.get('inner').all('.playlist-grid-listing').remove();
            this.hide();
        }
    }
}, {
    ATTRS: {
        CONTENT_TEMPLATE: { value: null },
        playlists: { value: [] },
        datasource: { value: null },
        playlistsds: { value: null },
        history: { value: null },
        header: { value: null },
        inner: { value: null },
        visible: { value: false }
    },
    HTML_PARSER: {
        header: '.view-header',
        inner: '.view-inner'
    }
});


}, '@VERSION@', {
    "requires": [
        "array-extras",
        "base-base",
        "base-pluginhost",
        "base-build",
        "escape",
        "imageloader",
        "music-widgets",
        "tablescroll",
        "unity",
        "transition",
        "widget-base",
        "widget-htmlparser",
        "widget-uievents",
        "widget-skin"
    ]
});
