YUI.add('music-widgets', function (Y, NAME) {

"use strict";
/*global GLOBAL_PLAYLIST_LIST */
var CLICK = 'click',
    ACTION = 'action',
    PARENTNODE = 'parentNode',
    OPTIONS = 'options',
    EVENTS = 'events',
    EXPANDED = 'expanded',
    LI = 'li',
    SLIDER = 'slider',
    BOUNDINGBOX = 'boundingBox',
    DIALOG = 'dialog',
    YESBUTTON = 'yesButton',
    NOBUTTON = 'noButton',
    BODYCONTENT = 'bodyContent',
    HASHADDTOPLAYLIST = '#add-to-playlist',
    VALUE = 'value',
    DIV = '<div></div>',
    exports = Y.namespace('Music');

exports.SubMenu = Y.Base.create('submenu', Y.Widget, [], {
    renderUI: function() {
        var first = this.get('first'),
        items = this.get('items'),
        action = Y.Node.create(DIV),
        options = Y.Node.create(DIV);
        action.addClass('yui3-submenu-action');
        first.insert(action, 0);
        this.set(ACTION, action);

        options.addClass('yui3-submenu-options');
        items.get(PARENTNODE).append(options);
        items.each(function(item) {
            if (item !== first) {
                item.remove();
                options.appendChild(item);
            }
        });
        this.set(OPTIONS, options);
    },
    bindUI: function() {
        this.get(ACTION).on(CLICK, this.onClick, this);
    },
    onClick: function(e) {
        e.preventDefault();
        var parentNode = this.get(OPTIONS).get(PARENTNODE),
        events = this.get(EVENTS),
        eventsLength;

        if (parentNode.hasClass(EXPANDED)) {
            parentNode.removeClass(EXPANDED);

            eventsLength = events.length;
            while (events.length) {
                events.pop().detach();
            }
        } else {
            parentNode.addClass(EXPANDED);

            events.push(Y.one('html').on(CLICK, function(e) {
                var action = this.get(ACTION);
                if (!action.contains(e.target)) {
                    action.simulate(CLICK);
                }
            }, this));
        }
        this.set(EVENTS, events);
    }
}, {
    ATTRS: {
        action: { value: null },
        events: { value: [] },
        first: { value: null },
        items: { value: [] },
        options: { value: null }
    },
    HTML_PARSER: {
        first: LI,
        items: [LI]
    }
});

exports.Slider = Y.Base.create('slideroverlay', Y.Overlay, [], {
    afterRender: function() {
        var slider = new Y.Slider({
            axis: 'y',
            value: 0,
            zIndex: 5
        });
        slider.render(this.get('contentBox'));
        slider.on('valueChange', this.get('valueChangeHandler'));
        this.set(SLIDER, slider);
    },
    renderUI: function() {
        this.get(BOUNDINGBOX).addClass('yui3-skin-sam');
    },
    bindUI: function() {
        this.after('render', this.afterRender, this);
    },
    getValue: function() {
        return this.get(SLIDER).getValue();
    }
}, {
    ATTRS: {
        height: { value: '150px' },
        valueChangeHandler: { value: null },
        slider: { value: false },
        width: { value: '20px' }
    }
});

exports.Dialog = Y.Base.create(DIALOG, Y.Widget,
                               [Y.WidgetStdMod, Y.WidgetPosition, Y.WidgetPositionAlign,
                                   Y.WidgetPositionConstrain], {
                                       destructor: function() {
                                           var focusNode = this.get('focusNode');
                                           if (focusNode) {
                                               focusNode.focus();
                                           }
                                       }
                                   }, {
                                       ATTRS: {
                                           centered: { value: true },
                                           focusNode: { value: null }
                                       }
                                   });

                                   exports.YesNoDialog = Y.Base.create('yes-no-dialog', exports.Dialog, [], {
                                       renderUI: function() {
                                           var footerContent = Y.Node.create(DIV),
                                           yesButton = Y.Node.create('<button>Yes</button>'),
                                           noButton = Y.Node.create('<button class="cancel">No</button>'),
                                           msg = this.get('message');

                                           this.set(YESBUTTON, yesButton);
                                           this.set(NOBUTTON, noButton);
                                           footerContent.appendChild(yesButton);
                                           footerContent.appendChild(noButton);

                                           if (this.get("escapeHTML") === true){
                                               msg = Y.Escape.html(msg);
                                           }

                                           this.set(BODYCONTENT, msg);
                                           this.set('footerContent', footerContent);
                                       },
                                       bindUI: function() {
                                           this.get(YESBUTTON).on(CLICK, this.get('yesFn'));
                                           this.get(NOBUTTON).on(CLICK, this.get('noFn'));
                                       }
                                   }, {
                                       ATTRS: {
                                           message: { value: null },
                                           yesButton : { value: null },
                                           yesFn: { value: function() {} },
                                           noButton: { value: null },
                                           noFn: { value: function() {} },
                                           escapeHTML: { value: true }
                                       }
                                   });

                                   exports.PlaylistDialog = Y.Base.create(DIALOG, exports.Dialog, [], {
                                       renderUI: function() {
                                           var playlistTemplate = Y.one('#playlist-select').getContent(),
                                           playlistDisplay = Y.Node.create(
                                               Y.one(HASHADDTOPLAYLIST).getContent()),
                                               selectionNode = playlistDisplay.one('.selection'),
                                               length = GLOBAL_PLAYLIST_LIST.length,
                                               escapeHtml = function(k,v){
                                                   return Y.Escape.html(v);
                                               },
                                               i,
                                               playlist,
                                               rendered;

                                               if (this.get('addFn') !== null) {
                                                   if (length > 0) {
                                                       selectionNode.setContent('');
                                                   }
                                                   for (i=0; i<length; i++) {
                                                       playlist = GLOBAL_PLAYLIST_LIST[i],
                                                       rendered = Y.Node.create(
                                                           Y.substitute(playlistTemplate, playlist, escapeHtml));
                                                           selectionNode.appendChild(rendered);
                                                   }
                                               } else {
                                                   selectionNode.get(PARENTNODE).remove();
                                                   this.set('height', '130px');
                                               }
                                               this.set(BODYCONTENT, playlistDisplay);
                                       },
                                       bindUI: function() {
                                           var createFn = this.get('createFn'),
                                           addFn = this.get('addFn'),
                                           self = this;
                                           this.get('createButton').on(CLICK, function(e) {
                                               var name = e.target.get(PARENTNODE).one('input').get(VALUE);
                                               createFn(name);
                                               self.destroy();
                                           });
                                           if (addFn !== null) {
                                               this.get('cancelButton').on(CLICK, Y.bind(this.destroy, this));
                                               this.get('addButton').on(CLICK, function(e) {
                                                   var radioButton = e.target.get(PARENTNODE).one('input:checked');
                                                   if (radioButton) {
                                                       addFn(radioButton.get(VALUE));
                                                       self.destroy();
                                                   } else {
                                                       Y.Global.fire('one:error', {message: 'No playlist selected. Please select a playlist to add to'});
                                                   }
                                               });
                                           }
                                       }
                                   }, {
                                       ATTRS: {
                                           createFn: { value: function() {} },
                                           addFn: { value: function() {} },
                                           headerContent: { value: 'Create a new playlist' },
                                           createButton: { getter: function() {
                                               return this.get(BOUNDINGBOX).one('#create-new');
                                           }},
                                           createPlaylist: { getter: function() {
                                               return this.get(BOUNDINGBOX).one('#new-playlist');
                                           }},
                                           cancelButton: { getter: function() {
                                               return this.get(BOUNDINGBOX).one('.cancel');
                                           }},
                                           addButton: { getter: function() {
                                               return this.get(BOUNDINGBOX).one(HASHADDTOPLAYLIST);
                                           }},
                                           height: { value: '420px' },
                                           width: { value: '400px' }
                                       }
                                   });


}, '@VERSION@', {
    "requires": [
        "base-base",
        "base-pluginhost",
        "base-build",
        "escape",
        "node-event-simulate",
        "overlay",
        "slider-base",
        "slider-value-range",
        "clickable-rail",
        "range-slider",
        "widget-base",
        "widget-htmlparser",
        "widget-uievents",
        "widget-skin",
        "widget-stdmod",
        "widget-position",
        "widget-stack",
        "widget-position-align",
        "widget-position-constrain"
    ]
});
