YUI.add('music', function (Y, NAME) {

"use strict";
var DOTPLAYER = '.player',
    EVREADY = 'ready',
    EVERROR = 'error',
    ONEERROR= 'oneerror',
    HASHNOWPLAYINGVIEW = '#nowplaying-view',
    HASHPLAYLISTVIEW = '#playlist-view',
    HASHSONGVIEW = '#song-view',
    HASHMYSONGVIEW = '#my-song-view',
    HASHMENU = '#menu',
    HASHARTISTVIEW = '#artist-view',
    HASHALBUMVIEW = '#album-view',
    HASHPLAYLISTGRIDVIEW = '#playlist-grid-view',
    TIMEOUT_COUNT,

    soundManagerLoading = false,
    appPromo,
    message,
    androidLink,
    iosLink,
    tmpl_vars,
    namespace = Y.namespace('Music');

// Namespaced var to store the currentSongId.
Y.Music.lastSong = null;

if (Y.UA.android > 0 || Y.UA.ios > 0) {
    appPromo = new Y.u1.AppPromo({"cookie_name": "u1m-mob-app", "cookie_path": "/music"}),
    message = "Did you know there's a Music app for your {device}? Would you like to visit the {store}?",
    androidLink = "https://one.ubuntu.com/android/music/app",
    iosLink = "https://one.ubuntu.com/iphone/music/app";

    if (Y.UA.android > 0) {

        tmpl_vars = {
            store: "Android Market",
            device: "Android",
            link: androidLink
        };

    } else if (Y.UA.ios > 0) {

        tmpl_vars = {
            store: "App Store",
            link: iosLink
        };

        if (Y.UA.iphone > 0) {
            tmpl_vars.device = "iPhone";
        } else if (Y.UA.ipad > 0) {
            tmpl_vars.device = "iPad";
        } else if (Y.UA.ipod > 0) {
            tmpl_vars.device = "iPod";
        }
    }
    message = Y.substitute(message, tmpl_vars);
    if (message && tmpl_vars.link) {
        appPromo.render(message, tmpl_vars.link);
    }
}

Y.on(EVREADY, function() {
    soundManagerLoading = true;
    var datasource = new Y.Music.DataSource(),
    datasourceV2 = new Y.Music.DataSourceV2(),
    playlistsDS = new Y.Music.PlaylistsDS(),
    history = new Y.HistoryHash();

    (new Y.Music.MenuView({
        boundingBox: HASHMENU,
        datasource: datasource,
        playlistsds: playlistsDS,
        history: history,
        srcNode: HASHMENU
    }).render());
    (new Y.Music.PlayerView({
        boundingBox: DOTPLAYER,
        datasource: datasource,
        history: history,
        srcNode: DOTPLAYER
    }).render());
    (new Y.Music.NowPlayingListView({
        boundingBox: HASHNOWPLAYINGVIEW,
        datasource: datasource,
        history: history,
        srcNode: HASHNOWPLAYINGVIEW
    }));
    (new Y.Music.PlaylistListView({
        boundingBox: HASHPLAYLISTVIEW,
        datasource: datasource,
        history: history,
        srcNode: HASHPLAYLISTVIEW
    }));
    (new Y.Music.SongListView({
        boundingBox: HASHSONGVIEW,
        datasource: datasource,
        history: history,
        srcNode: HASHSONGVIEW
    }));
    (new Y.Music.MySongListView({
        boundingBox: HASHMYSONGVIEW,
        datasource: datasource,
        datasourceV2: datasourceV2,
        history: history,
        srcNode: HASHMYSONGVIEW
    }));
    (new Y.Music.ArtistListView({
        boundingBox: HASHARTISTVIEW,
        datasource: datasource,
        history: history,
        srcNode: HASHARTISTVIEW
    }));
    (new Y.Music.AlbumListView({
        boundingBox: HASHALBUMVIEW,
        datasource: datasource,
        history: history,
        srcNode: HASHALBUMVIEW
    }));
    (new Y.Music.PlaylistGridView({
        boundingBox: HASHPLAYLISTGRIDVIEW,
        datasource: datasource,
        playlistsds: playlistsDS,
        history: history,
        srcNode: HASHPLAYLISTGRIDVIEW
    }));

    /* This is not a feature that YUI 3 has A-grade support for, because
     * it's non-standard.  It may not work in all browsers. To those
     * browsers that it doesn't work with, I say "Tough".
     */
    window.onbeforeunload = function() {
        var song = Y.Music.lastSong;
        if (song && song.playState === 1) {
            return "A song is playing. If you leave this page, the music will stop.";
        }
    };
    if (!history.get("view")) {
        history.add({ view: 'artists' });
    }
    Y.fire('loaded');

});
Y.on(EVERROR, function() {
    Y.Global.fire(ONEERROR,
                  'Unable to load player. Please try again in a few minutes.');
});

namespace.init = function() { Y.fire(EVREADY); };

TIMEOUT_COUNT = 0;
namespace.error = function() {
    /* Oddly, sometimes timeout() gets called even though everything is
     * fine
     */
    if (!soundManagerLoading) {
        if (TIMEOUT_COUNT === 3) {
            Y.Global.fire(EVERROR, {
message: 'The player is longer than usual to load. Have you enabled flash?'
});
return;
}
TIMEOUT_COUNT++;
setTimeout(namespace.error, 3000);
}
};


}, '@VERSION@', {
    "requires": [
        "base-base",
        "base-pluginhost",
        "base-build",
        "dump",
        "history-base",
        "history-hash",
        "history-hash-ie",
        "history-html5",
        "imageloader",
        "json-parse",
        "node-event-simulate",
        "node-base",
        "node-event-delegate",
        "node-pluginhost",
        "node-screen",
        "node-style",
        "app-promo",
        "music-io",
        "music-io-v2",
        "music-views",
        "selector-css3",
        "substitute",
        "widget-base",
        "widget-htmlparser",
        "widget-uievents",
        "widget-skin"
    ]
});
