YUI.add('musicstore-basket', function (Y, NAME) {

"use strict";
var PROXY_RE = /https?:\/\/cdn\.7static\.com|https?:\/\/www\.7digital\.com/i;

/* Basket class constructor */
function Basket() {
    Basket.superclass.constructor.apply(this, arguments);
}

Basket.NAME = "Basket";

Basket.ATTRS = {
    addForm: { value: ".add-basket-item" },
    allTrackButtons: { value: ".track-listing .atb" },
    allBasketButtons: { value: ".atb" },
    basketInfoUrl: { value: null },
    basketData: { value: null },
    basketItemLimit: { value: 4 },
    basketItemTotals: { value: ".basket-count"},
    basketContainerTemplateNode: { value: "#tpl-basket" },
    basketContainerTemplateString: { value: null },
    basketItemTemplateNode: { value: "#tpl-basket-item" },
    basketItemTemplateString: { value: null },
    buttonA11yTemplateNode: { value: "#tpl-button-added" },
    buttonA11yTemplateString: { value: null },
    buttonNode: { value: "button[type=submit]" },
    delegateNode: { value: "body" },
    deleteForm: { value: ".del-basket-item" },
    emptyBasketTemplateNode: { value: null },
    emptyBasketTemplateString: { value: null },
    inProgressClass: { value: "progress" },
    renderOnLoad: { value: true },
    releaseButtonClass: { value: "buy-all" },
    releaseButton: { value: ".release-detail .buy-all" },
    totalItemsTemplateNode: { value: "#tpl-basket-total" },
    totalItemsTemplateString: { value: null },
    viewAllItemsTemplateNode: { value: "#tpl-basket-view-all" },
    viewAllItemsTemplateString: { value: null },
    oneError: { value: "one:error" }

};

Y.extend(Basket, Y.Widget, {

    addedButtons: null,

    initializer: function() {
        var viewAllItemsTemplateNode = Y.one(this.get("viewAllItemsTemplateNode")),
            emptyBasketTemplateNode = Y.one(this.get("emptyBasketTemplateNode"));
        if (viewAllItemsTemplateNode) {
            // This one is optional.
            this.set("viewAllItemsTemplateString", viewAllItemsTemplateNode.getContent());
        }
        if (emptyBasketTemplateNode) {
            // This one is optional.
            this.set("emptyBasketTemplateString", emptyBasketTemplateNode.getContent());
        }

        this.set("basketItemTemplateString", Y.one(this.get("basketItemTemplateNode")).getContent());
        this.set("totalItemsTemplateString", Y.one(this.get("totalItemsTemplateNode")).getContent());
        this.set("basketContainerTemplateString", Y.one(this.get("basketContainerTemplateNode")).getContent());
        this.set("buttonA11yTemplateString", Y.one(this.get("buttonA11yTemplateNode")).getContent());
    },

    destructor: function() {
        this._add.detach();
        this._delete.detach();
        this.set("basketData", null);
    },

    renderUI : function() {
        // Initial rendering needs to fetch data and render the widget.
        if (this.get("renderOnLoad") === true) {
            this.getBasket();
        }
    },

    bindUI : function() {
        var delegate = this.get("delegateNode");

        // Ah dear old IE < 9 doesn't bubble submits. So for this we have to delegate the click and then
        // work out the form from the button. There's no visible inputs in these forms so we don't need
        // to watch for enter on other form elements presently.
        if (Y.UA.ie && Y.UA.ie < 9){
            // This is fired with enter too on the focussed button element so keyboard access is not a problem.
            this._add = Y.delegate("click", this.handleAdd, delegate, this.get("addForm") + " .atb:enabled", this);
            this._delete = Y.delegate("click", this.handleDeletion, delegate, this.get("deleteForm") + " .remove-basket-item:enabled", this);
        } else {
            this._add = Y.delegate("submit", this.handleAdd, delegate, this.get("addForm"), this);
            this._delete = Y.delegate("submit", this.handleDeletion, delegate, this.get("deleteForm"), this);
        }
        this.after("basketDataChange", this.syncUI, this);
    },

    syncUI: function() {
        var basket = this.get("basketData"),
            basketItems,
            basketLimit,
            basketItemsLength = 0,
            emptyBasketTemplateString,
            itemTid,
            itemRid,
            limit,
            releasesInBasket = Y.Array([]),
            renderedBasketItems,
            i, j, k, l;

        this.addedButtons = Y.Array([]);

        if (basket && basket.payload && basket.payload.basket_items) {
            basketItems = basket.payload.basket_items;
            basketLimit = this.get("basketItemLimit");
            basketItemsLength = basketItems.length;

            renderedBasketItems = [];

            if (basketLimit === 0) {
                limit = basketItemsLength;
            } else {
                limit = (basketLimit <= basketItemsLength) ? basketLimit : basketItemsLength;
            }

            for (i=0, j=limit; i<j; i++) {
                // TODO: Consider caching these items.
                renderedBasketItems.push(this.renderBasketItem(basketItems[i]));
            }
            if (limit < basketItemsLength) {
                renderedBasketItems.push(this.renderViewAllItems(basketItemsLength));
            }

            if (basketItemsLength > 0) {
                if (basket.payload.basket_summary && basket.payload.basket_summary.due) {
                    renderedBasketItems.push(this.renderTotal(basket.payload.basket_summary.due));
                }
                Y.all(".checkout-button").removeClass("hidden");
            } else {
                emptyBasketTemplateString = this.get("emptyBasketTemplateString");
                if (emptyBasketTemplateString) {
                    renderedBasketItems.push(emptyBasketTemplateString);
                }
                Y.all(".checkout-button").addClass("hidden");
            }

            // Get all the basket items and iterate over them
            for (k=0, l=basketItemsLength; k<l; k++){

                itemRid = basketItems[k].release_id;
                itemTid = basketItems[k].track_id;

                if (itemRid) {
                    itemRid = itemRid.toString();
                }
                if (itemTid) {
                    itemTid = itemTid.toString();
                }

                // Check for having dealt with this release already.
                if (Y.Array.indexOf(releasesInBasket, itemRid) > -1) {
                    continue;
                }

                // Set-up all release related buttons.
                if (itemRid && !itemTid) {
                    releasesInBasket.push(itemRid);
                    Y.all(this.get("allBasketButtons") + "[data-u1ms-id^='"+itemRid+"']").each(this.setButtonAdded, this);
                }
                // Set-up all track related buttons.
                if (itemRid && itemTid) {
                    // If there's only 1 track then set the release button too.
                    // This is a special case for releases with only one track.
                    if (Y.all(this.get("allTrackButtons")).size() === 1){
                        Y.all([this.get("allBasketButtons") + "[data-u1ms-id='"+itemRid+":"+itemTid+"']", this.get("allBasketButtons") + "[data-u1ms-id^='"+itemRid+"']"]).each(this.setButtonAdded, this);
                    } else {
                        Y.all(this.get("allBasketButtons") + "[data-u1ms-id='"+itemRid+":"+itemTid+"']").each(this.setButtonAdded, this);
                    }
                }
            }

            // Remove all the buttons not in this.addedButtons
            Y.all(this.get("allBasketButtons")).each(function(button){
                if (Y.Array.indexOf(this.addedButtons, button) === -1) {
                    this.setButtonRemoved(button);
                }
            }, this);

            this.renderBasket(renderedBasketItems);
            this.updateBasketTotals(basketItemsLength);
        }
    },

    updateBasketTotals: function(total) {
        Y.all(this.get("basketItemTotals")).each(function(o) {
            o.setContent(total);
        });
    },

    renderViewAllItems: function(num) {
        return Y.substitute(this.get("viewAllItemsTemplateString"), {"num": Y.Escape.html(num) });
    },

    renderTotal: function(total) {
        return Y.substitute(this.get("totalItemsTemplateString"), {"total": Y.Escape.html(total) });
    },

    renderBasket: function(items)  {
        var srcNode = this.get("srcNode"),
            basketContainer = this.get("basketContainerTemplateString"),
            basket,
            tfoot;
        if (srcNode && items && basketContainer){
            tfoot = (items.length > 1) ? items.pop() : "";

            // Y.Escape.html on values in this substitue will break rendering
            basket = Y.substitute(basketContainer, {"contents": items.join(""), "tfoot": tfoot});

            srcNode.setContent(basket);
        }
    },

    renderBasketItem: function(data) {
        var basketItemTemplate,
        re = PROXY_RE;
        if (data) {

            if (data.image && re.test(data.image)) {
                data.image = data.image.replace(re, U1MS_ASSET_PROXY_PREFIX );
            }

            if (data.image.substring(0, 4) !== "http"){
                data.image = MEDIA_URL + "img/music/default_album350x350.png";
            }

            basketItemTemplate = this.get("basketItemTemplateString");
            return Y.substitute(basketItemTemplate, data, function(k,v){return Y.Escape.html(v);});
        }
    },

    handleDeletion: function(e) {
        e.preventDefault();
        var currentTarget = e.currentTarget,
            uri;
        if (Y.UA.ie && Y.UA.ie < 9 && currentTarget.get("nodeName").toLowerCase() === "button") {
            currentTarget = currentTarget.ancestor("form");
        }
        if (currentTarget) {
            uri = currentTarget.getAttribute("data-u1ms-delete-url");
            if (!uri) {
                uri = currentTarget.get("action");
            }
            this.sendForm(currentTarget, uri);
        }
    },

    handleAdd: function(e) {
        e.halt();
        var currentTarget = e.currentTarget, uri;
        if (Y.UA.ie && Y.UA.ie < 9 && currentTarget.get("nodeName").toLowerCase() === "button") {
            currentTarget = currentTarget.ancestor("form");
        }
        if (currentTarget) {
            uri = currentTarget.getAttribute("data-u1ms-add-url");
            this.sendForm(currentTarget, uri);
        }
    },

    getButton: function(formNode) {
        if (formNode) {
            return formNode.one(this.get("buttonNode"));
        }
    },

    handleProgress: function(id, o) {
        var button;
        if (o && o.formNode) {
             button = this.getButton(o.formNode);
             if (button){
                button.addClass(this.get("inProgressClass"));
                button.setAttribute("disabled", "disabled");
             }
        }
    },

    handleBasketError: function(id, o, e) {
        // o is populated with status and statusText only http://yuilibrary.com/yui/docs/io/#events
        var button,
            errorMessage = "Communication with the Ubuntu One servers failed. Please try again in a few moments.";

        if (o && e.formNode) {
            button = this.getButton(e.formNode);
            button.removeClass(this.get("inProgressClass"));
            button.removeClass(this.get("addClass"));
            button.removeAttribute("disabled");
        }

        Y.Global.fire(this.get("oneError"), { message : errorMessage });
    },

    setButtonAdded: function(buttonNode){
        var inner = buttonNode.one(".inner");
        buttonNode.addClass("added");
        buttonNode.removeClass(this.get("inProgressClass"));
        buttonNode.setAttribute("disabled", "disabled");
        if (inner) {
            inner.appendChild(Y.Node.create(this.get("buttonA11yTemplateString")));
        }
        this.addedButtons.push(buttonNode);
    },

    setButtonRemoved: function(buttonNode) {
        var innerA11yMsg = buttonNode.one(".inner .acchide");
        buttonNode.removeClass("added");
        if (innerA11yMsg) {
            innerA11yMsg.remove();
        }
        buttonNode.removeAttribute("disabled");
    },

    handleBasketData: function(id, o){
        var action,
            basketChanges,
            data = o.responseText,
            itemId;

        try {
            data = Y.JSON.parse(data);
        } catch (e) {
        }

        if (data && data.payload && data.payload.basket_changes){
            basketChanges = data.payload.basket_changes;
            if (basketChanges) {
                itemId = basketChanges.item_id;
                action = basketChanges.change;
            }
        }

        if (data) {
            this.set("basketData", data);
        }

    },


    getBasket: function(io, complete){
        // Simply get the basket data.
        io = io || new Y.IO();
        complete = complete || this.handleBasketData;

        var uri = this.get("basketInfoUrl");

        io.send(uri, {
            method: 'GET',
            on: {
                success: Y.rbind(this.handleBasketData, this, {}),
                failure: Y.rbind(this.handleBasketError, this, {})
            }
        });
    },

    sendForm: function(formNode, uri, io, complete) {
        // Allow dep injection so we can mock out IO
        // for testing.
        io = io || new Y.IO();
        complete = complete || this.handleBasketData;
        var cfg = {
                method: 'POST',
                form: {
                    id: formNode,
                    useDisabled: false
                },
                on: {
                    start: Y.rbind(this.handleProgress, this, {"formNode": formNode}),
                    failure: Y.rbind(this.handleBasketError, this, {"formNode": formNode}),
                    success: Y.rbind(complete, this, {"formNode": formNode})
                }
            },
            request;
        request = io.send(uri, cfg);
    }

});

Y.namespace("U1MS").Basket = Basket;


}, '@VERSION@', {"requires": ["escape", "io-form", "json-parse", "substitute", "widget", "selector-css3"]});
