YUI.add('photos-album-actions-view', function (Y, NAME) {

"use strict";
var SORTNEWEST = 'last_modified-desc',
    SORTOLDEST = 'last_modified-asc',
    SORTNAME = 'name-asc',

    AlbumActionsView;

AlbumActionsView = Y.Base.create('albumActionsView', Y.View, [], {

    template: Y.one('#album-actions-template').getContent(),

    publicAccessTemplate: Y.one('#public-access').getContent(),

    events: {
        '.actions-links li a': {
            click: 'onShareClickHandler'
        }
    },

    initializer: function () {

        this.photoList = this.get('photoList');

    },

    render: function () {

        var container = this.get('container'),
            accessNode = Y.one('.public-info'),
            tt,
            currentSorting = this.photoList.get('photo_order'),
            link;

        // remove select and accessNode (empty destroys handlers as well)
        // not using empty cause recursive purge fails
        // TODO: investigate this #1086882
        container.get('childNodes').remove().destroy();

        // insert full template, we'll remove irrelevant nodes below
        container.setContent(this.template);

        // remove it regardless, we should always be given publicUrl if it exists
        if(accessNode) {
            accessNode.remove();
        }

        if (this.photoList.get('public_url')) {
            // create a new accessnode and insert it into parent
            accessNode = Y.Node.create(Y.substitute(this.publicAccessTemplate, {'public_url': this.photoList.get('public_url')}, function(k,v){return Y.Escape.html(v);}));
            Y.one('.last-updated').insert(accessNode, 'after');

            // rename action
            link = container.one('.option-share-link');
            if (link) {
                link.set('text', 'Share link to album');
            }
        } else {
            link = container.one('.option-disable-link');
            if (link) {
                link.remove();
            }
        }

        // remove the sorting option we're currently using
        container.one('option[value='+currentSorting+']').setAttribute('disabled', true);

        // can't use delegation because change does not bubble in IE
        container.one('select').on('change', this.onActionsChange, this);

        tt = new Y.U1.Widget.Tooltip({
            triggerNodes:".tooltip",
            delegate: ".actions-links",
            shim:false,
            zIndex:100,
            compass: 'north'
        });
        tt.render();

        // remove slideshow for small galleries
        if(this.photoList.get('total') < 2) {
            this.removeSlideshowControls();
        }
    },

    onActionsChange: function(e) {
        var value = e.target.get('value'),
            ShareUtil,
            conf;

        switch (value) {
            case 'public':
                this.fire('share:default');
                break;
            case 'private':
                this.fire('onFolderUnPublish');
                break;
            case SORTNEWEST:
            case SORTOLDEST:
            case SORTNAME:
                this.fire('sortPhotosList', {
                    newVal: value
                });
                break;

            case 'twitter':
            case 'facebook':
                ShareUtil = new Y.U1.util.Share(null, null),
                conf = ShareUtil.getConfString();

                Y.config.win.open("", value, conf);
                this.fire('share:' + value);
                e.halt();

                break;

            case 'slideshow':
                this.fire('slideshowstart');
                break;
            default:
                //Y.Global.fire(ONEERROR, {msg: 'Unknown error action'});
                break;
        }

        e.target.one('option').set('selected', true);
    },


    // we don't want slideshow controls on photoLists with only 1 photo
    removeSlideshowControls: function() {
        var slide = this.get('container').one('optgroup[label=Slideshow]');

        // public galleries don't have slideshow controls
        if (slide) {
            slide.remove();
        }
    },

    onShareClickHandler: function(e) {

        if (e.target.hasClass('photos-icon-link-white')) {
            this.fire('share:default');
            e.halt();
        } else {

            var ShareUtil = new Y.U1.util.Share(null, null),
                conf = ShareUtil.getConfString();

                if (e.target.hasClass('photos-icon-twitter-white')) {
                    Y.config.win.open("", "twitter", conf);
                    this.fire('share:twitter');
                    e.halt();
                }

                if(e.target.hasClass('photos-icon-facebook-white')) {
                    Y.config.win.open("", "facebook", conf);
                    this.fire('share:facebook');
                    e.halt();
                }
        }

        return false;
    }




}, {
    ATTRS: {
        container: {
            value: '.actions'
        }
    }
});

Y.namespace('Photos').AlbumActionsView = AlbumActionsView;


}, '@VERSION@', {"requires": ["view", "widget-tooltip"]});
