YUI.add('photos-album-app', function (Y, NAME) {

"use strict";
var AlbumApp = Y.Base.create('albumApp', Y.App, [Y.Photos.PhotosCommon], {
    views: {
        albumAppView: {
            type: Y.Photos.AlbumAppView
        }
    },

    currentQueries: {},

    initializer: function (config) {
        this.set('photoList', config.photoList);

        var photoList = this.get('photoList');
        photoList.addTarget(this);
        photoList._originalPagesize = photoList.get('pagesize');

        this.render().showView('albumAppView', {
            photoList: photoList
        }, {
            render: false
        });

        this.once('ready', function (e) {

            // if ie comes with a query string, dispatch
            if(!e.dispatched && !Y.HistoryBase.html5 && this.getQueryString()) {
                this.dispatch();
            }
            this.get('activeView').actionsView.render();
        });

        // when the actions dropdown changes sorting
        this.after('albumActionsView:sortPhotosList', this.changePhotosListSort);

        // publish/unpublish
        this.on('*:onFolderPublish', this.onFolderPublish);
        this.on('*:onFolderUnPublish', this.onFolderUnPublish);

        this.after('photoList:public_urlChange', this.afterPublicUrlChange);
        this.after('photoList:reset', this.afterAlbumReset);

        // view share events
        this.on('share:default', this.onFolderPublish);
        this.on('share:twitter', this.onShareSocial);
        this.on('share:facebook', this.onShareSocial);

        // start slideshow
        this.on('*:slideshowstart', this.onSlideshow);

        this.set('albumURL', Y.config.win.location.href.replace(/\#.+/, ''));
    },

    /**
     * onShareSocial - Handler for user initiated sharing of resource
     * @method onShareSocial
     * @param {Event} e custom event containing type of share
     */
    onShareSocial: function(e) {
        var path = this.get('photoList').get('path'),
            type = e.type || false;

        Y.Global.fire('one:busy');

        this.get('photoList').setPublic(path, Y.bind(this.onShareSocialSuccess, this, type));
    },

    /**
     * onShareSocialSuccess callback when share url created
     * @param {String} type twitter or facebook?
     * @param {String} id id
     * @param {Object} res the response xmlhttpreq
     */
    onShareSocialSuccess: function(type, id, res) {
        var popup,
            publicUrl = res.public_url;

        type = type.replace('share:', '');

        this.get('photoList').set('public_url', publicUrl);

        popup = new Y.U1.util.Share(publicUrl, null);
        popup.setUrl(type).open(null, null, type);
        Y.Global.fire('one:idle');

    },

    showShareLinkModal: function() {
        if (this.shareLinkWidget) {
            this.shareLinkWidget.show();
        } else {
            this.createShareLinkModal();
        }

        // update the url
        // make it easier for the user to copy by pre selecting link
        this.shareLinkWidget.getStdModNode(Y.WidgetStdMod.BODY, true).one('input').setAttribute('value', this.get('photoList').get('public_url')).select();
    },

    createShareLinkModal: function() {

        // the shared link for an album doesn't actually change, so it's
        // baked in to the widget
        this.shareLinkWidget = new Y.Panel({
            headerContent: '<h3>Share link to album</h3>',
            bodyContent: '<div><p>Copy the link to share this album:<input class="u1-input-share"></input></div>',
            width: 400,
            centered: true,
            render: true,
            visible: true,
            modal: true,
            zIndex: 10000,
            buttons: {
                header: ['close'],
                footer: [{
                    classNames: 'cta',
                    value: "Done",
                    action: function(e) {
                        e.preventDefault();
                        this.hide();
                    }
                },{
                    classNames: ['cta', 'secondary'],
                    value: "Disable Link",
                    action: function(e) {
                        e.preventDefault();
                        this.fire('onFolderUnPublish');
                        this.hide();
                    }
                }
                ]
            },
            plugins: [
                Y.U1Plugin.PanelAnim
            ]
        });
        this.shareLinkWidget.addTarget(this);
    },

    onFolderPublish: function () {

        var photoList = this.get('photoList'),
            path = photoList.get('path'),
            publicUrl = photoList.get('public_url');

        if(publicUrl) {
            this.showShareLinkModal();
        } else {
            Y.Global.fire('one:busy');

            photoList.setPublic(path, Y.bind(this.onFolderPublishSuccess, this));
        }



    },

    onFolderPublishSuccess: function (id, res) {

        this.get('photoList').set('public_url', res.public_url);

        Y.Global.fire('one:idle');


    },

    onFolderUnPublish: function () {
        var path = this.get('photoList').get('path');

        Y.Global.fire('one:busy');

        this.get('photoList').unsetPublic(path, Y.bind(this.onFolderUnPublishSuccess, this));

    },

    onFolderUnPublishSuccess: function () {
        this.get('photoList').set('public_url', null);

        Y.Global.fire('one:idle');
    },

    afterPublicUrlChange: function () {
        var url = this.get('photoList').get('public_url');

        if(url) {
            this.showShareLinkModal();
        }
        this.get('activeView').actionsView.render(url);
    },

    afterAlbumReset: function () {
        this.get('activeView').render(this);
    },

    onSlideshow: function () {
        var photo = Y.one(".photo-list .photo a"),
            url;

        if (photo) {
            url = photo.get("href");

            // remove any query string argument
            if(/\?/.test(url)) {
                url = url.replace(/\?.+/, '');
            }
            url = url + '?' + this.getQueryString();

            this.navigate(url + '&slideshow=1');
        }

    },

    changePhotosListSort: function (e) {
        this.updateQueryParameterAndNavigate(this.getCleanedQueryObject(), 'photo_order', e.newVal);
    },

    handlePhotoList: function (req, res) {

        Y.Global.fire('one:busy');

        var pagesize, total,
            photoList = res.photoList = this.get('photoList');

        this.set('albumURL', Y.config.win.location.href);

        pagesize = photoList.get('pagesize');
        total = photoList.get('total');

        // if gallery modified the model, let's use original pagesize
        if(photoList._galleryModified && photoList._originalPagesize !== pagesize) {

            // restore original value
            photoList.set('pagesize', photoList._originalPagesize);

            // clear any models
            photoList.reset({silent: true});

            // ie might encounter problems when resetting
            // forcefully remove all _items
            if(photoList._items.length > 0) {
                photoList._items = [];
            }
        }

        // update photoList attributes with relevant from req.query
        photoList.setAttrs(this.cleanQueryObjectParameters(req.query));

        // if showing less photoList than there exist in photoList, load from server
        // if no photoList loaded, load from server
        // if we have photoList and we're showing them all, just sort the photoList
        if(total > pagesize || photoList.size() === 0 ) {
            photoList.load({ silent: true }, function () {
                photoList.photoSort(null, function () { Y.Global.fire('one:idle'); });
            });
        } else if(total <= pagesize) {
            photoList.photoSort(null, function () { Y.Global.fire('one:idle'); });
        } else {
            Y.Global.fire('one:idle');
        }


    },

    showGallery: function (req) {
        var galleryView = this.get('activeView').galleryView,
            spawnedGallery = galleryView.spawnedGallery,
            galleryConfig,
            photoList = this.get('photoList');

        galleryConfig = {
            albumURLScheme: this.get('albumURLScheme'),
            galleryURLScheme: this.get('galleryURLScheme'),
            albumURL: this.get('albumURL'),
            photoList: photoList
        };

        if(!spawnedGallery) {
            galleryView.render(req.params.photo, galleryConfig);
        }
    }


}, {
    ATTRS: {

        root: {
            value: '/'
        },

        linkSelector: {
            value: '#bd a'
        },

        photoList: {
            value: null
        },

        viewContainer: {
            value: '.photos-body'
        },
        container: {
            value: '.photos-body'
        }
    }
});

Y.namespace('Photos').AlbumApp = AlbumApp;


}, '@VERSION@', {
    "requires": [
        "base",
        "app",
        "escape",
        "pages",
        "photos-common",
        "photos-io",
        "photos-gallery-app",
        "photos-album-app-view",
        "photos-photo-list-model",
        "photos-photo-model",
        "util-share",
        "querystring",
        "substitute"
    ]
});
