YUI.add('photos-common', function (Y, NAME) {

"use strict";
var PhotosCommon = Y.Base.create("photos-common", Y.Base, [], {


    /**
    List of queries we want to keep
    @property queryParameters
    @type Array
    **/

    queryParameters: [
        'folder_order',
        'folder_filter',
        'path',
        'photo_order',
        'page',
        'pagesize',
        'slideshow',
        'pagesize'
    ],

    getQueryString: function () {
        var query;

        if(Y.HistoryBase.html5) {
            query = this.getHTML5QueryString();
        } else {
            query = this.getHashQueryString();
        }
        return query;
    },

    getHTML5QueryString: function () {
        return Y.config.win.location.search.replace(/\?/, '');
    },

    getHashQueryString: function () {
        return Y.HistoryHash.getHash().replace(/.*\?/, '');
    },

    getQueryStringAsObject: function (q) {
        var query;

        // if we get a string, use it event if it's empty
        if(typeof q === 'string') {
            query = q;
        } else {
            query = this.getQueryString();
        }

        return Y.QueryString.parse(query) || {};
    },

    getCleanedQueryObject: function () {
        var query = this.getQueryStringAsObject(),
            cleanQuery;

        cleanQuery = this.cleanQueryObjectParameters(query);

        return cleanQuery;
    },

    updateQueryObjectParameter: function (queryObject, parameter, value) {
        if(!parameter) {
            throw new Error('updateQueryObjectParameter: Cannot update queryObject with falsy parameter');
        }
        queryObject[parameter] = value;
        return queryObject;
    },

    updateQueryObjectParameters: function (queryObject, parameters) {
        return Y.merge(queryObject, parameters);
    },

    updateQueryParameterAndNavigate: function (queryObject, parameter, value) {
        var modifiedQueryObject,
            path;

        if(typeof parameter === 'string') {
            modifiedQueryObject = this.updateQueryObjectParameter(queryObject, parameter, value);
        } else {
            modifiedQueryObject = this.updateQueryObjectParameters(queryObject, parameter);
        }

        path = this.getPath() + '?' + Y.QueryString.stringify(modifiedQueryObject);

        this.navigate(path);
    },


    /**
    Poorly named method, but clean means to remove query paramaters that we do not want/need
    @param {Object} queryObject the object to be cleansed
    @return {Object} cleanQueryObject a object that has had keys removed by comparing queryObject with this.queryParameters
    */
    cleanQueryObjectParameters: function (queryObject) {
        var cleanQueryObject = {};
        Y.Object.each(queryObject, function (value, key) {
            if(Y.Array.indexOf(this.queryParameters, key) !== -1) {
                cleanQueryObject[key] = value;
            }
        }, this);
        return cleanQueryObject;
    },

    defaultQueryParameters: {
        'pagesize': 102,
        'page': 1,
        'folder_order': 'last_modified-desc',
        'photo_order': 'last_modified-desc',
        'folder_filter': 'my',
        'folder_cache': true,
        'photo_cache': true
    },

    /**
    Reflecting behaviour of photos/common.py

    If we find a defaultQueryParameter here, remove it
    We also cast the given value as number if its default equivalent is a number

    @param {Object} queryObject set of keys to sanitize
    @param {String|Array} keep string or array of keys to keep
    @return {Object} sanitizedQueryObject a object with keys that are default removed
    */
    sanitizeQueryParameters: function (queryObject, keep) {
        var sanitizedQueryObject = {};

        // convert string to arr
        if(keep && typeof keep === 'string') {
            keep = [keep];
        }

        Y.Object.each(queryObject, function (value, key) {
            var defaultValue;

            // if we're ignoring key, just add it and quit
            if(keep && Y.Array.indexOf(keep, key) !== -1) {
                sanitizedQueryObject[key] = value;
                return false;
            }

            // if the default key exist and is a Number, try parseInt
            if(this.defaultQueryParameters[key]) {
                defaultValue = this.defaultQueryParameters[key];

                // if defaultValue is a number
                if(!isNaN(parseFloat(defaultValue)) && isFinite(defaultValue) ) {
                    value = parseInt(value, 10);
                }
            }

            // if not key is in default and value is not equal to default key value
            if( !(this.defaultQueryParameters[key] && this.defaultQueryParameters[key] === value) ) {
                sanitizedQueryObject[key] = value;
            }
        }, this);
        return sanitizedQueryObject;
    }


});

Y.namespace('Photos');
Y.Photos.PhotosCommon = PhotosCommon;


}, '@VERSION@', {"requires": ["base", "history", "querystring"]});
