YUI.add('photos-gallery-actions-view', function (Y, NAME) {

"use strict";
var GalleryActionsView = Y.Base.create('galleryActionsView', Y.View, [], {

    template: Y.one('#actions-template').getContent(),

    events: {
        '.share-link-url': {
            focus: function (e) {
                e.target.select();
            }
        },
        '.share-link-close': {
            click: function () {
                this.fire('shareLinkCloseClick');
            }
        },
        '.action-get-permalink': {
            click: function(e) {
                this.fire('share:default');
                e.halt();
            }
        },
        '.action-twitter a': {
            click: function(e) {
                // We open the window here and then refresh the named window with
                // the url provided by the share. To side-stop popup blockers
                // which require the window.open to be a product of the direct
                // user interaction.
                var ShareUtil = new Y.U1.util.Share(null, null),
                    conf = ShareUtil.getConfString();
                Y.config.win.open("", "twitter", conf);
                this.fire('share:twitter');
                e.halt();
            }
        },
        '.action-facebook a': {
            click: function(e) {
                // We open the window here and then refresh the named window with
                // the url provided by the share. To side-stop popup blockers
                // which require the window.open to be a product of the direct
                // user interaction.
                var ShareUtil = new Y.U1.util.Share(null, null),
                    conf = ShareUtil.getConfString();
                Y.config.win.open("", "facebook", conf);
                this.fire('share:facebook');
                e.halt();
            }
        }
    },

    initializer: function () {

        var container = this.get('container'),
            actionsHTML = Y.Node.create(this.template);

        this.tt = new Y.U1.Widget.Tooltip({
            triggerNodes: '.tooltip',
            delegate: '.photo-actions',
            shim:false,
            zIndex:100,
            compass: 'north'
        });

        container.setContent(actionsHTML);

        // listen to the actions dropdown changes
        // can't use delegation because change does not bubble in IE
        container.one('select').on('change', this.actionsChange, this);

        // cache/store variables the render method will be hiding/showing
        this.nodes = {
            linkShare: container.one('.share-link'),
            linkDisable: container.one('.disable-link'),
            linkCreate: container.one('.create-link'),

            slideshowStart: container.one('.slideshow-start'),
            slideshowStop: container.one('.slideshow-stop'),
            slideshowActions: container.one('optgroup[label=Slideshow]')
        };

        // since we now have nodes, render tooltip widget
        this.tt.render();

    },

    onCreateLinkClick: function() {
        this.fire('photoCreateLink');
    },

    onShareLinkClick: function() {
        this.fire('photoShareLink');
    },

    onDisableLinkClick: function() {
        this.fire('photoDisableLink');
    },

    actionsChange: function (e) {
        var selected = e.target.get('options').item(e.target.get('selectedIndex')),
            ShareUtil,
            conf,
            share;

        if(selected.hasClass('create-link')) {
            this.onCreateLinkClick();
        } else if (selected.hasClass('share-link')) {
            this.onShareLinkClick();
        } else if (selected.hasClass('disable-link')) {
            this.onDisableLinkClick();
        } else if (selected.hasClass('slideshow-start')) {
            this.fire('toggleSlideShow');
        } else if (selected.hasClass('slideshow-stop')) {
            this.fire('toggleSlideShow');
        } else if (selected.hasClass('gallery-close')) {
            this.fire('escape');
        } else if (selected.hasClass('option-share-twitter') || selected.hasClass('option-share-facebook')) {

            share = selected.hasClass('option-share-twitter') ? 'twitter' : 'facebook';

            ShareUtil = new Y.U1.util.Share(null, null);
            conf = ShareUtil.getConfString();
            Y.config.win.open("", share, conf);
            this.fire('share:' + share);
            e.halt();

        } else if (selected.hasClass('option-download')) {
            this.fire('download');
        }

        e.target.one('option').set('selected', true);
    },

    // the following two functions will be unnecessary immediately after we get to use Handlebars

    togglePublicActions: function (publicurl) {
        if (publicurl) {
            this.nodes.linkCreate.hide();
            this.nodes.linkShare.show();
            this.nodes.linkDisable.show();
        }
        else {
            this.nodes.linkCreate.show();
            this.nodes.linkShare.hide();
            this.nodes.linkDisable.hide();
        }
    },

    toggleSlideshowActions: function (slideshow) {
        if(slideshow) {
            this.nodes.slideshowStart.hide();
            this.nodes.slideshowStop.show();
        } else {
            this.nodes.slideshowStart.show();
            this.nodes.slideshowStop.hide();
        }
    },

    /**

    @method renderActions
    @chainable
    @return {GalleryActionsView} GalleryActionsView
    **/

    render: function () {
        // whether slideshow is playing or not
        // TODO: be provided by app through options?
        var slideshow = Y.one('.playing') !== null,
            photo = this.get('photoList').get('current_photo');

        // if published, remove publish action
        this.togglePublicActions(photo.public_url);

        // remove start/stop slideshow
        this.toggleSlideshowActions(slideshow);

        if (this.get('photoList').get('total') < 2) {
            this.nodes.slideshowActions.hide();
            //this.detach('click');
        } else {
            this.nodes.slideshowActions.show();
        }

        return this;
    }


}, {
    ATTRS: {
        container: {
            value: '.photo-actions'
        }
    }
});

Y.namespace('Photos').GalleryActionsView = GalleryActionsView;


}, '@VERSION@', {"requires": ["view", "substitute", "widget-tooltip", "util-share"]});
