YUI.add('photos-gallery-app-view', function (Y, NAME) {

"use strict";
var GalleryView;

/**
PhotosGalleryView represents the gray area and is a view composite of some smaller views and itself.

It creates all the children views needed for the gallery and subscribes to their events.

<ul>
    <li>Listens to the user's keyboard presses</li>
    <li>Listens to children views events</li>
</ul>

@class PhotosGalleryView
@extends Y.View
**/

GalleryView = Y.Base.create('galleryView', Y.View, [], {

    events: {
        '.close': {
            click: function (e) {
                this.fire('escape');
                e.halt();
            }
        }
    },

    /**

    <p>Defines `leftarrow` and `rightarrow` key event</p>

    <p>Listening to the following keys:</p>

    <ul>
        <li><kbd>←</kbd> (left arrow, `37`)</li>
        <li><kbd>→</kbd> (right arrow, `39`)</li>
        <li><kbd>esc</kbd></li>
    </ul>

    @method initializer
    **/

    initializer: function (config) {
        var view = this,
            win = Y.one(Y.config.win);

        this.photoList = config.photoList;

        // define left and right arrow event keys
        Y.Node.DOM_EVENTS.key.eventDef.KEY_MAP.leftarrow = 37;
        Y.Node.DOM_EVENTS.key.eventDef.KEY_MAP.rightarrow = 39;
        Y.Node.DOM_EVENTS.key.eventDef.KEY_MAP.space = 32;

        // on escape key press
        win.on('key', function (e) {
            e.preventDefault();
            view.fire('escape');
        }, 'esc');

        // on left key press
        win.on('key', function () {
            view.fire('left');
        }, 'leftarrow');

        // on right key press
        win.on('key', function () {
            view.fire('right');
        }, 'rightarrow');

        // on space
        win.on('key', function () {
            // is basically the same as a click on the photo
            view.fire('toggleSlideShow');
        }, 'space');

        this.on('*:photoLoad', this.onPhotoLoad);

        this.pagerView = new Y.Photos.GalleryPagerView({photoList: this.photoList});
        this.pagerView.addTarget(this);

        this.photoView = new Y.Photos.GalleryPhotoView({photoList: this.photoList});
        this.photoView.addTarget(this);


        // no actions in public lists
        if(!this.photoList.get('public_list')) {

            this.actionsView = new Y.Photos.GalleryActionsView({photoList: this.photoList});
            this.actionsView.addTarget(this);

        }

        this.metaView = new Y.Photos.GalleryMetaView({photoList: this.photoList});
        this.metaView.addTarget(this);
    },

    render: function (app) {
        this.pagerView.render(app);
        this.photoView.render(app);

        if (this.actionsView) {
            this.actionsView.render();
        }

        this.metaView.render();

    }

}, {
    ATTRS: {
        container: {
            value: '.preview'
        }
    }
});

Y.namespace('Photos').GalleryView = GalleryView;


}, '@VERSION@', {"requires": ["view"]});
