YUI.add('photos-gallery-pager-view', function (Y, NAME) {

"use strict";
/**
View representing the pager (arrows left right)

@class GalleryPagerView
@constructor
@extends View

**/

var GalleryPagerView = Y.Base.create('galleryPager', Y.View, [], {

    /**
    Templates hash for this view's container next and previous links

    @property templates
    @type Object

    **/

    templates: {
        nextTemplate: Y.one('#next-template').getContent(),
        prevTemplate: Y.one('#previous-template').getContent()
    },

    /**
    Creates missing next/prev links

    @method initializer
    **/

    initializer: function () {
        var container = this.get('container'),
            prevNode = container.one('.prev'),
            nextNode = container.one('.next');

        // if server has returned only one or no pagers, create them
        if(!prevNode) {
            prevNode = Y.Node.create(this.templates.prevTemplate).hide();
            container.append(prevNode);
        }

        if(!nextNode) {
            nextNode = Y.Node.create(this.templates.nextTemplate).hide();
            container.append(nextNode);
        }

    },

    /**
    Renders this view

    <ul>
        <li>Throws an exception if there are no photos</li>
        <li>Updates pager links</li>
        <li>Hides/shows links as they are needed</li>
    </ul>

    @method render
    @chainable
    @param {Photo} currentPhoto a photo object
    @param {Array} photoList a list of photo objects
    @return {GalleryPagerView} GalleryPagerView
    **/

    render: function (app) {

        var container = this.get('container'),
            photoList = this.get('photoList'),
            currentPhoto = photoList.get('current_photo'),
            pList = photoList.toArray(),
            index = photoList.indexOf(currentPhoto),

            prevPhoto = (index > 0) ? pList[index - 1] : null,
            nextPhoto = (index < pList.length - 1) ? pList[index + 1] : null,

            prevNode = container.one('.prev'),
            nextNode = container.one('.next'),

            loc = Y.config.win.location,
            rootURL = loc.protocol + '//' + loc.host;

        if(prevPhoto) {
            prevNode.show().one('a').setAttribute('href', rootURL + app.getPhotoURL(prevPhoto));
        } else {
            prevNode.hide();
        }

        if(nextPhoto) {
            nextNode.show().one('a').setAttribute('href', rootURL + app.getPhotoURL(nextPhoto));
        } else {
            nextNode.hide();
        }

        return this;
    }


}, {
    ATTRS: {
        container: {
            value: '.preview-pager'
        }
    }
});

Y.namespace('Photos').GalleryPagerView = GalleryPagerView;


}, '@VERSION@', {"requires": ["view", "substitute"]});
