YUI.add('photos-io', function (Y, NAME) {

    "use strict";
    var ONEERROR = 'one:error',
        exports = Y.namespace('Photos');
    exports.PhotoClient = Y.Base.create('photoClient', Y.DataSource.IO, [], {
        /* At some point, this should be extracted out. */
        _defDataFn: function(e) {
            var data = e.data,
                meta = e.meta,
                response = {
                    results: (Y.Lang.isArray(data)) ? data: [data],
                    meta: (meta) ? meta : {}
                };
            try {
                response.json = Y.JSON.parse(e.data.responseText);
            } catch(err) {
                var errorString = 'The API returned invalid JSON';
                Y.Do.Halt(errorString);
                Y.Global.fire(ONEERROR, { message: errorString });
            }
            this.fire('response', Y.mix({response: response}, e));
        },
        onError: function() { Y.Global.fire(ONEERROR); },
        getFolders: function(args, callback) {
            this.sendRequest({
                request: 'collections?' + Y.QueryString.stringify(args),
                callback: {
                    success: callback,
                    failure: this.onError
                }
            });
        },
        getFolderPath: function(args, callback) {
            var request = 'collection/by-path/';

            request = request+'?'+Y.QueryString.stringify(args);
            this.sendRequest({
                request: request,
                callback: {
                    success: callback,
                    failure: this.onError
                }
            });
        },
        getPublicAlbum: function(path, args, callback) {
            var request = 'public-collection/';

            request = request + path + '?' + Y.QueryString.stringify(args);
            this.sendRequest({
                request: request,
                callback: {
                    success: callback,
                    failure: this.onError
                }
            });
        },
        setNodekeyPrivate: function(path, callback) {
            var request = this.get('source') + 'collection/by-path/';
            /* Datasource doesn't support POST... */
            Y.io(request, {
                method: 'POST',
                data: {
                    'public': 'false',
                    'csrfmiddlewaretoken': Y.one('[name=csrfmiddlewaretoken]').get('value'),
                    'path': path
                },
                on: {
                    success: callback,
                    failure: this.onError
                }
            });
        },
        setNodekeyPublic: function(path, callback) {
            var request = this.get('source') + 'collection/by-path/';
            /* Datasource doesn't support POST... */
            Y.io(request, {
                method: 'POST',
                data: {
                    'public': 'true',
                    'csrfmiddlewaretoken': Y.one('[name=csrfmiddlewaretoken]').get('value'),
                    'path': path
                },
                on: {
                    success: callback,
                    failure: this.onError
                }
            });
        }
    }, {
        ATTRS: { source: { value: '/api/photos/v1/' }}
    });

    exports.PhotoEmitter = Y.Base.create('photoEmitter', Y.Base, [], {
        baseURL: '/api/photos/v1/',
        getFolderPath: function(path, args) {
            var url = this.baseURL + 'collection/by-path/';
            if (args) {
                url = url + '?path=' + path + '&' + Y.QueryString.stringify(args);
            }
            this._getResource(url);
        },
        getPublicCollection: function(path, args) {
            var url = this.baseURL + 'public-collection/';
            if (args) {
                url = url + path + '?' + Y.QueryString.stringify(args);
            }
            this._getResource(url);
        },
        _getResource: function(url) {
            Y.io(url, {
                on: {
                    success: Y.bind(this._onSuccess, this),
                    failure: this._onFailure
                }
            });
        },
        _onFailure: function() {
        },
        _onSuccess: function(id, res) {
            var response = res.response || res.responseText;
                response = Y.JSON.parse(response);

            this.fire('emitPhotos', {data: response});

            if (response.next_page) {
                this._getResource(response.next_page);
            } else {
                this.fire('complete');
            }
        }
    });


}, '@VERSION@', {
    "requires": [
        "base-build",
        "datasource-local",
        "datasource-io",
        "datasource-get",
        "datasource-function",
        "datasource-cache",
        "datasource-jsonschema",
        "datasource-xmlschema",
        "datasource-arrayschema",
        "datasource-textschema",
        "datasource-polling",
        "io-base",
        "io-xdr",
        "io-form",
        "io-upload-iframe",
        "io-queue",
        "querystring-parse",
        "querystring-stringify"
    ]
});
