YUI.add('photos-photo-list-model', function (Y, NAME) {

"use strict";
var PhotoList = Y.Base.create('photoList', Y.LazyModelList, [Y.Photos.Sync], {

    model    : Y.Photos.Photo,

    privateUrl: '/api/photos/v1/collection/by-path/{args}',

    publicUrl: '/api/photos/v1/public-collection/{path}/{args}',

    setPublic: function (path, callback) {
        this.sync('publish', {path: path}, callback);
    },

    unsetPublic: function (path, callback) {
        this.sync('unpublish', {path: path}, callback);
    },

    photoSort: function (options, callback) {
        var facade;

        options = options || {};

        // no reset should be fired here as we're not done sorting
        this.sort(Y.merge(options, { silent: true}));

        facade = Y.merge(options, {
            models: this._items,
            src   : 'sort'
        });

        if(!options.silent) {
            this.fire('reset', facade);
        }


        if (callback) {
            callback();
        }
    },

    buildURL: function () {
        var args = Y.QueryString.stringify({
            path: this.getAsURL('path'),
            photo_order: this.getAsURL('photo_order'),
            page: this.getAsURL('page'),
            pagesize: this.getAsURL('pagesize')
        }),
        url;

        args = '?' + args;

        if(this.get('public_list')) {
            url = Y.Lang.sub(this.publicUrl, {
                args: args,
                path: this.getAsURL('path')
            });
        } else {
            url = Y.Lang.sub(this.privateUrl, {
                args: args
            });
        }

        return url;
    },

    comparator: function (photo) {
        var order = this.get('photo_order'),
            photoName = photo.name,
            lastModified = photo.last_modified,
            compare;

        if(order === 'last_modified-desc') {
            compare = [lastModified, photoName];
        } else if(order === 'last_modified-asc') {
            compare = [-lastModified, photoName];
        } else if(order === 'name-desc' || order === 'name-asc') {
            compare = photoName;
        }

        return compare;
    },

    _compare: function (a, b) {
        // multisorting on first field, then the next
        if(Y.Lang.isArray(a)) {
            if (a[0] < b[0]) { return  1; }
            if (a[0] > b[0]) { return -1; }
            if (a[1] > b[1]) { return  1; }
            if (a[1] < b[1]) { return -1; }
            return 0;
        }

        return a < b ? -1 : (a > b ? 1 : 0);
    },

    parse: function (response) {
        if (typeof response === 'string') {
            try {
                Y.log('Response was a string', 'warn', 'Y.PhotoList.parse');
                return Y.JSON.parse(response).results.images || [];
            } catch (ex) {
                Y.log('Error parsing response', 'error', 'Y.PhotoList.parse');
                return null;
            }
        }

        this.set('cover_photo', response.results.cover_photo);
        this.set('display_name', response.results.display_name);
        this.set('last_modified', response.results.last_modified);
        this.set('owner', response.results.owner);

        if(!this.get('public_list')) {
            this.set('public_url', response.results.public_url);
        }

        return response.results.images || [];
    },

    /**

    Like a standin for getById

    @method getPhoto
    @param {String|Number} identifier. Either a name, nodekey or index
    @return {Photo} photo

    **/

    getPhoto: function (identifier) {
        var photo;

        // photos have not been fetched from server
        if(this.size() === 0) {
            return null;
        }

        if(typeof identifier === 'number') {

            // out of bounds
            if(identifier > this.size() - 1) {
                return null;
            }
            photo = this.toArray()[identifier];
        }
        else if (typeof identifier === 'string') {
            photo = this.filter(function (p) {

                // if name (decoded or not) or nodekey matches identifier, we got our match
                return (p.name === decodeURIComponent(identifier) || p.name === identifier) || (p.nodekey === identifier);

            })[0];

        }

        return photo || null;
    }


}, {
    ATTRS: {
        path: {value: null},
        photo_order: {value: null},
        folder_order: {value: null},
        folder_filter: {value: null},
        total: {value: null },
        pagesize: {value: null},
        page: { value: null},
        public_list: {value: null},
        cover_photo: {value: null},
        display_name: {value: null},
        last_modified: {value: null},
        owner: {value: null},
        public_url: {value: null},
        current_photo: {value:null}
    }
});

Y.namespace('Photos').PhotoList = PhotoList;


}, '@VERSION@', {"requires": ["lazy-model-list", "photos-sync", "querystring", "photos-photo-model"]});
