YUI.add('services', function (Y, NAME) {

"use strict";
/*global storage_pack_bytes:true, storage_monthly_price:true */
/*global storage_annual_price:true, currency: true, sub_id:true */

var CLICK,
    total_packs,
    storage_pack_gb,
    is_int,
    validate,
    update_and_validate,
    add_20_pack,
    remove_20_pack,
    submit_change_qty;

CLICK = 'click';
total_packs = Y.one('#qty');
storage_pack_gb = storage_pack_bytes / Math.pow(1024, 3);

is_int = function(value){
    if((parseFloat(value, 10) === parseInt(value, 10)) && !isNaN(value)){
        return true;
    } else {
        return false;
    }
};

validate = function(qty){
    var current_value = parseInt(qty, 10);
    if(is_int(current_value) && current_value > 0){
        return current_value;
    } else {
        return 1;
    }
};
update_and_validate = function() {
    var totalStorageNode = Y.one('#total-storage'),
    currentStorageNode = Y.one('#current-storage'),
    qty = validate(total_packs.get('value')),
    annualPriceNode =  Y.one('#storage-annual-price'),
    monthlyPriceNode = Y.one('#storage-monthly-price'),
    monthly_price,
    annual_price,
    curStorageVal;

    Y.one('#storage-gb').set('innerHTML', (storage_pack_gb*qty) + 'GB');
    monthly_price = Math.round(storage_monthly_price*qty*100, 2)/100;
    annual_price = Math.round(storage_annual_price*qty*100, 2)/100;
    if (annualPriceNode) {
        annualPriceNode.set('innerHTML', currency + annual_price);
    }
    if (monthlyPriceNode) {
        monthlyPriceNode.set('innerHTML', currency + monthly_price);
    }
    // Only used in /account/add-storage/
    if (totalStorageNode && currentStorageNode) {
        curStorageVal = parseFloat(currentStorageNode.get('innerHTML').replace("GB", ""), 10);
        totalStorageNode.set('innerHTML', curStorageVal + (storage_pack_gb*qty) + "GB");
    }
    // Set the input box to a validated value to clean garbage
    total_packs.set('value', qty);
};

add_20_pack = function(e, simple) {
    e.halt();
    var current_value = validate(total_packs.get('value'));
    total_packs.set('value', current_value+1);
    if (!simple) {
        // For some reason, on.change isn't fired
        update_and_validate();
    }
};
remove_20_pack = function(e, simple) {
    e.halt();
    var current_value = validate(total_packs.get('value'));
    if(current_value > 1){
        total_packs.set('value', current_value-1);
    } else {
        total_packs.set('value', 1);
    }
    if (!simple) {
        // For some reason, on.change isn't fired
        update_and_validate();
    }
};
submit_change_qty = function() {
    // See web/services/urls.py:subscription_qty_change
    var new_qty = validate(total_packs.get('value'));
    window.location.href = '/services/subscription/' + sub_id + '/qty_change/' + new_qty + '/';
};

window.hookup = function() {
    var qty = Y.one("#qty"),
    plus,
    minus;

    if (qty) {
        plus = Y.one('#plus_qty');
        minus = Y.one('#minus_qty');
        plus.setStyle('display', 'inline');
        plus.on(CLICK, add_20_pack);
        minus.setStyle('display', 'inline');
        minus.on(CLICK, remove_20_pack);
        Y.one('#qty').on('change', update_and_validate);
        // run immediately to update for values
        // the browser keeps in the input
        update_and_validate();
    }
};
// A more simplistic version that just updates the qty
// Used to reduce storage qty
window.hookup_simple = function() {
    var qty = Y.one("#qty"),
    plus,
    minus,
    change_btn;

    if (qty) {
        plus = Y.one('#plus_qty');
        minus = Y.one('#minus_qty');
        change_btn = Y.one('#change_qty');
        plus.setStyle('display', 'inline');
        plus.on(CLICK, add_20_pack, '', true);
        minus.setStyle('display', 'inline');
        minus.on(CLICK, remove_20_pack, '', true);
        change_btn.on(CLICK, submit_change_qty);
    }
};
// Another version, used by newplans account subscription update
window.hookup_storage_info = function() {
    var qty = Y.one("#qty"),
    plus,
    minus,
    change_btn;

    if (qty) {
        plus = Y.one('#plus_qty');
        minus = Y.one('#minus_qty');
        change_btn = Y.one('#change_qty');
        plus.setStyle('display', 'inline');
        plus.on(CLICK, add_20_pack, '', true);
        minus.setStyle('display', 'inline');
        minus.on(CLICK, remove_20_pack, '', true);
    }
};


}, '@VERSION@', {
    "requires": [
        "base-base",
        "base-pluginhost",
        "base-build",
        "event-base",
        "event-delegate",
        "event-synthetic",
        "event-mousewheel",
        "event-mouseenter",
        "event-key",
        "event-focus",
        "event-resize",
        "event-hover",
        "event-outside",
        "node-base",
        "node-event-delegate",
        "node-pluginhost",
        "node-screen",
        "node-style"
    ]
});
