YUI.add('util-oops', function (Y, NAME) {

"use strict";
Y.namespace('U1.util');

var Oops = Y.Base.create('oneUtilOops', Y.Base, [], {

    initializer: function (config) {

        config = config || {};

        var globalContext = config.globalContext || this.get('globalContext'),
            globalHandler = config.globalErrorHandler || this._globalErrorHandler;

        // save older onerror
        // replace with our handler
        this._oldOnError = globalContext.onerror;
        globalContext.onerror = Y.bind(globalHandler, this);

    },

    destructor: function () {

        // revert global error handler
        this.get('globalContext').onerror = this._oldOnError;

    },

    // overloaded to handle directly calling window.onerror(new Error("Foo"))
    _globalErrorHandler: function (message, url, line) {
        var error;

        if(Y.Lang.isObject(message)) {
            error = message;
        } else {
            error = {
                message: message,
                url: url,
                line: line
            };
        }

        // send returns supress
        return this.send(error);
    },

    _createOopsReport: function (error) {

        return new Y.U1.util.OopsReport(error, {
            ioConfig:       this.get('ioConfig'),
            endPoint:       this.get('endPoint'),
            reporter:       this.get('reporter'),
            sessionData:    this.get('sessionData')
        });

    },

    // creates OOPS report and sends it
    send: function (error) {

        var report = this._createOopsReport(error);

        if(report) {

            this.fire('registered');

            report.send();

        }

        return this.get('suppress');
    }

}, {
    ATTRS: {
        globalErrorHandler: { value: null },
        globalContext: { value: Y.config.win },
        ioConfig: {
            value: {
                method: 'POST',
                xdr: {
                    use: 'native'
                }
            }
        },
        reporter: { value: 'u1.' + Y.config.win.location.hostname },
        sessionData: { value: null },
        suppress: { value: false },
        endPoint: {
            value: '',
            writeOnce: true
        }
    }
}),

OopsReport = function (error, config) {

    if(!error) {
        return null;
    }

    this.error = error;
    this.config = config;

    this.initializer();
    return this;
};

OopsReport.prototype = {

    _io: null,

    initializer: function () {

        this._io = new Y.IO();

    },

    send: function () {

        return this._io.send(this.config.endPoint, Y.merge(this.config.ioConfig, {
            data: this._createReport()
        }));

    },

    _createReport: function () {

        var reportElements = [
            { reporter: this.config.reporter },
            this.uaToJSON(),
            this.config.sessionData,
            this._normalizeError(this.error)
        ],
            ret,
            i = 0,
            j = reportElements.length;

        // Will try to report all reportElements
        // If it fails, drop one element at a time
        // This will end up with just the basic error

        // e.g. if reporter, ua and sessiondata are all borked,
        // the report will still include the error
        for(; i < j; i++) {

            try {
                ret = Y.JSON.stringify(Y.merge.apply(this, reportElements.splice(i)));
            } catch (e) {

                // report the stringification failure
                // but drop everything but the error
                new Y.U1.util.OopsReport(e, {
                    endPoint: this.config.endPoint,
                    ioConfig: this.config.ioConfig
                }).send();

            } finally {
                if(Y.Lang.isString(ret)) {
                    break;
                }
            }

        }

        return ret;

    },

    _normalizeError: function (error) {

        var e       = {},
            props   = [
                { accepts: ['message', 'description'], output: 'message' },
                { accepts: ['url', 'fileName'], output: 'url' },
                { accepts: ['line', 'lineNumber'], output: 'line' },
                { accepts: ['column', 'columnNumber'], output: 'column' },
                { accepts: ['name'], output: 'name' },
                { accepts: ['number'], output: 'errorCode' },
                { accepts: ['stack', 'stacktrace'], output: 'stacktrace' }
            ];

        // take accepted inputs and add as output on e
        Y.each(props, function (prop) {
            Y.each(prop.accepts, function (accept) {
                if(error[accept]) {
                    e[prop.output] = error[accept];
                }
            });
        });

        return e;
    },

    uaToJSON: function () {
        var queryObj = {};

        Y.each(Y.UA, function(v, k) {
            if (!Y.Lang.isFunction(v)) {
                if (v) {
                    try {
                        queryObj[k] = v;
                        if(Y.Lang.isNumber(v)) {
                            queryObj.browser = k;
                            queryObj.browserVersion = v;
                        }
                    } catch (e) {
                        void(0);
                    }
                }
            }
        });

        return queryObj;
    }
};

Y.U1.util.Oops = Oops;
Y.U1.util.OopsReport = OopsReport;


}, '@VERSION@', {"requires": ["base", "io-base", "json-stringify"]});
