YUI.add('widget-cc-input', function (Y, NAME) {

"use strict";
Y.attrValidatePatch = function () {};

Y.attrValidatePatch.prototype = {
    _defAttrChangeFn: function (e) {
        if (!this._setAttrVal(e.attrName, e.subAttrName, e.prevVal, e.newVal) && (e.prevVal !== e.newVal)) {
            e.stopImmediatePropagation();
            this.fire(e.attrName + 'ValidationFail', e);
        } else {
            e.newVal = this.get(e.attrName);
        }
    }
};

var Lang = Y.Lang,
    DATE_LENGTH = 4,
    DEFAULT_CSC_LENGTH = 3,
    /**
     * CCN (credit card number) the whole number can generate two possible
     * errors, an invalid card type (from the IIN) and a failed Luhn validation
     */
    DEFAULT_CCN_SIZE = 16,
    DEFAULT_MASK = '0000 0000 0000 0000   12/15   123',

    E_INVALID_IIN = 0,
    E_FAILED_LUHN = 1,
    E_INVALID_DATE = 2,
    E_INVALID_CSC = 3,

    CTYPE = [{
        name: 'Visa',
        test: /^4/,
        mask: DEFAULT_MASK,
        size: DEFAULT_CCN_SIZE
    }, {
        name: 'MasterCard',
        test: /^5[1-5]/,
        mask: DEFAULT_MASK,
        size: DEFAULT_CCN_SIZE
    }, {
        name: 'American Express',
        test: /^3[47]/,
        mask: '0000 000000 00000   09/15   1234',
        size: 15,
        csc: 4
    }, {
        name: 'Diners Club',
        test: /^3(?:0[0-5]|[68][0-9])/,
        mask: '0000 000000 0000   09/15   123',
        size: 14
    }, {
        name: 'Discover',
        test: /^6(?:011|5)/,
        mask: DEFAULT_MASK,
        size: DEFAULT_CCN_SIZE
    }, {
        name: 'Japanese Credit Bank',
        test: /^35/,
        mask: DEFAULT_MASK,
        size: DEFAULT_CCN_SIZE
    }, {
        name: 'Dankort',
        test: /^5019/,
        mask: DEFAULT_MASK,
        size: DEFAULT_CCN_SIZE
    }],

EasyCard = Y.Base.create("payments-ccinput", Y.Widget, [Y.attrValidatePatch], {

    initializer: function (config) {


        this._ccn_input = config.ccn_input;
        this._issuer_select = config.issuer_select;
        this._expiry_input = config.expiry_input;
        this._expiry_month = config.expiry_month;
        this._expiry_year = config.expiry_year;
        this._csc_input = config.csc_input;

        var inputList = [
            this._ccn_input,
            this._issuer_select,
            this._csc_input
        ];
        if (this._expiry_month && this._expiry_year) {
            inputList.push(this._expiry_month, this._expiry_year);
            // store length of the first year
            this._expiry_year_length = this._expiry_year.one('option').get('value').length;
        } else {
            inputList.push(this._expiry_input);
        }
        this._inputList = new Y.NodeList(inputList);

        this._active = config.active;
        this._mask = config.mask;
        this._guide = config.guide;
        this._message = config.message;
        this._ccn = config.ccn;
        this._date = config.date;
        this._csc = config.csc;

        this._button = config.button;

        this._mask.plug(Y.Plugins.RangyInputs);

        this.publish("badInput", {
            defaultFn: this._defBadInput,
            bubbles: false
        });

        Y.mix(Y.Node.DOM_EVENTS, {
            animationend: 1,
            webkitAnimationEnd: 1
        });

        this.errors = config.message.all('.ccinput-error li');
        this.valid = config.message.one('.ccinput-valid');

        this.errors.hide();
        this.valid.hide();

        this.isPaste = false;

    },
    renderUI: function () {

        /**
        this.button = new Y.ToggleButton({
            srcNode: this._button,
            label: 'show/hide'
        });

        this.button.render();
        this.button.addTarget(this);
       **/
    },
    bindUI: function () {

        this._mask.on('keydown', this._onKeyDown, this);
        this._mask.on('keyup', this._onKeyUp, this);

        this._mask.get('parentNode').on(['webkitAnimationEnd', 'animationend'], this._onShakeEnd, this);

        // iin
        this.after('issuerChange', this._afterIssuerChange);
        this.on('issuerValidationFail', this._onIssuerFail);

        // luhn
        this.after('ccnChange', this._afterCcnChange);
        this.on('ccnValidationFail', this._onCcnFail);

        this.after('dateChange', this._afterDateChange);
        this.on('dateValidationFail', this._onDateFail);

        this.after('cscChange', this._afterCscChange);

        this.on('errorChange', this._onErrorChange);

        this.on('toggleButton:pressedChange', this.onButtonChange);
    },

    syncUI: function () {
        this.drawMaskedInput(this._mask.get('value'));
    },

    /**
    onButtonChange: function(e) {
        var pressed = e.newVal;

        if (pressed) {
            this._mask.get('parentNode').hide();
            this._inputList.show(true);
        } else {
            this._mask.get('parentNode').show();
            this._inputList.hide(true);
        }
    },
   **/

    //TODO use constants for the key codes
    /**
     * Capture number keypresses and format output accordingly.
     * Prevent anything that isn't relevant to card input.
     */
    _onKeyDown: function (e) {
        var code = e.charCode,
            node = this._mask,
            isDel;

        this.sel = node.rangy.getSelection();
        this.curVal = node.get('value');

        // don't prevent copy or paste
        if ((e.ctrlKey || e.metaKey) && (code === 86 || code === 67)) {
            if (code === 86) {
                this.isPaste = true;
            }
            return;
        }

        // some extra keys we don't want to prevent like tab, shift, etc
        if (!this._isPreventable(code)) {
            return;
        }

        // is this a isDelete action
        isDel = (code === 8 || code === 46) ? true : false;

        if (code > 95 && code < 106) {
            code -= 48;
        }

        // isDeletes and numbers, we deal with them
        if (isDel || code > 45 && code < 58) {
            e.preventDefault();
            this._onNumberKey(code, isDel);
            return;
        }

        // everything else is verbotten
        e.preventDefault();
        this._onPreventedKey();
    },

    // TODO refactor, have a preventable config like object
    _isPreventable: function(code) {
        var prevent = false;
        switch (code){
            // a lot of fall through
            case 9: // tab
            case 16: // shift
            case 17: // ctrl
            case 18: // alt
            case 35: // home
            case 36: // end
            case 37: // left arrow
            case 38: // up arrow
            case 39: // right arrow
            case 40: // down arrow
            case 91: // osx cmd
            break;

            default:
                prevent = true;
        }

        return prevent;
    },

    _onKeyUp: function () {

        if (this.isPaste) {
            this._onPaste();
        }

        this.isPaste = false;
    },

    _onIssuerFail: function () {
        this.set('error.iin', true);
    },

    _afterIssuerChange: function (e) {
        function toCls(s) {
            return s ? s.replace(/\s/g, '').toLowerCase() : s;
        }
        var oldcard = e.prevVal && e.prevVal.name,
            newcard = e.newVal.name;

        this.set('error.iin', false);
        this._guide.replaceClass(toCls(oldcard), toCls(newcard));
    },

    _onCcnFail: function () {
        this.set('error.luhn', true);
    },

    _afterCcnChange: function () {
        this.set('error.luhn', false);
    },


    _onDateFail: function () {
        this.set('error.date', true);
    },

    _afterDateChange: function () {
        this.set('error.date', false);
    },

    _afterCscChange: function () {
        this.set('error.csc', false);

        // TODO mark whole input as valid, blur
    },

    _onErrorChange: function(e) {
        var path = e.subAttrName.split(".")[1],
            newVal = e.newVal[path],
            hasError = false,
            hadError = false;

        switch (path) {
            case "iin":
                this._handleIinError(newVal);
            break;
            case "luhn":
                this._handleLuhnError(newVal);
            break;
            case "date":
                this._handleDateError(newVal);
            break;
            case "csc":
                this._handleCscError(newVal);
            break;
        }

        hasError = Y.Object.some(e.newVal, function(val) {
            return val;
        });

        hadError = Y.Object.some(e.prevVal, function(val) {
            return val;
        });

        this._handleGlobalError(hasError, hadError);
    },

    _handleGlobalError: function(error, prev) {
        if (error) {
            if (!prev) {
                this._shake();
            }
            this._mask.addClass('error');
        } else {
            this._mask.removeClass('error');
        }
    },

    _handleIinError: function(error) {
        if (error) {
            this._ccn.addClass('error');
            this.errors.item(E_INVALID_IIN).show('fadeIn');
        } else {
            this._ccn.removeClass('error');
            this.errors.item(E_INVALID_IIN).hide('fadeOut');
        }
    },

    _handleLuhnError: function(error) {
        if (error) {
            this._ccn.addClass('error');
            this.errors.item(E_FAILED_LUHN).show('fadeIn');
        } else {
            if (!this.get('error.iin')) {
                this._ccn.removeClass('error');
            }
            this.errors.item(E_FAILED_LUHN).hide('fadeOut');
        }
    },

    _handleDateError: function(error){
        if (error) {
            this._date.addClass('error');
            this.errors.item(E_INVALID_DATE).show('fadeIn');
        } else {
            this._date.removeClass('error');
            this.errors.item(E_INVALID_DATE).hide('fadeOut');
        }
    },

    _handleCscError: function(error) {
        if (error) {
            this._csc.addClass('error');
            this.errors.item(E_INVALID_CSC).show('fadeIn');
        } else {
            this._csc.removeClass('error');
            this.errors.item(E_INVALID_CSC).hide('fadeOut');
        }
    },

    _defBadInput: function () {
        this._shake();
    },

    _onPaste: function () {
        var mask = this._mask,
            /**
             * @type String
             */
            start = this.sel.start,
            end = this.sel.end,
            value = mask.get('value'),
            out;

        this._tokenize(value);
        out = this.drawMaskedInput(value.replace(/[^\d]/g, ''));

        mask.rangy.setSelection(this.findSelectionStart(this.curVal, value, out, start, end, false, true));
    },

    /**
     * @param {Number} code keycode from key event
     */
    _onNumberKey: function (code, isDel) {
        var char = String.fromCharCode(code) + '',
            mask = this._mask,
            /**
             * @type String
             */
            value = this.curVal,
            newVal,
            out,
            sel = mask.rangy.getSelection(),
            start = sel.start,
            end = sel.end,
            howMany;

        howMany = end - start;
        if (isDel) {
            char = null;
            if (!howMany) {
                // when end === start we need to still delete
                // so we pretend we're selecting the last item
                howMany = 1;
                --start;
            }
        }

        // insert or delete input at selectionStart to selectionEnd
        newVal = this.spliceNumericString(value, char, start, howMany);

        this._tokenize(newVal);
        out = this.drawMaskedInput(newVal, isDel);

        mask.rangy.setSelection(
            this.findSelectionStart(value, newVal, out, start, end, isDel)
        );
    },

    /**
     * calculate the correct index for the caret
     *
     * @param {Number} start selectionStart index
     * @param {Number} end selectionEnd index
     * @param {Boolean} isDel action is a deletion
     */
    findSelectionStart: function(curVal, newVal, out, start, end, isDel, isPaste) {

        var diff,
            formatChars = 0,
            position = out.length,
            count = 0;

        if ( curVal && curVal.length && (end < curVal.length) ) {
            if (start === end && !isPaste) {
                // non selection case, diff of previous and new is always 1
                diff = 1;
            } else {
                diff = (end - start);
                // don't count formatting chars from original selection in diff
                diff -= this.countMaskFormatChars(curVal.slice(start, end));
                diff -= (curVal.replace(/[^\d]/g, '').length - newVal.replace(/[^\d]/g, '').length);

                // selection now needs to include new formatting chars
                count = this.countMaskFormatChars(out.slice(start, start + diff));
                if (count) {
                    count = this.countMaskFormatChars(out.slice(start, start + diff + count));
                }

                diff +=count;

            }

            // we need to account for any format items which have been added to
            // the outer edge of the selection
            if (isDel) {
                // do we have a fc in the next deletable item?
                if ( /[^\d]/g.test(out[start - 1]) && start > 0) {
                    // we do! let's look back and see how many there are
                    formatChars = this.countLeadingFormatChars(out.slice(0, start), true);
                }
            } else {
                // have we added a formatting char, like space or /
                if ( /[^\d]/g.test(out[start + diff]) && start > 0) {
                    // we have! let's look ahead and see how many were added
                    formatChars = this.countLeadingFormatChars(out.slice(start + diff));
                }
            }

            // if start is negative we don't care, let the input do the default
            // caret position, otherwise...
            if (start >= 0) {
                if (isDel) {
                    position = start - diff - formatChars;
                } else {
                    position = start + diff + formatChars;
                }
            }
        }

        return position;
    },

    countLeadingFormatChars: function(string, reverse) {
        var c;

        if (reverse) {
            string = string.split("").reverse().join("");
        }

        c = string.match(/^[^\d]+/);
        c = c ? c[0].length : 0;

        return c;
    },

    countMaskFormatChars: function(string) {
        var c;

        c = string.match(/[^\d]/g);
        c = c ? c.length : 0;

        return c;
    },

    /**
     *
     * Splice two strings together
     *
     * keep formatting in place until finished, otherwise position indexes are
     * off
     *
     * @param {String} a
     * @param {String} b
     * @param {Number} position
     * @param {Number} howMany number of items to remove
     */
    spliceNumericString: function(a, b, position, howMany) {

        howMany = howMany || 0;

        a = a.split('');               // make array
        a.splice(position, howMany, b);  // splice in the new stuff
        a = a.join('');               // back to string
        a = a.replace(/[^\d]/g, '');  // remove mask formatting

        return a;
    },

    /**
     * tokenize
     * break a string up according to length values defined in the
     * current card type
     *
     * TODO refactor to just be about tokenizing
     * TODO refactor to simply slice, no if length
     * @param {string} value string containing ccn, date and csc
     */
    _tokenize: function(value) {
        var l = value.length,
            ccnLength,
            dateLength,
            cscLength,

            card = this.get('issuer');

        ccnLength = card.size;
        dateLength = card.size + DATE_LENGTH;
        cscLength = card.size + DATE_LENGTH + (card.csc || DEFAULT_CSC_LENGTH);

        // enough to identify the card via the CTYPE regexes
        if (l > 3) {
            this.set('issuer', value);
            // enough for a complete ccn on which to do luhn validation
            if (l >= ccnLength) {
                this.set('ccn', value.slice(0, ccnLength));
                // enough for a partial date
                if (l > ccnLength) {
                    this.set('date', value.slice(ccnLength, dateLength));
                    // enough for a complete csc
                    if (l >= cscLength) {
                        this.set('csc', value.slice(dateLength, cscLength));
                    }
                }
            }
        }

        if (l < 4) {
            this.set('error.iin', false);
        }

        if (l < cscLength) {
            this.valid.hide();

            if (l < ccnLength) {
                this.set('error.luhn', false);
            }
        }

        // always be setting input values, ready to submit
        this._ccn_input.set('value', value.slice(0, ccnLength));
        this._issuer_select.one('option[value="'+card.name+'"]').set('selected', 'selected');

        if (this._expiry_month && this._expiry_year) {
            this.setExpiryDates(value, ccnLength, dateLength);
        } else {
            this._expiry_input.set('value', value.slice(ccnLength, dateLength));
        }

        this._csc_input.set('value', value.slice(dateLength, cscLength));

        return this;
    },

    // sets expiry month and year SELECTs to reflect cc input value
    setExpiryDates: function (value, ccnLength, dateLength) {

        var yearOption = this.getYearOption(value.slice(ccnLength, dateLength).slice(2)),
            monthOption = this.getMonthOption(value.slice(ccnLength, dateLength).slice(0,2));

        if (monthOption) {
            monthOption.set('selected', true);
        }

        if (yearOption) {
            yearOption.set('selected', true);
        }
    },

    // returns OPTION representing given month
    getMonthOption: function (month) {
        month = parseInt(month, 10);
        return this._expiry_month.one('option[value="'+month+'"]');
    },

    // returns OPTION representing given year
    getYearOption: function (year) {
        year = this.normalizeYear(year, this._expiry_year_length);
        return this._expiry_year.one('option[value="'+year+'"]');
    },

    isValidCsc: function (csc) {
        var card = this.get('issuer'),
            len = card.csc;

        return (csc.length === len);
    },

    /**
     * Validate a string/number of form MMYY as it's typed
     * @param {String|Number} Date of form MMYY
     * @return Boolean
     */
    isValidExpiryDate: function (date) {
        var m1, m2, y1, y2;

        if (!date || !Lang.isNumber(+date)) {
            return false;
        }

        date += '';

        m1 = date[0] && +date[0];
        m2 = date[1] && +date[1];
        y1 = date[2] && +date[2];
        y2 = date[3] && +date[3];

        if (m1 > 1) {
            return false;
        }

        if (Lang.isNumber(m2)) {
            if (m1 > 0 && m2 > 2) {
                return false;
            }

            if ((m1 + m2 === 0)) {
                return false;
            }

            if (this._expiry_month && !this.getMonthOption(m1.toString() + m2.toString())) {
                return false;
            }
        }

        if (Lang.isNumber(y2)) {
            if (this._expiry_year && !this.getYearOption(y1.toString() + y2.toString())) {
                return false;
            }

        }

        // TODO check year is in future
        return true;
    },

    /**
     * @param {String} value the card number, unformatted, as a string
     * @param {Boolean} isDel true if change is a deletion
     */
    drawMaskedInput: function (value, isDel) {
        var mask = this.get('issuer.mask'),
            out = this.maskNumericString(value, mask, isDel);

        this._mask.set('value', out);
        this.renderGuide(out);

        return out;
    },

    /**
     * @param {String} value unformatted card combined number
     */
    renderGuide: function (value) {
        var ccn = this._ccn,
            date = this._date,
            csc = this._csc,
            masks = this.get('issuer.mask').split('   '),
            b;

        if (value && value.length) {
            Y.Array.each(value.split('   '), function (item, index) {
                b = this.mergeString(item, masks[index], true);
                masks[index] = b;
            }, this);
        }

        Y.Array.each([ccn, date, csc], function (item, index) {
            item.setHTML(masks[index]);
        }, this);
    },

    _shake: function () {
        var node = this._mask.get('parentNode');

        node.addClass('shake');
    },

    _onShakeEnd: function () {
        var node = this._mask.get('parentNode');

        node.removeClass('shake');
    },

    // a is the incomplete string
    // b is the string used to complete a
    mergeString: function (a, b, wrap, len) {
        if (!a) {
            return b;
        }
        a = a.split('');
        b = b.split('');

        len = a.length;

        Y.Array.each(b, function (item, index) {
            if (!a[index]) {
                a[index] = item;
            }
        });

        if (wrap) {
            a.unshift('<b>');
            a.splice(len + 1, 0, '</b>');
        }

        return a.join('');
    },

    maskNumericString: function (string, mask, isDel) {
        mask = mask || this.get('issuer').mask;


        var input = string.split('');

        /**
         * Loop over the mask, comparing each mask item with the same index in
         * the input string, if the mask has a non numeric char in a truthy
         * index in the input, insert that char; do same at end of input
         */
        Y.Array.some(mask.split(''), function (item, index) {

            // end of input string and nothing to append
            if (!input[index] && /[0-9]/.test(item)) {
                return true;
            }

            // inside input string and something to insert
            if (input[index] && !/[0-9]/.test(item)) {
                input.splice(index, 0, item); // append item to input
            } else
                // end of input string and something to append
                if (!isDel && !/[0-9]/.test(item)) {
                    input.splice(index, 0, item);
                }

        }, this);

        return input.join('');
    },

    _onPreventedKey: function () {
        this._shake();
    },

    /**
     * gets the card type object from the iin
     * @param {Number} val Issuer Identification Number (iin), 6 digits
     */
    getCardByIssuerIdentificationNumber: function (val) {
        var i = 0, l = CTYPE.length,
        card, re;

        for (; i < l; i++) {
            card = CTYPE[i],
            re = card.test;

            if (re.test(val)) {
                return card;
            }
        }
        return Y.Attribute.INVALID_VALUE;
    },

    /**
     * luhn check against number
     * http://en.wikipedia.org/wiki/Luhn_algorithm
     * @param {Number} number
     * @return {Boolean}
     */

    isValidLuhn: function (number) {
        var i, d, // digit
        column, sum = 0;

        number = +number;

        if (!Lang.isNumber(number)) {
            return false;
        }
        number += ''; // make the number a string/array-like

        for (i = number.length, column = i % 2; i--;) { // work right to left
            d = +number[i]; // take each digit in the number
            // double every second digit, from the rightmost, then sum all the
            // individual digits
            sum += (column === i % 2) ? ((d *= 2) > 9) ? (d - 9) : d : d;
        }

        // if the sum is a multiple of 10, the number is possibly valid
        sum %= 10;
        return !sum;
    },

    // convert year to the desired length
    // supported desiredLengths are 2 and 4
    normalizeYear: function (year, desiredLength) {
        var curLen = year.length;

        if (curLen > desiredLength) {
            return this.fullYearToShortYear(year);
        } else if (curLen < desiredLength) {
            return this.shortYearToFullYear(year);
        } else {
            return year;
        }
    },

    // convert short year to full year
    // years in the past are coerced into future years
    shortYearToFullYear: function (shortYear) {

        var now = new Date(),
            fullYear = now.getFullYear().toString(), // "2014"
            century = fullYear.slice(0,2), // "20"
            convertedFullYear;

        // if shortYear has leading zero that zero will be dropped, re-add it
        shortYear = this.paddy(shortYear, 2);

        // if shortYear is less then current short year, bump century
        if (parseInt(shortYear,10) < this.fullYearToShortYear(fullYear)) {
            century = parseInt(century, 10) + 1;
            century = century.toString();
        }

        convertedFullYear = parseInt(century + shortYear, 10);

        return convertedFullYear;

    },

    // remove the century part of a year
    fullYearToShortYear: function (fullYear) {
        return parseInt(fullYear.toString().slice(2), 10);
    },

    paddy: function(n, p, c) {
        var pad_char = typeof c !== 'undefined' ? c : '0',
            pad = new Array(1 + p).join(pad_char);
        return (pad + n).slice(-pad.length);
    }

}, {
    HTML_PARSER: {
        ccn_input: 'input.ccinput-ccn',
        issuer_select: 'select.ccinput-issuer',
        expiry_input: 'input.ccinput-expires',
        expiry_month: 'select.ccinput-month',
        expiry_year: 'select.ccinput-year',
        csc_input: 'input.ccinput-csc',

        mask: '.ccinput-mask',
        guide: '.ccinput-guide',

        message: '.ccinput-message',

        ccn: 'span.ccinput-ccn',
        date: 'span.ccinput-date',
        csc: 'span.ccinput-csc',

        button: '.ccinput-button'
    },

    ATTRS: {
        ccn: {
            value: null,
            validator: 'isValidLuhn'
        },
        date: {
            value: 0, // current date plus one
            validator: 'isValidExpiryDate'
        },
        csc: {
            value: null
        },
        active: {
            value: true,
            validator: function (value) {
                return Lang.isBoolean(value);
            }
        },
        issuer: {
            // card type is set by the first six digits of the long number
            // which identify the issuer (IIN: Issuer Identification Number)
            value: '42',
            setter: 'getCardByIssuerIdentificationNumber'
        },
        error: {
            value: {
                    iin: 0,
                    luhn: 0,
                    expiry: 0,
                    csc: 0
            },
            setter: 'setErrorState'
        }
    }

});

Y.EasyCard = EasyCard;


}, '@VERSION@', {"requires": ["node-pluginhost", "widget", "base", "transition", "button", "rangyinputs", "template"]});
