var exec = require('child_process').spawn,
    path = require('path');

module.exports = function(grunt) {

    grunt.initConfig({
        pkg: grunt.file.readJSON('package.json'),

        cm: './branches/config-manager/cm.py',

        configManager: 'config-manager.txt',

        connect: {
            server: {
                options: {
                    hostname: '*',
                    port: 9003
                }
            }
        },

        jshint: {
            options: {
                asi:          false,
                bitwise:      true,
                boss:         false,
                browser:      true,
                curly:        true,
                debug:        false,
                eqeqeq:       true,
                eqnull:       false,
                es5:          false,
                esnext:       false,
                evil:         false,
                expr:         true,
                forin:        false,
                funcscope:    false,
                globalstrict: false,
                immed:        true,
                iterator:     false,
                lastsemic:    false,
                latedef:      true,
                laxbreak:     true,
                laxcomma:     false,
                loopfunc:     false,
                maxerr:       500,
                maxlen:       0,
                multistr:     false,
                newcap:       true,
                noarg:        true,
                node:         true,
                noempty:      true,
                nomen:        false,
                onecase:      false,
                onevar:       false,
                passfail:     false,
                plusplus:     false,
                predef: [
                    "after",
                    "afterEach",
                    "assert",
                    "before",
                    "beforeEach",
                    "d3",
                    "describe",
                    "escape",
                    "expect",
                    "GlobalConfig",
                    "jsyaml",
                    "consoleManager",
                    "it",
                    "saveAs"
                ],
                proto:        false,
                regexdash:    false,
                regexp:       false,
                scripturl:    false,
                shadow:       false,
                smarttabs:    false,
                strict:       true,
                sub:          false,
                supernew:     false,
                trailing:     true,
                unused:       'vars',
                white:        false,
                yui:          true
            },
            post: {
                src: [
                    'build/**/*.js',
                    '!build/**/*-coverage.js',
                    '!build/**/*-debug.js',
                    '!build/**/*-min.js',
                    '!build/**/*-templates.js'
                ]
            },
            pre: {
                options: {
                    strict: false
                },
                src: [
                    'src/**/*.js{on,}',
                    '!src/**/tmpl/*.js'
                    ]
            }
        },

        watch: {
            options: {
                spawn: false
            },
            css: {
                files: [
                    'src/**/*.css'
                ],
                tasks: [
                    'modules',
                    'shifter',
                    'mocha'
                ]
            },
            src: {
                files: [
                    'src/**/*.js',
                    'src/**/build.json',
                    '!src/**/tmpl/*.js'
                ],
                tasks: [
                    'yui_template',
                    'jshint:pre',
                    'modules',
                    'shifter',
                    'mocha'
                ]
            },
            templates: {
                files: ['src/**/js/templates/*.html'],
                tasks: [
                    'yui_template',
                    'mocha'
                ]
            },
            tests: {
                files: ['src/**/tests/unit/**.{html,js}'],
                tasks: ['mocha']
            }
        },

        shifter: {
            options: {
                strict: true,
                lint: false,
                csslint: false,
                silent: true
            }
        },

        mocha: {
            test: {
                src: ['src/**/tests/unit/*.html', '!src/**/tests/unit/*-yui.html']
            }
        },

        grover: {
            unit: {
                src: ['src/**/tests/unit/*-yui.html']
            }
        },

        modules: {
            // online services modules
            os: {
                src: ['src/**/build.json'],
                dest: 'build/config.js'
            }
        },

        clean: ['./build'],

        compress: {
            options: {
                archive: 'release/<%= pkg.version %>/<%= pkg.name %>-<%= pkg.version %>.tgz',
                mode: 'tgz'
            },
            src: ['build/**']
        },

        yui_template: {
            all: {
                files: [{
                    expand: true,
                    src: 'src/**/js/templates/*.html',
                    ext: '.js'
                }]
            }
        },

        shell: {
            configmanager: {
                options: {
                    stdout: true
                },
                command: [
                    'echo "Pulling config-manager"',
                    'bzr branch lp:~ubuntuone-pqm-team/config-manager/stable branches/config-manager'
                ].join('&&')
            },
            bootstrap: {
                options: {
                    stdout: true
                },
                command: '<%= cm %> update <%= configManager %>',
            }
        }

    });

    /*
     * make sure watch only acts on the changed files and their
     * associated files
     */
    grunt.event.on('watch', function(action, filepath) {


        var module = filepath.split(path.sep)[1];

        // jshint
        grunt.config('jshint.pre.src', filepath);

        // yui_template
        grunt.config('yui_template.all.src', filepath);

        // shifter
        grunt.config('shifter.options.modules', [module]);
        grunt.config('shifter.options.silent', false);

        // mocha
        grunt.config('mocha.test.src', './src/' + module + '/tests/unit/*.html');

    });

    grunt.loadNpmTasks('grunt-contrib-clean');
    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-contrib-compress');
    grunt.loadNpmTasks('grunt-contrib-connect');
    grunt.loadNpmTasks('grunt-yui-template-compile');
    grunt.loadNpmTasks('grunt-mocha');
    grunt.loadNpmTasks('grunt-shell');

    grunt.registerTask('default', 'build');
    grunt.registerTask('build', ['clean', 'yui_template', 'shifter', 'modules', 'jshint:post']);
    grunt.registerTask('release', ['build', 'compress']);
    grunt.registerTask('test', ['build', 'grover', 'mocha']);
    grunt.registerTask('run', ['connect', 'watch']);
    grunt.registerTask('bootstrap', ['shell:configmanager', 'shell:bootstrap']);

    grunt.registerTask('shifter', 'Build YUI modules with Shifter', function() {
        var done = this.async(),
            config = grunt.config.get('shifter.options') || {},
            shifter = config.bin || '../node_modules/.bin/shifter',
            args = [
                shifter,
                'build',
                '--walk'
            ],
            child;

        if (config) {
            if (config.modules && config.modules.length) {
                config.modules.forEach(function(module) {
                    args = args.concat('--modules', module);
                });
            }

            if (config.strict) {
                args.push('--strict');
            }

            if (!config.nolint) {
                args.push('--no-lint');
            }

            if (!config.nocsslint) {
                args.push('--no-csslint');
            }

            if (config.quiet) {
                args.push('--quiet');
            }

            if (config.silent) {
                args.push('--silent');
            }
        }

        child = exec(process.execPath, args, {
            cwd: path.join(process.cwd(), 'src'),
            stdio: 'inherit',
            env: process.env
        });

        child.on('exit', function(code) {
            if (code) {
                grunt.fail.fatal('shifter build exited with code: ' + code);
            }
            done();
        });
    });

    grunt.registerMultiTask('grover', 'Test with Grover', function() {
        var done = this.async(),
        // FIXME hard coded path path
            grover = this.data.bin || './node_modules/.bin/grover',
            args,
            child;

        args = this.filesSrc;
        args.unshift('./node_modules/.bin/phantomjs');
        args.unshift('--phantom-bin');
        args.unshift(grover);

        child = exec(process.execPath, args, {
            cwd: process.cwd(),
            stdio: 'inherit',
            env: process.env
        });

        child.on('exit', function(code) {
            if (code) {
                grunt.fail.fatal('grover build exited with code: ' + code);
            }
            done();
        });
    });

    grunt.registerMultiTask('modules', 'Build modules hash for YUI', function() {

        var modules = {},
            config = {},
            out = '',
            group = this.target,
            template = this.data.tmpl ||
                'YUI.GlobalConfig = <%= config %>;';

        this.filesSrc.forEach(function(filepath) {
            var meta = grunt.file.readJSON(filepath),
                key;

            // allow for multiple modules per build file
            for (key in meta.builds) {

                if (meta.builds.hasOwnProperty(key)) {
                    modules[key] = {};
                    modules[key] = meta.builds[key].config;

                    if (meta.builds[key].skinnable) {
                        modules[key].skinnable = meta.builds[key].skinnable;
                    }
                }

            }
            for (key in meta.rollups) {

                if (meta.rollups.hasOwnProperty(key)) {
                    modules[key] = {};
                    modules[key] = meta.rollups[key].config;
                }

            }
        });

        config.groups = {};
        config.groups[group] = {};
        config.groups[group].modules = modules;

        out = grunt.template.process(template, {data: {config: JSON.stringify(config) }});

        grunt.file.write(this.data.dest, out);

    });

};
