YUI.add('uploader-myapps-packages', function (Y, NAME) {

"use strict";
var UploaderMyappsPackages = Y.Base.create('uploaderMyappsPackage', Y.Base, [], {

    _cfg: null,

    _binaryUploader: null,

    _sourceUploader: null,

    initializer: function (config) {

        this._cfg = config;

        this.publish('nobinary');
        this.publish('binaryisnullbyte');
        this.publish('binaryisnotunique');
        this.publish('error');

        this._createSourceUploader();
        this._createBinaryUploader();

        Y.one('.devportal-form').on('submit', this.submitHandler, this);

        this.bindUI();

    },

    _createSourceUploader: function () {

        this._sourceUploader = new Y.UploaderMyapps(Y.merge(this._cfg, {
            fileFieldName:      'source',
            simpleUploader:     '.field-source input',
            dragAndDropArea:    '.field-source .dd-proxy'
        }));
        this._sourceUploader.addTarget(this);

    },

    _createBinaryUploader: function () {

        this._binaryUploader = new Y.UploaderMyapps(Y.merge(this._cfg, {
            fileFieldName:      'binary',
            simpleUploader:     '.field-binary input',
            dragAndDropArea:    '.field-binary .dd-proxy'
        }));
        this._binaryUploader.addTarget(this);

    },

    bindUI: function () {

        this.on([ 'partialcomplete', 'incomplete' ], function () {
            this.fire('error', { message: this.get('strings.could_not_process') });
        });

        this.on(['nobinary', 'binaryisnullbyte', 'binaryisnotunique'], this._binaryFailureHandler);

        this.on('myappsUploader:uploadfailed', this._packageUploadFailed);

    },

    _binaryFailureHandler: function (e) {

        var uploader = this._binaryUploader,
            f = '.field-binary';

        this.fire('error', { message: this.get('strings.correct_errors')});

        switch (e.type) {
            case 'uploaderMyappsPackage:nobinary':
                this.fire('error', { message: this.get('strings.binary_none'), field: f });
                break;
            case 'uploaderMyappsPackage:binaryisnullbyte':
                this.fire('error', { message: this.get('strings.binary_nullbyte'), field: f });
                break;
            case 'uploaderMyappsPackage:binaryisnotunique':
                this.fire('error', { message: this.get('strings.already_uploaded'), field: f });
                uploader.resetAll();

                // re-create sourceuploader
                this._createSourceUploader();

                // re-render sourceuploader
                this.render();
                break;
        }

        uploader.once('fileListChange', function () { this.fire('clear'); }, this);
    },

    _packageUploadFailed: function (e) {

        var response,

            // TODO: convoluted
            generatedMessage = e.context.getFileError(e.originEvent).message,
            fieldNode = e.context.get('contentBox').ancestor('.field'),
            f;

        try {
            response = Y.JSON.parse(e.file.get('xhr').response);
        } catch(e) {
            response = null;
        }

        if(response) {

            for (f in response.errors) {
                if(response.errors.hasOwnProperty(f)) {
                    this.fire('error', { message: response.errors[f], field: fieldNode });

                    // XXX: if non-unique error, notify
                    /*jshint loopfunc: true */
                    Y.each(response.errors[f], function (error) {
                        if(f === 'binary_path' && /unique/g.test(error)) {
                            this.fire('binaryisnotunique');
                        }
                    }, this);
                }
            }

        } else {
            this.fire('error', { message: this.get('strings.correct_errors') });
            this.fire('error', { message: generatedMessage, field: fieldNode });
        }

    },

    render: function () {

        if(!this._binaryUploader.get('rendered')) {
            this._binaryUploader.render('.field-binary .field-content');
            Y.one('.field-binary').show();
        }

        if(!this._sourceUploader.get('rendered')) {
            this._sourceUploader.render('.field-source .field-content');
            Y.one('.field-source').show();
        }

    },

    submitHandler: function (e) {

        var binUploader = this._binaryUploader,
            srcUploader = this._sourceUploader,
            binFile, srcFile;

        e.preventDefault();

        // TODO: make less ambiguous?
        this.fire('clear');

        // validate and update version number before allowing the upload
        // when ok, it will populate path and signature fields
        if (!this.updateVersion()) {
            return;
        }

        // updown checks that a source file also has a binary file, so combine the
        // fileLists and upload only one list
        binFile = binUploader.get('fileList')[0];
        srcFile = srcUploader.get('fileList')[0];

        if (binFile && !binFile.get('size')) {

            binUploader.cancelFile(binFile);
            this.fire('binaryisnullbyte');
            return;

        }

        // have binary
        if(binFile && binFile.get('size')) {

            this.uploadBinary(srcFile);

            // hide and destroy source uploader
            Y.one('.field-source').hide();
            srcUploader.destroy();

            // rebind onbeforeunload
            window.onbeforeunload = Y.bind(binUploader._beforeUnload, binUploader);

            this.setCompleteHandler(binFile, srcFile);

        // no binary, MUST have a binary
        } else {

            this.fire('nobinary');

        }
    },

    setCompleteHandler: function (binFile, srcFile) {
        this._binaryUploader.once('fullycomplete', function () {
            var has_src = Boolean(srcFile);
            this.get('successFn')(binFile.get('size'), has_src);
        }, this);
    },

    uploadBinary: function (src) {

        // get updated field values
        var vars = {
            binary_path: Y.one('#id_binary_path').get('value'),
            source_path: Y.one('#id_source_path').get('value'),
            signature: Y.one('#id_signature').get('value')
        };

        // XXX:
        // We need to upload the source file in the same
        // request as the binary. Adding the source file
        // as a parameter accomplishes this
        if(src) {
            vars.source = src.get('file');
        }

        this._binaryUploader.uploadAll(null, vars);

    },

    getCallbackConfig: function () {
        return {
            sync: true,
            method: 'POST',
            data: {
                'version': Y.one('#id_version').get('value'),
                'framework': Y.one('#id_framework').get('value'),
                'architectures': Y.all('.field-architectures input[name=architectures]').filter(':checked').get('value'),
                'architecture_independent': Y.one('#id_architectures_3').get('checked'),
                'changelog': Y.one('#id_changelog').get('value'),
                'csrfmiddlewaretoken': Y.Cookie.get('csrftoken')
            },
            on: {
                failure : Y.bind(function () {

                    this.fire('error', {
                        message: this.get('strings.could_not_process')
                    });

                }, this)
            }
        };
    },

    updateVersion: function () {
        var valid = true,
            cfg = this.getCallbackConfig(),
            response = Y.io(this.get('updateVersionURL'), cfg),
            field;

        try {
            response = Y.JSON.parse(response.responseText);
        }
        catch (e) {
            this.fire('error', {
                message: this.get('strings.could_not_process')
            });
            return false;
        }

        if (!response.success) {
            this.fire('error', {
                message: this.get('strings.correct_errors')
            });

            for (field in response.errors) {
                if(response.errors.hasOwnProperty(field)) {
                    this.fire('error', {
                        message: response.errors[field],
                        field: '.field-' + field
                    });
                    valid = false;
                }
            }
        }

        Y.one('#id_binary_path').set('value', response.binary_path);
        Y.one('#id_source_path').set('value', response.source_path);
        Y.one('#id_signature').set('value', response.signature);

        return valid;
    },

    destructor: function () {
        this._binaryUploader.destroy();
        this._sourceUploader.destroy();
    }


}, {
    ATTRS: {
        strings: { value: null },
        successFn: { value: function () {} },
        template: { value: null },
        updateVersionURL: { value: '' }
    }
});

Y.UploaderMyappsPackages = UploaderMyappsPackages;


}, '@VERSION@', {"requires": ["base", "uploader-myapps", "util", "cookie", "io", "json-parse"]});
