YUI.add('uploader-myapps', function (Y, NAME) {

 "use strict";

var Uploader = Y.Base.create('uploaderMyapps', Y.UploaderAugmented, [], {

    initializer: function () {

        this.after(['statusChange', 'totaluploadprogress', 'dropAreaStatusChange', 'render', 'fullycomplete'], this.syncUI);

        this.on('render', this._dpRender, this);
        this.after('render', this._dpPostRender, this);

        this.on('enabledChange', this._dpEnabledChange, this);

        this.get('contentBox').append(Y.Node.create('<div class="dd-proxy"/>'));

        this.after('fileListChange', this._dpFileListChange, this);

        this.on('uploaderror', this._dpUploadError);

    },

    _dpFileListChange: function (e) {

        // user selected a file, disable dragdrop and select button
        // TODO: changing the DD area should cover the DD changes
        if(e.newVal.length === 1) {

            this.get('selectFilesButton').hide();
            this.endDrag();
            this.set('dragAndDropArea', null);

        } else if (e.newVal.length === 0) {

            this.get('selectFilesButton').show();
            this.set('dragAndDropArea', '.dd-proxy');

        }

        this.syncUI();

    },

    _dpEnabledChange: function (e) {

        this.get('selectFilesButton').set('disabled', !e.newVal);

        // TODO: not part of uploader, refactor out
        Y.one('button[type=submit]').set('disabled', !e.newVal);

    },

    _dpUploadError: function (event) {

        /**

        XXX: If a file fails, we will retry it.
        When it ultimately fails after n retries, we should report it to the user.
        The uploader queue (which is handling the file error) currently only reports the status and status code (e.g. 400 BAD REQUEST).
        It will also immediately call .abort() on the XHR, which makes it hard to access the reponse afterwards.

        So, if the file ultimately fails, we:

            1. Detach the queue error handler (which cancels our XHR)
            2. Attach our own error handler, which notifies our subscribers of the
                ultimate failure *AND* includes our XHR response in the event payload
            3. Call the upload handler we just attached, so the queue can cancel the XHR and remove it from the queue

        */

        if(this.queue.uploadRetries[event.file.get('id')] === this.get('retryCount')) {

            event.file.detach('uploaderror', this.queue._uploadErrorHandler);

            event.file.on('uploaderror', function (e) {

                this.fire('uploadfailed', {
                    file: event.file,
                    originEvent: event,
                    context: this,
                    response: e.target.get('xhr').response
                });

                if(this.queue) {
                    this.queue._uploadErrorHandler(e);
                }


            }, this);

        }

    },

    _dpPostRender: function () {

        if(this.get('simpleUploader')) {
            this.get('simpleUploader').hide();
        }

        // unset width height on select button
        this.get("selectFilesButton").setStyles({width:'auto', height:'auto'});

    },

    resetAll: function () {

        if(this.queue) {
            this.queue.destroy();
            this.queue = null;
        }

        this.setAttrs({
            fileList: [],
            enabled: true,
            "status.text": this.get('strings.uploads_empty'),
            dropAreaStatus: this.get('strings.drop_ready')
        });

    },


    destructor: function () {

        if(this.get('simpleUploader')) {
            this.get('simpleUploader').show();
        }

    },

    _dpRender: function () {
        this.get('contentBox').append(Y.Node.create('<div class="wrapper"/>'));
    },

    syncUI: function (event) {

        event = event || {};

        var context = {
                files:  this.get('fileList'),
                status: this.get('status.text'),
                dd:     this.get('dropAreaStatus')
            };

        // if event reports percent loaded or that everything is uploaded
        // add percent which will render a <progress> element
        switch(event.type) {
            case 'uploaderqueue:totaluploadprogress':
                    context.percent = event.percentLoaded;
                break;
            case 'uploaderMyapps:fullycomplete':
                    context.percent = 100;
                break;
        }

        this.get('contentBox').one('.wrapper').setHTML(this.get('template')(context));

    }

}, {
    ATTRS: {
        follow: { value: false },
        simpleUploader: {
            value: null,
            setter: function (val) {
                if(Y.Lang.isString(val)) {
                    return Y.one(val);
                } else {
                    return val;
                }
            }
        },
        template: { value: Y.Template.get('uploader-myapps') }
    }
});

Y.UploaderMyapps = Uploader;


}, '@VERSION@', {"skinnable": true, "requires": ["template", "uploader-augmented", "uploader-myapps-templates", "node"]});
