YUI.add('util-share', function (Y, NAME) {

"use strict";
Y.namespace("U1.util");

// TODO web intents?

var Share = function(link, photo) {

    this.photo = photo;
    this.link = link;

};

Share.prototype = {

    getUrl: function() {
        if (this.url && this.url.length) {
            return this.url;
        } else {
            throw new Error("must set a url type before get");
        }
    },

    setUrl: function(type) {
        switch (type) {
            case 'facebook':
                this.url = this._facebookUrl(this.link, this.photo);
            break;
            case 'twitter':
                this.url = this._twitterUrl(this.link);
            break;
            default:
        }

        return this;

    },

    /**
     * Opens a window to share the link
     */
    open: function(w, h, name) {
        name = name || "U1Share";
        var url = this.getUrl(),
            conf = this.getConfString(w, h),
            u1Window;
        u1Window = Y.config.win.open(url, name, conf);
    },

    /**
     * Generated a conf string from width and height
        @method generateConfString
        @param {Number} w the window width (defaults to 420)
        @param {Number} h the window height (defaults to 480)
     */
    getConfString: function(w, h) {
        w = w || 420;
        h = h || 480;
        var win = Y.config.win,
            left_ = parseInt((win.screen.width/2)-(w/2), 10),
            top_ = parseInt((win.screen.height/2)-(h/2), 10);
        return "width="+w+",height="+h+",left="+left_+",top="+top_+",status=yes";
    },

    _facebookUrl: function(link, photo) {
        var url = "http://www.facebook.com/dialog/feed?",
            id = "166704313352575",
            display = "popup",
            redirect = "https://one.ubuntu.com/window-close/";

        if (link && link.length) {

            url += ["app_id=" + id,
                "link="+ window.encodeURI(link),
                "redirect_uri="+ window.encodeURI(redirect),
                "display="+ display
            ].join('&');

            if (photo && photo.length) {
                url += "&picture="+ window.encodeURI(photo);
            }

            return url;

        } else {
            throw new Error("_createFacebookUrl requires link to share");
        }
    },

    _twitterUrl: function(link) {
        var url = "https://twitter.com/share?";

        if (link && link.length) {

            url += [
                "url="+ window.encodeURI(link),
                "text="+ window.encodeURI("Sharing a link from UbuntuOne."),
                "hashtags=ubuntuone"
            ].join('&');


            return url;

        } else {
            throw new Error("createTwitterUrl requires link to share");
        }
    }

};

Y.U1.util.Share = Share;


}, '@VERSION@', {"requires": ["base", "node"]});
YUI.add('util-stacktrace', function (Y, NAME) {

"use strict";
Y.namespace("U1.util");

/**
* Main function giving a function stack trace with a forced or passed in Error
*
* @cfg {Error} e The error to create a stacktrace from (optional)
* @cfg {Boolean} guess If we should try to resolve the names of anonymous functions
* @return {Array} of Strings with functions, lines, files, and arguments where possible
*/
var Stacktrace = function (options) {
    options = options || {guess: true};
    var ex = options.e || null,
        guess = !!options.guess,
        p = new Y.U1.util.Stacktrace.implementation(), result = p.run(ex);
    return (guess) ? p.guessAnonymousFunctions(result) : result;
},
ActiveXObject = ActiveXObject || undefined;

Stacktrace.implementation = function() {};

Stacktrace.implementation.prototype = {

    /**
    * @param {Error} ex The error to create a stacktrace from (optional)
    * @param {String} mode Forced mode (optional, mostly for unit tests)
    */
    run: function(ex, mode) {
        ex = ex || this.createException();
        // examine exception properties w/o debugger
        //for (var prop in ex) {alert("Ex['" + prop + "']=" + ex[prop]);}
        mode = mode || this.mode(ex);
        if (mode === 'other') {
            Y.log('Mode ignored.');
            void(0);
            //return this.other(arguments.callee);
        } else {
            return this[mode](ex);
        }
    },

    createException: function() {
        try {
            this.undef();
        } catch (e) {
            return e;
        }
    },

    /**
    * Mode could differ for different exception, e.g.
    * exceptions in Chrome may or may not have arguments or stack.
    *
    * @return {String} mode of operation for the exception
    */
    mode: function(e) {
        if (e['arguments'] && e.stack) {
            return 'chrome';
        } else if (e.stack && e.sourceURL) {
            return 'safari';
        } else if (e.stack && e.number) {
            return 'ie';
        } else if (typeof e.message === 'string' && typeof window !== 'undefined' && window.opera) {
            // e.message.indexOf("Backtrace:") > -1 -> opera
            // !e.stacktrace -> opera
            if (!e.stacktrace) {
                return 'opera9'; // use e.message
            }
            // 'opera#sourceloc' in e -> opera9, opera10a
            if (e.message.indexOf('\n') > -1 && e.message.split('\n').length > e.stacktrace.split('\n').length) {
                return 'opera9'; // use e.message
            }
            // e.stacktrace && !e.stack -> opera10a
            if (!e.stack) {
                return 'opera10a'; // use e.stacktrace
            }
            // e.stacktrace && e.stack -> opera10b
            if (e.stacktrace.indexOf("called from line") < 0) {
                return 'opera10b'; // use e.stacktrace, format differs from 'opera10a'
            }
            // e.stacktrace && e.stack -> opera11
            return 'opera11'; // use e.stacktrace, format differs from 'opera10a', 'opera10b'
        } else if (e.stack && !e.fileName) {
            // Chrome 27 does not have e.arguments as earlier versions,
            // but still does not have e.fileName as Firefox
            return 'chrome';
        } else if (e.stack) {
            return 'firefox';
        }
        return 'other';
    },

    /**
    * Given a context, function name, and callback function, overwrite it so that it calls
    * printStackTrace() first with a callback and then runs the rest of the body.
    *
    * @param {Object} context of execution (e.g. window)
    * @param {String} functionName to instrument
    * @param {Function} callback function to call with a stack trace on invocation
    */
    instrumentFunction: function(context, functionName, callback) {
        context = context || window;
        var original = context[functionName];
        context[functionName] = function instrumented() {
            callback.call(this, Y.U1.util.Stacktrace().slice(4));
            return context[functionName]._instrumented.apply(this, arguments);
        };
        context[functionName]._instrumented = original;
    },

    /**
    * Given a context and function name of a function that has been
    * instrumented, revert the function to it's original (non-instrumented)
    * state.
    *
    * @param {Object} context of execution (e.g. window)
    * @param {String} functionName to de-instrument
    */
    deinstrumentFunction: function(context, functionName) {
        if (context[functionName].constructor === Function &&
            context[functionName]._instrumented &&
            context[functionName]._instrumented.constructor === Function) {
            context[functionName] = context[functionName]._instrumented;
    }
    },

    /**
    * Given an Error object, return a formatted Array based on Chrome's stack string.
    *
    * @param e - Error object to inspect
    * @return Array<String> of function calls, files and line numbers
    */
    chrome: function(e) {
        var stack = (e.stack + '\n').replace(/^\S[^\(]+?[\n$]/gm, '').
            replace(/^\s+(at eval )?at\s+/gm, '').
            replace(/^([^\(]+?)([\n$])/gm, '{anonymous}()@$1$2').
                replace(/^Object.<anonymous>\s*\(([^\)]+)\)/gm, '{anonymous}()@$1').split('\n');
                stack.pop();
                return stack;
            },

    /**
    * Given an Error object, return a formatted Array based on Safari's stack string.
    *
    * @param e - Error object to inspect
    * @return Array<String> of function calls, files and line numbers
    */
    safari: function(e) {
        return e.stack.replace(/\[native code\]\n/m, '')
        .replace(/^(?=\w+Error\:).*$\n/m, '')
        .replace(/^@/gm, '{anonymous}()@')
        .split('\n');
    },

    /**
    * Given an Error object, return a formatted Array based on IE's stack string.
    *
    * @param e - Error object to inspect
    * @return Array<String> of function calls, files and line numbers
    */
    ie: function(e) {
        var lineRE = /^.*at (\w+) \(([^\)]+)\)$/gm;
        return e.stack.replace(/at Anonymous function /gm, '{anonymous}()@')
        .replace(/^(?=\w+Error\:).*$\n/m, '')
        .replace(lineRE, '$1@$2')
        .split('\n');
    },

    /**
    * Given an Error object, return a formatted Array based on Firefox's stack string.
    *
    * @param e - Error object to inspect
    * @return Array<String> of function calls, files and line numbers
    */
    firefox: function(e) {
        return e.stack.replace(/(?:\n@:0)?\s+$/m, '').replace(/^[\(@]/gm, '{anonymous}()@').split('\n');
    },

    opera11: function(e) {
        var ANON = '{anonymous}', lineRE = /^.*line (\d+), column (\d+)(?: in (.+))? in (\S+):$/,
        lines = e.stacktrace.split('\n'), result = [],
        i, match, location, fnName, len;

        for (i = 0, len = lines.length; i < len; i += 2) {
            match = lineRE.exec(lines[i]);
            if (match) {
                location = match[4] + ':' + match[1] + ':' + match[2];
                fnName = match[3] || "global code";
                fnName = fnName.replace(/<anonymous function: (\S+)>/, "$1").replace(/<anonymous function>/, ANON);
                result.push(fnName + '@' + location + ' -- ' + lines[i + 1].replace(/^\s+/, ''));
            }
        }

        return result;
    },

    opera10b: function(e) {
        // "<anonymous function: run>([arguments not available])@file://localhost/G:/js/stacktrace.js:27\n" +
        // "printStackTrace([arguments not available])@file://localhost/G:/js/stacktrace.js:18\n" +
        // "@file://localhost/G:/js/test/functional/testcase1.html:15"
        var lineRE = /^(.*)@(.+):(\d+)$/,
        lines = e.stacktrace.split('\n'), result = [],
        i, match, fnName, len;

        for (i = 0, len = lines.length; i < len; i++) {
            match = lineRE.exec(lines[i]);
            if (match) {
                fnName = match[1]? (match[1] + '()') : "global code";
                result.push(fnName + '@' + match[2] + ':' + match[3]);
            }
        }

        return result;
    },

    /**
    * Given an Error object, return a formatted Array based on Opera 10's stacktrace string.
    *
    * @param e - Error object to inspect
    * @return Array<String> of function calls, files and line numbers
    */
    opera10a: function(e) {
        // " Line 27 of linked script file://localhost/G:/js/stacktrace.js\n"
        // " Line 11 of inline#1 script in file://localhost/G:/js/test/functional/testcase1.html: In function foo\n"
        var ANON = '{anonymous}', lineRE = /Line (\d+).*script (?:in )?(\S+)(?:: In function (\S+))?$/i,
        lines = e.stacktrace.split('\n'), result = [],
        i, match, fnName, len;

        for (i = 0, len = lines.length; i < len; i += 2) {
            match = lineRE.exec(lines[i]);
            if (match) {
                fnName = match[3] || ANON;
                result.push(fnName + '()@' + match[2] + ':' + match[1] + ' -- ' + lines[i + 1].replace(/^\s+/, ''));
            }
        }

        return result;
    },

    // Opera 7.x-9.2x only!
    opera9: function(e) {
        // " Line 43 of linked script file://localhost/G:/js/stacktrace.js\n"
        // " Line 7 of inline#1 script in file://localhost/G:/js/test/functional/testcase1.html\n"
        var ANON = '{anonymous}', lineRE = /Line (\d+).*script (?:in )?(\S+)/i,
        lines = e.message.split('\n'), result = [],
        i, match, len;

        for (i = 2, len = lines.length; i < len; i += 2) {
            match = lineRE.exec(lines[i]);
            if (match) {
                result.push(ANON + '()@' + match[2] + ':' + match[1] + ' -- ' + lines[i + 1].replace(/^\s+/, ''));
            }
        }

        return result;
    },

    // Safari 5-, IE 9-, and others
    other: function(curr) {
        var ANON = '{anonymous}', fnRE = /function\s*([\w\-$]+)?\s*\(/i, stack = [], fn, args, maxStackSize = 10;
            while (curr && curr['arguments'] && stack.length < maxStackSize) {
                fn = fnRE.test(curr.toString()) ? RegExp.$1 || ANON : ANON;
                args = Array.prototype.slice.call(curr['arguments'] || []);
                stack[stack.length] = fn + '(' + this.stringifyArguments(args) + ')';
                curr = curr.caller;
            }
            return stack;
    },

    /**
    * Given arguments array as a String, substituting type names for non-string types.
    *
    * @param {Arguments,Array} args
    * @return {String} stringified arguments
    */
    stringifyArguments: function(args) {
        var result = [],
            slice, i, arg;

        slice = Array.prototype.slice;
        for (i = 0; i < args.length; ++i) {
            arg = args[i];
            if (arg === undefined) {
                result[i] = 'undefined';
            } else if (arg === null) {
                result[i] = 'null';
            } else if (arg.constructor) {
                if (arg.constructor === Array) {
                    if (arg.length < 3) {
                        result[i] = '[' + this.stringifyArguments(arg) + ']';
                    } else {
                        result[i] = '[' + this.stringifyArguments(slice.call(arg, 0, 1)) + '...' + this.stringifyArguments(slice.call(arg, -1)) + ']';
                    }
                } else if (arg.constructor === Object) {
                    result[i] = '#object';
                } else if (arg.constructor === Function) {
                    result[i] = '#function';
                } else if (arg.constructor === String) {
                    result[i] = '"' + arg + '"';
                } else if (arg.constructor === Number) {
                    result[i] = arg;
                }
            }
        }
        return result.join(',');
    },

    sourceCache: {},

    /**
    * @return the text from a given URL
    */
    ajax: function(url) {
        var req = this.createXMLHTTPObject();
        if (req) {
            try {
                req.open('GET', url, false);
                //req.overrideMimeType('text/plain');
                //req.overrideMimeType('text/javascript');
                req.send(null);
                //return req.status == 200 ? req.responseText : '';
                return req.responseText;
            } catch (e) {}
        }
        return '';
    },

    /**
    * Try XHR methods in order and store XHR factory.
    *
    * @return <Function> XHR function or equivalent
    */
    createXMLHTTPObject: function() {
        var xmlhttp, XMLHttpFactories = [
            function() {
                return new XMLHttpRequest();
            }, function() {
                return new ActiveXObject('Msxml2.XMLHTTP');
            }, function() {
                return new ActiveXObject('Msxml3.XMLHTTP');
            }, function() {
                return new ActiveXObject('Microsoft.XMLHTTP');
            }
        ],
        i;

        for (i = 0; i < XMLHttpFactories.length; i++) {
            try {
                xmlhttp = XMLHttpFactories[i]();
                // Use memoization to cache the factory
                this.createXMLHTTPObject = XMLHttpFactories[i];
                return xmlhttp;
            } catch (e) {}
        }
    },

    /**
    * Given a URL, check if it is in the same domain (so we can get the source
    * via Ajax).
    *
    * @param url <String> source url
    * @return <Boolean> False if we need a cross-domain request
    */
    isSameDomain: function(url) {
        return typeof location !== "undefined" && url.indexOf(location.hostname) !== -1; // location may not be defined, e.g. when running from nodejs.
    },

    /**
    * Get source code from given URL if in the same domain.
    *
    * @param url <String> JS source URL
    * @return <Array> Array of source code lines
    */
    getSource: function(url) {
        // TODO reuse source from script tags?
        if (!(url in this.sourceCache)) {
            this.sourceCache[url] = this.ajax(url).split('\n');
        }
        return this.sourceCache[url];
    },

    guessAnonymousFunctions: function(stack) {
        var i, reStack, reRef, frame, m, file, functionName, ref, lineno, charno;

        for (i = 0; i < stack.length; ++i) {
            reStack = /\{anonymous\}\(.*\)@(.*)/,
            reRef = /^(.*?)(?::(\d+))(?::(\d+))?(?: -- .+)?$/,
            frame = stack[i], ref = reStack.exec(frame);

            if (ref) {
                m = reRef.exec(ref[1]);
                if (m) { // If falsey, we did not get any file/line information
                    file = m[1], lineno = m[2], charno = m[3] || 0;
                    if (file && this.isSameDomain(file) && lineno) {
                        functionName = this.guessAnonymousFunction(file, lineno, charno);
                        stack[i] = frame.replace('{anonymous}', functionName);
                    }
                }
            }
        }
        return stack;
    },

    guessAnonymousFunction: function(url, lineNo) {
        var ret;
        try {
            ret = this.findFunctionName(this.getSource(url), lineNo);
        } catch (e) {
            ret = 'getSource failed with url: ' + url + ', exception: ' + e.toString();
        }
        return ret;
    },

    findFunctionName: function(source, lineNo) {
        // FIXME findFunctionName fails for compressed source
        // (more than one function on the same line)
        // function {name}({args}) m[1]=name m[2]=args
        var reFunctionDeclaration = /function\s+([^(]*?)\s*\(([^)]*)\)/,
        // {name} = function ({args}) TODO args capture
        // /['"]?([0-9A-Za-z_]+)['"]?\s*[:=]\s*function(?:[^(]*)/
        reFunctionExpression = /['"]?([$_A-Za-z][$_A-Za-z0-9]*)['"]?\s*[:=]\s*function\b/,
        // {name} = eval()
        reFunctionEvaluation = /['"]?([$_A-Za-z][$_A-Za-z0-9]*)['"]?\s*[:=]\s*(?:eval|new Function)\b/,
        // Walk backwards in the source lines until we find
        // the line which matches one of the patterns above
        code = "", line, maxLines = Math.min(lineNo, 20), m, commentPos, i;
        for (i = 0; i < maxLines; ++i) {
            // lineNo is 1-based, source[] is 0-based
            line = source[lineNo - i - 1];
            commentPos = line.indexOf('//');
            if (commentPos >= 0) {
                line = line.substr(0, commentPos);
            }
            // TODO check other types of comments? Commented code may lead to false positive
            if (line) {
                code = line + code;
                m = reFunctionExpression.exec(code);
                if (m && m[1]) {
                    return m[1];
                }
                m = reFunctionDeclaration.exec(code);
                if (m && m[1]) {
                    //return m[1] + "(" + (m[2] || "") + ")";
                    return m[1];
                }
                m = reFunctionEvaluation.exec(code);
                if (m && m[1]) {
                    return m[1];
                }
            }
        }
        return '(?)';
    }
};

Y.U1.util.Stacktrace = Stacktrace;


}, '@VERSION@', {"requires": []});
YUI.add('util-oops', function (Y, NAME) {

"use strict";
Y.namespace('U1.util');

var Oops = Y.Base.create('oneUtilOops', Y.Base, [], {

    initializer: function (config) {

        config = config || {};

        var globalContext = config.globalContext || this.get('globalContext'),
            globalHandler = config.globalErrorHandler || this._globalErrorHandler;

        // save older onerror
        // replace with our handler
        this._oldOnError = globalContext.onerror;
        globalContext.onerror = Y.bind(globalHandler, this);

    },

    destructor: function () {

        // revert global error handler
        this.get('globalContext').onerror = this._oldOnError;

    },

    // overloaded to handle directly calling window.onerror(new Error("Foo"))
    _globalErrorHandler: function (message, url, line) {
        var error;

        if(Y.Lang.isObject(message)) {
            error = message;
        } else {
            error = {
                message: message,
                url: url,
                line: line
            };
        }

        // send returns supress
        return this.send(error);
    },

    _createOopsReport: function (error) {

        return new Y.U1.util.OopsReport(error, {
            ioConfig:       this.get('ioConfig'),
            endPoint:       this.get('endPoint'),
            reporter:       this.get('reporter'),
            sessionData:    this.get('sessionData')
        });

    },

    // creates OOPS report and sends it
    send: function (error) {

        var report = this._createOopsReport(error);

        if(report) {

            this.fire('registered');

            report.send();

        }

        return this.get('suppress');
    }

}, {
    ATTRS: {
        globalErrorHandler: { value: null },
        globalContext: { value: Y.config.win },
        ioConfig: {
            value: {
                method: 'POST',
                xdr: {
                    use: 'native'
                }
            }
        },
        reporter: { value: 'u1.' + Y.config.win.location.hostname },
        sessionData: { value: null },
        suppress: { value: false },
        endPoint: {
            value: '',
            writeOnce: true
        }
    }
}),

OopsReport = function (error, config) {

    if(!error) {
        return null;
    }

    this.error = error;
    this.config = config;

    this.initializer();
    return this;
};

OopsReport.prototype = {

    _io: null,

    initializer: function () {

        this._io = new Y.IO();

    },

    send: function () {

        return this._io.send(this.config.endPoint, Y.merge(this.config.ioConfig, {
            data: this._createReport()
        }));

    },

    _createReport: function () {

        var reportElements = [
            { reporter: this.config.reporter },
            this.uaToJSON(),
            this.config.sessionData,
            this._normalizeError(this.error)
        ],
            ret,
            i = 0,
            j = reportElements.length;

        // Will try to report all reportElements
        // If it fails, drop one element at a time
        // This will end up with just the basic error

        // e.g. if reporter, ua and sessiondata are all borked,
        // the report will still include the error
        for(; i < j; i++) {

            try {
                ret = Y.JSON.stringify(Y.merge.apply(this, reportElements.splice(i)));
            } catch (e) {

                // report the stringification failure
                // but drop everything but the error
                new Y.U1.util.OopsReport(e, {
                    endPoint: this.config.endPoint,
                    ioConfig: this.config.ioConfig
                }).send();

            } finally {
                if(Y.Lang.isString(ret)) {
                    break;
                }
            }

        }

        return ret;

    },

    _normalizeError: function (error) {

        var e       = {},
            props   = [
                { accepts: ['message', 'description'], output: 'message' },
                { accepts: ['url', 'fileName'], output: 'url' },
                { accepts: ['line', 'lineNumber'], output: 'line' },
                { accepts: ['column', 'columnNumber'], output: 'column' },
                { accepts: ['name'], output: 'name' },
                { accepts: ['number'], output: 'errorCode' },
                { accepts: ['stack', 'stacktrace'], output: 'stacktrace' }
            ];

        // take accepted inputs and add as output on e
        Y.each(props, function (prop) {
            Y.each(prop.accepts, function (accept) {
                if(error[accept]) {
                    e[prop.output] = error[accept];
                }
            });
        });

        return e;
    },

    uaToJSON: function () {
        var queryObj = {};

        Y.each(Y.UA, function(v, k) {
            if (!Y.Lang.isFunction(v)) {
                if (v) {
                    try {
                        queryObj[k] = v;
                        if(Y.Lang.isNumber(v)) {
                            queryObj.browser = k;
                            queryObj.browserVersion = v;
                        }
                    } catch (e) {
                        void(0);
                    }
                }
            }
        });

        return queryObj;
    }
};

Y.U1.util.Oops = Oops;
Y.U1.util.OopsReport = OopsReport;


}, '@VERSION@', {"requires": ["base", "io-base", "json-stringify"]});
YUI.add('util-myapps', function (Y, NAME) {

"use strict";
var ExpandableLabel,
    SelectRelation,
    CheckboxDependency,
    DPErrors;

/**
 * A class for expandable labels
 *
 * @class ExpandableLabel
 * @extends Object
 * @namespace Devportal.Utils
 */
ExpandableLabel = function (labelId, expandedInitially) {

    this.label = this.getLabel(labelId);

    this.labelRow = this.getLabelRow(this.label);

    // TODO: static
    this.classes = {
        expanded: 'expandablelabel-expanded',
        collapsed: 'expandablelabel-collapsed',
        base: 'expandablelabel'
    };

    this.expandedInitially = Y.Lang.isBoolean(expandedInitially) ? expandedInitially : false;

    this.render();
    this.bind();

};

ExpandableLabel.prototype.getLabel = function (id) {
    return Y.one('label[for='+id+']');
};

ExpandableLabel.prototype.getLabelRow = function (label) {
    return label.ancestor('.field');
};

ExpandableLabel.prototype.collapse = function () {

    var c = this.classes;
    this.labelRow
        .addClass(c.collapsed)
        .removeClass(c.expanded);

};

ExpandableLabel.prototype.expand = function () {

    var c = this.classes;
    this.labelRow
        .removeClass(c.collapsed)
        .addClass(c.expanded);

};

ExpandableLabel.prototype.render = function () {

    // add base class
    this.labelRow.addClass(this.classes.base);

    // expand / collapse class
    if(this.expandedInitially) {
        this.expand();
    } else {
        this.collapse();
    }

};

ExpandableLabel.prototype.bind = function () {

    this.labelRow.delegate('click', this.click, '.'+this.classes.base, this);

};

ExpandableLabel.prototype.click = function (event) {

    var ct = event.currentTarget,
        cls = this.classes;

    // if expanded, only collapse if the target is a label and direct descendant of expandablelabel
    if(ct.hasClass(cls.expanded) && event.target.test('.'+cls.base+' .field-label')) {

        event.halt();
        this.collapse();

    } else if(ct.hasClass(cls.collapsed)) {

        event.halt();
        this.expand();

    }


};

ExpandableLabel.prototype.collapsedClick = function () {

    this.expand();

};

ExpandableLabel.prototype.expandedClick = function () {

    this.collapse();

};

/**
 * A class that lets two select boxes exclude each other
 *
 * @class SelectRelation
 * @extends Object
 * @namespace Devportal.Utils
 */
SelectRelation = function (select, antiSelect) {
    this.select = select;
    this.antiSelect = antiSelect;

    this.sync();
    this.bind();
};

SelectRelation.prototype.sync = function () {

    this.antiSelect.set('disabled', this.select.get('selectedIndex') > -1);

};

SelectRelation.prototype.bind = function () {

    this.select.on('change', this.change, this);

};

SelectRelation.prototype.change = function () {

    this.sync();

};

/**
 * A class that defines a checkbox dependant on another
 *
 * @class CheckboxDependency
 * @extends Object
 * @namespace Devportal.Utils
 */
CheckboxDependency = function (depender, dependee) {

    this.depender = depender;
    this.dependee = dependee;

    this.sync();
    this.bind();
};

CheckboxDependency.prototype.sync = function () {

    this.dependee.set('disabled', !this.depender.get('checked'));

};

CheckboxDependency.prototype.bind = function () {

    this.depender.on('change', this.change, this);

};

CheckboxDependency.prototype.change = function () {

    this.dependee.set('checked', false);
    this.sync();
};

/**
 * A class for handling errors in a devportal form
 *
 * @class Errors
 * @extends Y.Base
 * @namespace Devportal.Utils
 */
DPErrors = Y.Base.create('devportalUtilsErrors', Y.Base, [], {

    initializer: function (cfg) {

        if(!cfg || !cfg.notifiers || cfg.notifiers.length === 0) {
            throw new Error('No notifiers given.');
        }

        Y.each(cfg.notifiers, function (notifier) {
            notifier.addTarget(this);
        }, this);

        this.on('*:error', this.create);
        this.on('*:clear', this.clear);

    },

    create: function (cfg) {

        var errors = cfg.message,
            field = cfg.field;

        if(!Y.Lang.isArray(errors)) {
            errors = [errors];
        }

        if(field) {
            field = Y.Lang.isString(field) ? this.get('form').one(field) : field;
        }

        this._create(errors, field);

    },

    // create form or field errors
    _create: function (errors, field) {

        var reciever,
            tmpl = this.get('template');

        if(field) {
            this.markField(field, true);
            reciever = field.one(this.get('fieldErrorListCss') + ' .errorlist');

            if(!reciever) {
                reciever = Y.Node.create(this.get('errorListTemplate'));
                field.one(this.get('fieldErrorListCss')).append(reciever);
            }

        } else {
            reciever = this.get('formErrors');
        }

        Y.each(errors, function (error) {

            var node = Y.Node.create(Y.Lang.sub(tmpl, {
                message: error,
                'class': field ? 'field-error' : 'form-error'
            }));

            reciever.append(node);

        }, this);

    },

    // clears field error, or all errors
    clear: function (cfg) {

        var field = cfg.field;

        if(field) {
            Y.one(field).one(this.get('fieldErrorListCss')).empty();
            this.markField(field, false);
        } else {
            Y.all(this.get('fieldErrorListCss') + ', .form-errors').empty();
            this.markAllFields(false);
        }

    },

    // mark all fields
    markAllFields: function (hasErrors) {
        Y.all('.field').each(function (field) {
            this.markField(field, hasErrors);
        }, this);
    },

    // mark field either with an error, or without
    markField: function (field, hasErrors) {

        var cls = 'has-errors';

        if(hasErrors) {
            field.addClass(cls);
        } else {
            field.removeClass(cls);
        }
    }

}, {

    ATTRS: {
        errorListTemplate: { value: '<ul class="errorlist" />' },
        fieldErrorListCss: { value: '.field-errors' },
        form: {
            value: null,
            setter: function (val) {
                return Y.Lang.isString(val) ? Y.one(val) : val;
            }
        },
        formErrors: {
            value: null,
            setter: function (val) {
                return Y.Lang.isString(val) ? Y.one(val) : val;
            }
        },
        template: { value: '<li class="{class}">{message}</li>' }
    }

});


Y.namespace('Util').ExpandableLabel = ExpandableLabel;
Y.namespace('Util.Utils').SelectRelation = SelectRelation;
Y.namespace('Util').CheckBoxDependency = CheckboxDependency;
Y.namespace('Util').DPErrors = DPErrors;


}, '@VERSION@', {"skinnable": true, "requires": ["base", "node"]});
YUI.add('util', function (Y, NAME) {"use strict"; }, '@VERSION@', {"use": ["util-share", "util-stacktrace", "util-oops", "util-myapps"]});
