'use strict';

(function() {
    describe('Y.Carousel', function() {
        var Y;
        var html;
        var caro;

        function makeHTML () {
            html.setHTML('<ul class="carousel-slides">'+
                '<li style="width:10px"><img/></li>'+
                '<li style="width:10px"><img/></li>'+
                '<li style="width:10px"><img/></li>'+
            '</ul>'+
            '<div class="controls" />');
        }

        before(function(done) {
            Y = YUI().use(
                'carousel', 'node-event-simulate', function(Y) {

                    html = Y.Node.create('<div id="test" />');
                    Y.one('body').append(html);

                done();
            });
        });

        beforeEach(function (done) {

            makeHTML();

            caro = new Y.Carousel({
                nodeContainer: '#test',
                controlsContainer: '#test .controls',
                slideAnimDuration: 0
            });

            done();
        });

        afterEach(function (done) {
            html.empty();
            caro.destroy();
            done();
        });

        describe('#initializer()', function () {

            it('should be loaded', function() {

                expect(Y.Carousel).to.exist;

            });

            it('should throw error on bad container config', function() {

                function badCarousel () {
                    new Y.Carousel({
                        nodeContainer: '.nonexistant'
                    });
                }
                expect(badCarousel).to.throw(/invalid selector/);

            });

        });

        describe('#getCarouselNodeList()', function () {

            it('should be able to get all slides', function() {

                expect(caro.getCarouselNodeList().size()).to.eql(3);

            });

        });

        describe('#generatePagination()', function () {

            it('should generate ol.pagination', function() {

                expect(caro.get('nodeContainer').one('.pagination')).to.be.ok;

            });

            it('should generate to pagination list elements', function() {

                var nodeContainer = caro.get('nodeContainer');

                // check total amount of pagination elements
                expect(nodeContainer.all('.pagination li a span').size()).to.eql(3);

                // check all pagination elements
                nodeContainer.all('.pagination li a span').each(function (n, i) {
                    expect(n.get('text')).to.eql('Page'+(i+1));
                });

            });

        });

        describe('#generateControls()', function () {

            it('should generate prev and next', function() {

                var nodeContainer = caro.get('controlsContainer');

                expect(nodeContainer.one('.next span')).to.be.ok;
                expect(nodeContainer.one('.prev span')).to.be.ok;

            });

        });

        describe('#updatePagination()', function () {

            it('should make first slide pagination element active and others not', function() {

                var firstA = caro.get('nodeContainer').one('.pagination li:first-child a');
                var otherA = caro.get('nodeContainer').all('.pagination li:not(:first-chid) a');

                expect(firstA.hasClass('active')).to.be.true;

                otherA.each(function (a) {
                    expect(a.hasClass('active')).to.be.false;
                });

            });

        });

        describe('#gotoSlide()', function () {

            it('should change style slide images', function() {

                var n = caro.nodeCarousel;

                // assert first slide
                expect(caro.curSlide).to.eql(0);

                caro.gotoSlide(2);

                expect(n.getStyle('left')).to.eql('-20px');
                expect(caro.curSlide).to.eql(2);

            });

            it('should go to first slide for indices out of bounds', function() {

                caro.gotoSlide(100);
                expect(caro.nodeCarousel.getStyle('left')).to.eql('0px');
                expect(caro.curSlide).to.eql(0);

            });

        });


    });
})();
