YUI.add('global-nav-test', function (Y) {

var Assert = Y.Assert,
    suite;

suite = new Y.Test.Suite('GlobalNav');

suite.add(new Y.Test.Case({

    name: "General",

    setUp: function() {
        this.nav = new Y.U1.GlobalNav();

        this.data = [{
            url: "www.ubuntu.com",
            title: "Ubuntu"
        },{
            url: "www.foo.com",
            title: "Foo"
        },{
            url: "www.ubuntu.com/foo",
            title: "Ubuntu Foo"
        }];
    },

    "_reduceUrlList should reduce a list of urls to the single longest": function() {

        var obj = this.nav._reduceUrlList(this.data);
        Assert.isObject(obj);
        Assert.areEqual('www.ubuntu.com/foo', obj.url);
    },

    "_filterUrls should filter urls": function() {
        var list = this.nav._filterUrls(this.data, "www.foo.com");
        Assert.isArray(list);
        Assert.areEqual(1, list.length, 1);
        Assert.areEqual('www.foo.com', list[0].url);

        list = this.nav._filterUrls(this.data, "www.ubuntu.com/foo");
        Assert.isArray(list);
        Assert.areEqual(2, list.length);
    },

    "_getActiveUrl should do just that": function() {

        this.nav.location = "www.ubuntu.com";
        var index = this.nav.getActiveUrl(this.data);
        Assert.areEqual(0, index);

        this.nav.location = "www.ubuntu.com/foo";
        index = this.nav.getActiveUrl(this.data);
        Assert.areEqual(2, index);
    },

    "click on menu should toggle class": function() {
        var moreEl = Y.one('.u-nav-global-more');
        moreEl.one('a').simulate("click");
        Assert.areEqual(true, moreEl.hasClass('u-nav-global-show'));
        moreEl.one('a').simulate("click");
        Assert.areEqual(false, moreEl.hasClass('u-nav-global-show'));
    },

    "click off menu should close menu": function() {
        var moreEl = Y.one('.u-nav-global-more');
        moreEl.one('a').simulate("click");

        Y.one('body').simulate('click');
        Assert.areEqual(false, moreEl.hasClass('u-nav-global-show'));
    }
}));

Y.Test.Runner.add(suite);

}, '@VERSION@', {
        requires: ['global-navigation', 'node-event-simulate', 'test']
});
