(function() {

// store height of collapse once opened
var baseCollapseHeight;

// whever something happens with re: to the autocomplete,
// we need to change the height of the collapse to match the
// length of the list
function afterAutoCompleteEvents (e, args) {

    // transition config of collapse
    var transitionConfig;
    // height of autocomplete list node
    var acListHeight;
    // collapse itself
    var collapse = args.collapse;
    // input which has autocomplete plugin
    var input = args.input;

    // only calculate new height if autocomplete collapse is expanded
    if(!collapse.get('state')) {
        return;
    }

    // would only happen if collapse did not open properly
    if(!baseCollapseHeight) {
        throw new Error('baseCollapseHeight was falsy or 0');
    }

    // list was made invisible
    if(e.type === 'autocompleteListPlugin:visibleChange' && !e.newVal) {
        acListHeight = 0;
    } else {
        acListHeight = input.ac.get('listNode').get('scrollHeight');
    }

    // config from collapse
    transitionConfig = collapse.getAttrs(['duration', 'easing']);
    // add height to config, using base value and the current autocomplete list node height
    transitionConfig.height = baseCollapseHeight + acListHeight + 'px';

    // transition using created config
    collapse.get('host').transition(transitionConfig);

    Y.log('collapse host height: ' + transitionConfig.height, 'debug', 'navbar-autocomplete');
}

var navbars = Y.all('.ues-navbar');

if (navbars.size()) {
    navbars.each(function(node, i) {

        // the autocomplete wrapper
        var wrapper = node.one('.autocomplete');
        // input which has autocomplete plugin
        var input;
        // autocomplete boundingbox
        var autoCompleteBox;

        if(!wrapper) {
            Y.log('no AutoComplete wrapper in navbar #' + i, 'debug', 'navbar-autocomplete');
            return;
        }

        input = wrapper.one('input[type="search"]');
        if(!input) {
            throw new Error('autocomplete is missing input');
        }

        // plug autocomplete
        input.plug(Y.Plugin.AutoComplete, {
            resultTextLocator: 'text',
            resultHighlighter: 'phraseMatch',
            tabSelect: true
        });

        // whenever the autocomplete list node changes in height (or becomes invisible)
        // we need to change the height of the collapse
        input.ac.after(['clear', 'results', 'visibleChange'], afterAutoCompleteEvents, window, {
            collapse: wrapper.collapse,
            input: input
        });

        // store collapse height the first time the collapse opens
        wrapper.collapse.once('open', function (e) {
            baseCollapseHeight = e.target.get('host').get('scrollHeight');
            Y.log('baseCollapseHeight: ' + baseCollapseHeight, 'debug', 'navbar-autocomplete');
        });

        // add ues classes
        autoCompleteBox = input.ac.get('boundingBox');
        autoCompleteBox.addClass('ues-autocomplete').addClass('ues-dropdown');
        autoCompleteBox.one('ul').addClass('ues-dropdown-menu');

        // make ac control the visibility of the ues-dropdown
        input.ac.before('visibleChange', function (e) {
            autoCompleteBox.toggleClass('open', e.newVal);
        });

    });
}

})();
