'use strict';

/*
    If run in Firefox, small screen styles needs to have been applied
    for tests to pass
*/

(function () {


    describe('plugAndToggleSetupHelper', function () {
        var Y;

        before(function (done) {
            Y = YUI().use(
                'navbar', function (Y) {
                done();
            });
        });

        after(function (done) {
            Y.all('.ues-icon-menu, .ues-icon-search-bordered').remove(true);
            done();
        });

        it('should be loaded', function () {
            expect(Y.navbar.plugAndToggleSetupHelper).to.exist;
        });

        it('should do some pluggin', function () {

            var pluggable = Y.Node.create('<div />');
            var toggle = Y.Node.create('<div />');

            Y.navbar.plugAndToggleSetupHelper(pluggable, toggle);

            expect(pluggable.hasPlugin('collapse')).to.be.instanceof(Y.Plugin.Collapse);

        });

    });

    describe('navbar', function () {
        var Y;

        before(function (done) {
            Y = YUI().use(
                'navbar', 'node-event-simulate', function (Y) {
                done();
            });
        });

        after(function (done) {
            Y.all('.ues-icon-menu, .ues-icon-search-bordered').remove(true);
            done();
        });

        it('should have added a hamburger', function () {
            expect(Y.one('[data-qa="navbar-full"] .ues-icon-menu')).to.exist;
        });

        it('should have a menu with collapse plugin plugged in', function () {
            var menu = Y.one('[data-qa="navbar-full"] .ues-navbar-collapse');
            expect(menu.collapse).to.exist;
        });

        it('should open menu when hamburger clicked', function (done) {
            var hamburger = Y.one('[data-qa="navbar-full"] .ues-navbar-header > .ues-icon-menu');
            var menu = Y.one('[data-qa="navbar-full"] .ues-navbar-collapse');

            var event = menu.collapse.on('stateChange', function (e, done) {
                expect(this.hasClass('in')).to.be.false;
                expect(this.hasClass('collapse')).to.be.true;
                expect(this.hasClass('collapsing')).to.be.false;

                event.detach();
                done();
            }, menu, done);

            hamburger.simulate('click');
            // transition is 300ms, so this should immediately be true
            expect(menu.hasClass('collapsing')).to.be.true;
        });

        it('should close an open menu when hamburger clicked', function (done) {
            var hamburger = Y.one('[data-qa="navbar-full"] .ues-navbar-header > .ues-icon-menu');
            var menu = Y.one('[data-qa="navbar-full"] .ues-navbar-collapse');

            var event = menu.collapse.on('stateChange', function (e, done) {
                expect(this.hasClass('in')).to.be.true;
                expect(this.hasClass('collapse')).to.be.true;
                expect(this.hasClass('collapsing')).to.be.false;

                event.detach();
                done();
            }, menu, done);

            hamburger.simulate('click');
            // transition is 300ms, so this should immediately be true
            expect(menu.hasClass('collapsing')).to.be.true;
        });

        describe('search', function () {

            it('should have added a search icon', function () {
                expect(Y.one('[data-qa="navbar-search"] .ues-icon-search-bordered')).to.exist;
            });

            it('should have a search form with collapse plugin plugged in', function () {
                var search = Y.one('[data-qa="navbar-search"] .ues-navbar-collapse.search');
                expect(search.collapse).to.exist;
            });

            it('should open search when search icon clicked', function (done) {
                var searchIcon = Y.one('[data-qa="navbar-search"] .ues-navbar-header > .ues-icon-search-bordered');
                var search = Y.one('[data-qa="navbar-search"] .ues-navbar-collapse.search');

                var event = search.collapse.on('stateChange', function (e, done) {
                    expect(this.hasClass('in')).to.be.false;
                    expect(this.hasClass('collapse')).to.be.true;
                    expect(this.hasClass('collapsing')).to.be.false;

                    event.detach();
                    done();
                }, search, done);

                searchIcon.simulate('click');
                // transition is 300ms, so this should immediately be true
                expect(search.hasClass('collapsing')).to.be.true;
            });

            it('should close an open search when search icon clicked', function (done) {
                var searchIcon = Y.one('[data-qa="navbar-search"] .ues-navbar-header > .ues-icon-search-bordered');
                var search = Y.one('[data-qa="navbar-search"] .ues-navbar-collapse.search');

                var event = search.collapse.on('stateChange', function (e, done) {
                    expect(this.hasClass('in')).to.be.true;
                    expect(this.hasClass('collapse')).to.be.true;
                    expect(this.hasClass('collapsing')).to.be.false;

                    event.detach();
                    done();
                }, search, done);

                searchIcon.simulate('click');
                // transition is 300ms, so this should immediately be true
                expect(search.hasClass('collapsing')).to.be.true;
            });

        });

        describe('both search and menu together', function () {

            it('should open search', function (done) {
                var searchIcon = Y.one('[data-qa="navbar-search"] .ues-navbar-header > .ues-icon-search-bordered');
                var search = Y.one('[data-qa="navbar-search"] .ues-navbar-collapse.search');

                var event = search.collapse.on('stateChange', function (e, done) {
                    event.detach();
                    done();
                }, search, done);

                searchIcon.simulate('click');

            });

            it('should close an open search when menu icon clicked', function (done) {

                var hamburger = Y.one('[data-qa="navbar-search"] .ues-navbar-header > .ues-icon-menu');
                var menu = Y.one('[data-qa="navbar-search"] .ues-navbar-collapse');

                var search = Y.one('[data-qa="navbar-search"] .ues-navbar-collapse.search');

                var menuEvent = menu.collapse.on('stateChange', function (e) {
                    expect(this.hasClass('in')).to.be.false;
                    expect(this.hasClass('collapse')).to.be.true;
                    expect(this.hasClass('collapsing')).to.be.false;

                    // this would start collapsing search
                    expect(search.hasClass('collapsing')).to.be.true;

                    menuEvent.detach();

                }, menu);

                var searchEvent = search.collapse.on('stateChange', function (e, done) {
                    expect(this.hasClass('in')).to.be.true;
                    expect(this.hasClass('collapse')).to.be.true;
                    expect(this.hasClass('collapsing')).to.be.false;

                    searchEvent.detach();
                    done();
                }, search, done);

                hamburger.simulate('click');
                // transition is 300ms, so this should immediately be true
                expect(menu.hasClass('collapsing')).to.be.true;
            });

            it('should close an open menu when search icon clicked', function (done) {

                var menu = Y.one('[data-qa="navbar-search"] .ues-navbar-collapse');

                var search = Y.one('[data-qa="navbar-search"] .ues-navbar-collapse.search');
                var searchIcon = Y.one('[data-qa="navbar-search"] .ues-navbar-header > .ues-icon-search-bordered');

                var menuEvent = menu.collapse.on('stateChange', function (e) {
                    expect(this.hasClass('in')).to.be.true;
                    expect(this.hasClass('collapse')).to.be.true;
                    expect(this.hasClass('collapsing')).to.be.false;


                    menuEvent.detach();
                    done();

                }, menu);

                var searchEvent = search.collapse.on('stateChange', function (e, done) {
                    expect(this.hasClass('in')).to.be.false;
                    expect(this.hasClass('collapse')).to.be.true;
                    expect(this.hasClass('collapsing')).to.be.false;

                    // this would start collapsing menu
                    expect(menu.hasClass('collapsing')).to.be.true;

                    searchEvent.detach();
                }, search, done);

                searchIcon.simulate('click');
                // transition is 300ms, so this should immediately be true
                expect(search.hasClass('collapsing')).to.be.true;
            });

        });

    });

})();

(function () {

    describe('navbar-autocomplete', function () {
        var Y, input;

        beforeEach(function (done) {
            Y = YUI().use(
                'navbar', 'navbar-autocomplete', 'node-event-simulate', function (Y) {
                    input = Y.one('.autocomplete input');
                    done();
            });
        });

        afterEach(function () {

            if(input.ac) {
                input.ac.destroy();
            }

            Y.all('.ues-icon-menu, .ues-icon-search-bordered').remove(true);

        });

        it('should plug autocomplete input node', function () {
            Y.one('.autocomplete input').hasPlugin('ac');
        });

        describe('autocollapse implementation tests', function () {

            var autoCompleteQuery = function () {
                var val = 'face';

                input.set('value', '');
                input.ac._onInputValueChange({newVal: ''});

                input.set('value', val);
                input.ac._onInputValueChange({newVal: val});
            };

            var autoCompleteClear = function () {
                var val = 'face';

                input.set('value', val);
                input.ac._onInputValueChange({newVal: val});

                input.set('value', '');
                input.ac._onInputValueChange({newVal: ''});
            };

            var wrapper;

            beforeEach(function (done) {

                wrapper = Y.one('.autocomplete');
                wrapper.collapse.set('duration', 0);

                var openEvent = wrapper.collapse.on('open', function (e, done) {

                    done();
                    openEvent.detach();

                }, wrapper, done);

                wrapper.collapse.open();

            });

            it('should resize to fit input etc as well as autocomplete list on results', function (done) {

                input.ac.set('source', [
                    { text: 'Facebook' },
                    { text: 'Google+' }
                ]);

                var baseHeight = wrapper.get('scrollHeight');

                autoCompleteQuery();

                var checkAutoComplete = function () {
                    var listNodeHeight = input.ac.get('listNode').get('scrollHeight');
                    var wrapperHeight = wrapper.get('scrollHeight');
                    var targetHeight = baseHeight + listNodeHeight;

                    // estimates are good enough here
                    expect(wrapperHeight).to.be.within(targetHeight, targetHeight + 10);
                    done();
                };

                // wait for autocomplete to act
                window.setTimeout(checkAutoComplete, 100);

            });

            it('should resize back to normal on clear', function (done) {

                input.ac.set('source', [
                    { text: 'Facebook' },
                    { text: 'Google+' }
                ]);

                var baseHeight = wrapper.get('scrollHeight');

                autoCompleteClear();

                var checkAutoComplete = function () {
                    var wrapperHeight = wrapper.get('scrollHeight');

                    // estimates are good enough here
                    expect(wrapperHeight).to.be.within(baseHeight, baseHeight + 10);
                    done();
                };

                // wait for autocomplete to act
                window.setTimeout(checkAutoComplete, 100);

            });

        });

    });

    describe('navbar-autocomplete (loaded disorderly)', function () {

        it('should not throw errors even if loaded before navbar', function () {

            var fun = function () {
                YUI().use(
                    'navbar-autocomplete', 'navbar', function (Y) {
                });
            };

            expect(fun).to.not.throw(/./);
        });

    });

})();
