'use strict';

(function() {
    describe('plugin', function() {
        var Y;

        before(function(done) {
            Y = YUI().use(
                'node',
                'plugin-collapse', function(Y) {
                done();
            });
        });

        it('should be loaded', function() {
            expect(Y.Plugin.Collapse).to.exist;
        });

        it('should plug and configure', function() {

            var node = Y.one('.test');
            var STATE = true;
            var EASING = 'ease-in';
            var DURATION = 0.5;

            node.plug(Y.Plugin.Collapse, {
                duration: 0.5,
                easing: 'ease-in',
                state: true
            });

            expect(node.collapse).to.exist;
            expect(node.collapse.get('duration')).to.equal(DURATION);
            expect(node.collapse.get('easing')).to.equal(EASING);
            expect(node.collapse.get('state')).to.equal(STATE);
        });

        it('open() should collapse out node', function(done) {
            var node = Y.one('.test');

            var event = node.collapse.on('stateChange', function(e, done) {
                expect(this.hasClass('in')).to.be.false;
                expect(this.hasClass('collapse')).to.be.true;
                expect(this.hasClass('collapsing')).to.be.false;

                event.detach();
                done();
            }, node, done);

            node.collapse.open();
            // transition is 500ms, so this should immediately be true
            expect(node.hasClass('collapsing')).to.be.true;
        });

        it('close() should collapse in node', function(done) {
            var node = Y.one('.test');

            var on = node.collapse.on('stateChange', function(e) {
                expect(this.hasClass('in')).to.be.true;
                expect(this.hasClass('collapse')).to.be.true;
                expect(this.hasClass('collapsing')).to.be.false;

                on.detach();
            }, node);

            var after = node.collapse.after('stateChange', function(e, done) {
                expect(this.collapse.get('state')).to.be.false;

                after.detach();
                done();
            }, node, done);

            node.collapse.close();
            // transition is 500ms, so this should immediately be true
            expect(node.hasClass('collapsing')).to.be.true;
        });

        it('toggle() should switch state', function(done) {
            var node = Y.one('.test');
            var state = node.collapse.get('state');

            var after = node.collapse.after('stateChange', function(e, done) {
                expect(this.collapse.get('state')).to.equal(!state);

                after.detach();
                done();
            }, node, done);

            node.collapse.toggle();
        });

        it('should fire open event', function (done) {
            var node = Y.one('.test');

            var open = node.collapse.on('open', function(e, done) {
                open.detach();
                done();
            }, node, done);

            node.collapse.open();
        });

        it('should fire close event', function (done) {
            var node = Y.one('.test');

            var close = node.collapse.on('close', function(e, done) {
                close.detach();
                done();
            }, node, done);

            node.collapse.close();
        });

        it('should fire toggle event', function (done) {
            var node = Y.one('.test');

            var toggle = node.collapse.on('toggle', function(e, done) {
                toggle.detach();
                done();
            }, node, done);

            node.collapse.toggle();
        });

    });
})();
