YUI.add('plugin-jit-bar-test', function (Y) {

var Assert = Y.Assert,
    suite;

suite = new Y.Test.Suite('Plugin: Jit-Bar');

suite.add(new Y.Test.Case({

    name: '',

    setUp : function () {
        this.bar = Y.Node.create('<div>');
        Y.one('#test').append(this.bar);

        this.bar.setStyles({
            minHeight: 30,
            height: 60
        });

        this.bar.plug(Y.U1Plugin.JITBar);
    },

    tearDown: function () {
        this.bar && this.bar.destroy();
        delete this.bar;
        Y.one('#test').empty();
    },
    "test getScrolled": function() {
        this.bar.JITBar.getScrolled();
    },

    "test scrollToReduction": function() {
        this.bar.JITBar.scrollToReduction();
    },

    "test scrollToReduction beyond transformationTreshold": function () {
        this.bar.JITBar.set('transformationTreshold', -1);
        this.bar.JITBar.scrollToReduction();
    },

    "test getMinHeight": function() {
        this.bar.JITBar.getMinHeight();
    },

    "test getOriginalHeight": function() {
        this.bar.JITBar.getOriginalHeight();
    },

    "test getReducedHeight": function() {
        this.bar.JITBar.getReducedHeight();
    },

    "test getOriginalFontSize": function() {
        this.bar.JITBar.getOriginalFontSize();
    },

    "test getReducedFontSize": function() {
        this.bar.JITBar.getReducedFontSize();
    },

    "test animate": function() {
        this.bar.JITBar.animate(0);

        this.bar.JITBar.animation.on('end', function() {
            this.resume();
        }, this);

        this.wait();
    },

    "test reduce": function() {
        Y.U1Plugin.JITBar.prototype.reduce.call({
            set: function () {

            }
        });
    },

    "test elevate": function() {
        Y.U1Plugin.JITBar.prototype.elevate.call({
            set: function () {

            }
        });
    },

    "test redraw": function () {
        this.bar.JITBar.redraw();
    },

    "test setCssClasses": function () {

        Y.each([0, 5, 30, 50, 75, 80, 95, 100], function (amount) {
            this.bar.JITBar.setCssClasses(amount);
        }, this);

    },

    "test refreshDimensions": function () {

        Y.Assert.areSame(30, this.bar.JITBar.getMinHeight());

        this.bar.setStyles({
            minHeight: 40
        });

        // cached
        Y.Assert.areSame(30, this.bar.JITBar.getMinHeight());

        // delete cached values
        this.bar.JITBar.refreshDimensions();

        // get new value
        Y.Assert.areSame(40, this.bar.JITBar.getMinHeight());
    },

    "test useVirtualkeyboardFix": function () {
        this.bar.JITBar.useVirtualkeyboardFix();
    },

    "test attr scollingParent": function () {
        this.bar.JITBar.set('scrollingParent', Y.one('body'));
        this.bar.JITBar.set('scrollingParent', Y.config.win);
        this.bar.JITBar.set('scrollingParent', Y.one('#test'));
    },

    "test attr reduction (and rounding)": function () {

        // detach reductionChange
        this.bar.JITBar.detach('reductionChange');

        this.bar.JITBar.set('reduction', 17);
        Y.Assert.areSame(20, this.bar.JITBar.get('reduction'));
        this.bar.JITBar.set('reduction', 4);
        Y.Assert.areSame(0, this.bar.JITBar.get('reduction'));

    },

    "test attr proxy": function () {
        var host = this.bar.JITBar.get('host'),
            proxy = host.cloneNode();

        this.bar.JITBar.set('proxy', null);
        this.bar.JITBar.set('proxy', proxy);

        Y.Assert.isTrue(proxy.hasClass('jit-proxy'));
        Y.Assert.areSame('hidden', host.getStyle('visibility'));
        Y.Assert.areSame('visible', proxy.getStyle('visibility'));
    }

}));

Y.Test.Runner.add(suite);

}, '@VERSION@', {
        requires: [ 'plugin-jit-bar', 'test']
});
