YUI.add('uploader-augmented-test', function (Y) {

var suite,

    Base,
    Intergration,
    StatusAttribute,
    File,
    Follow;

suite = new Y.Test.Suite('uploader-augmented')

Base = new Y.Test.Case({

    name: 'Base',

    setUp: function () {

        this.app = new Y.UploaderAugmented();

        this.filearray = [];
        for (var i = 0; i <= 2; i++) {
            this.filearray.push(new Y.FileHTML5({name: i + ".jpg", size: i*1000, type: "jpg"}));
        }

        this.app.render();

        this.queue = new Y.UploaderHTML5.Queue({
            simUploads: 1,
            //errorAction: Y.Uploader.Queue.RESTART_ASAP,
            fileList: this.filearray,
            uploadURL: "test.html"
        });

       // set filelist
        this.app.set('fileList', this.filearray);
        this.app.fire('fileselect', {
            fileList: this.filearray
        });

        this.confirm = Y.config.win.confirm;
        Y.config.win.confirm = function () {};

    },

    tearDown: function () {
        if(this.app) {
            this.app.destroy();
        }
        delete this.app;
        delete this.filearray;
        delete this.queue;

        Y.config.win.confirm = this.confirm;
    },

    "instance ok": function () {
        Y.Assert.isInstanceOf(Y.UploaderAugmented, new Y.UploaderAugmented());
    },

    "test destructor": function () {
        this.app.destructor();
        Y.config.win.onbeforeunload = {_owner: this.app.name};
        this.app.destructor();
    },

    "test setting beforeunload": function () {

        Y.Assert.isNotNull(Y.config.win.onbeforeunload);

        Y.Assert.areSame('Discard 3 pending files?', this.app._beforeUnload({}));

        this.app.destroy();

        Y.Assert.isNull(Y.config.win.onbeforeunload);
    },

    "test getFile": function () {
        Y.Assert.areSame(this.filearray[0], this.app.getFile(this.filearray[0].get('id')));
    },

    "test get files in progress": function () {
        this.app.set('simLimit', 3);
        this.app.start();

        Y.Assert.areSame(
            this.app.get('fileList').length,
            this.app.getProgressingFiles().length
        );

    },

    "test have files in progress": function () {
        this.app.start();
        Y.Assert.isTrue(this.app.haveProgressingFiles());
    },

    "test getPendingFiles": function () {
        Y.Assert.areSame(this.app.get('fileList').length, this.app.getPendingFiles().length);
        this.app.set('fileList', []);
        Y.Assert.areSame(0, this.app.getPendingFiles().length);
    },

    "test havePendingFiles": function () {
        Y.Assert.isTrue(this.app.havePendingFiles());
        this.app.set('fileList', []);
        Y.Assert.isFalse(this.app.havePendingFiles());
    },
/*
    "test noFiles": function () {
        Y.Assert.isFalse(this.app.noFiles());
        this.app.set('fileList', []);
        this.queue.destroy();
        Y.Assert.isTrue(this.app.noFiles());
    },
*/
    "test getServerFiles": function () {
        this.app.getServerFiles();
    },

    "test haveServerFiles": function () {
        this.app.haveServerFiles();
    },

    "test getFilesTotals": function () {
        var file1 = new Y.FileHTML5({name: 'foo.jpg', size: 21, type: 'jpg'}),
            file2 = new Y.FileHTML5({name: 'bar.jpg', size: 21, type: 'jpg'});

        this.app.set('fileList', [file1, file2]);

        Y.Assert.areSame(42, this.app.getFilesTotals().size);
        Y.Assert.areSame(2, this.app.getFilesTotals().count);
    },

    "test getFriendlySize": function () {
        Y.Assert.areSame('1000 B', this.app.getFriendlySize(1000));
    },

    "test cancelFile": function () {

        // make queue active and push to queuedFiles
        this.app.queue = this.queue;
        this.app.queue._currentState = Y.UploaderHTML5.Queue.UPLOADING;
        this.app.start();

        this.app.cancelFile(this.filearray[1]);

        Y.ArrayAssert.doesNotContain(this.filearray[1], this.app.queue.queuedFiles);
        Y.ArrayAssert.contains(this.filearray[2], this.app.queue.queuedFiles);
    },

    "test getLocalFiles": function () {

        var file1 = new Y.FileHTML5({name: 'foo.jpg', size: 21, type: 'jpg'}),
            file2 = new Y.FileHTML5({name: 'bar.jpg', size: 21, type: 'jpg'});

        this.app.set('fileList', [file1, file2]);

        Y.Assert.areSame(2, this.app.getLocalFiles().length, 'no files uploaded');

        this.app.fire('uploadcomplete', {
            file: file1
        });

        Y.Assert.areSame(1, this.app.getLocalFiles().length, 'one file left');

    },

    "test retryFile": function () {
        this.app.retryFile(this.filearray[0]);
    },

    "test cancel": function () {
        this.app.cancel();
    },

    "test cancelFile": function () {
        this.app.cancelFile(this.filearray[0]);
    },

    "test cancelAllUploads": function () {
        this.app.cancelAllUploads();
    },

    "test start": function () {
        this.app.start();
    },

    "test getFileError": function () {
        var e = {
            source: 'http',
            status: 400,
            statusText: 'no good'
        };
        Y.Assert.areSame(
            Y.Lang.sub(this.app.get('strings.error_generic'), {
                status:         e.status,
                status_text:    e.statusText,
                source:         this.app.get('strings.error_http')
            }),
            this.app.getFileError(e).message
        );
    }

});

Intergration = new Y.Test.Case({

    name: 'Intergration',

    setUp: function () {

        this.app = new Y.UploaderAugmented();

        this.filearray = [];
        for (var i = 0; i <= 2; i++) {
            this.filearray.push(new Y.FileHTML5({name: i + ".jpg", size: i*1000, type: "jpg"}));
        }

        this.app.render();

        this.queue = new Y.UploaderHTML5.Queue({
            simUploads: 1,
            //errorAction: Y.Uploader.Queue.RESTART_ASAP,
            fileList: this.filearray,
            uploadURL: "test.html"
        });

       // set filelist
        this.app.set('fileList', this.filearray);
        this.app.fire('fileselect', {
            fileList: this.filearray
        });

    },

    tearDown: function () {
        if(this.app) {
            this.app.destroy();
        }
        delete this.app;
        delete this.filearray;
        delete this.queue;
    },

    "test haveProgressingFiles for only one file": function () {
        this.app.set('fileList', [this.filearray[0]]);
        this.app.start();
        Y.Assert.isTrue(this.app.haveProgressingFiles());
    },

    "test getLocalFiles after cancel": function () {

        var fileList = this.app.get('fileList');

        Y.Assert.areSame(3, this.app.getLocalFiles().length);

        // finish two files, cancel one
        this.app.fire('uploadcomplete', {
            'file': fileList[0]
        });
        this.app.fire('uploadcomplete', {
            'file': fileList[1]
        });

        this.app.cancelFile(fileList[2]);

        Y.Assert.areSame(0, this.app.getLocalFiles().length);

    },

    "all files cancelled or failed fires alluploadcancelled": function () {
        var eventCalled = false;

        this.app.on('alluploadcancelled', function () {
            eventCalled = true;
        });

        // cancel all uploads while they are uploading
        Y.Array.each(this.filearray, function (file) {
            this.app.cancelFile(file);
        }, this);

        this.filearray[0].set('status', Y.UploaderAugmented.FAILED);

        Y.Assert.isTrue(eventCalled, 'alluploadcancelled event called');

    },

    "all complete fires fullycomplete": function () {
        var eventCalled = false;

        this.app.on('fullycomplete', function () {
            eventCalled = true;
        });

        // cancel all uploads while they are uploading
        Y.Array.each(this.filearray, function (file) {
            file.set('status', Y.UploaderAugmented.COMPLETE);
        }, this);

        this.app.fire('alluploadscomplete');

        Y.Assert.isTrue(eventCalled, 'fullycomplete event called');

    },

    "some complete and some failed fires partialcomplete": function () {
        var eventCalled = false;

        this.app.on('partialcomplete', function () {
            eventCalled = true;
        });

        this.filearray[0].set('status', Y.UploaderAugmented.FAILED)
        this.filearray[1].set('status', Y.UploaderAugmented.CANCELLED)
        this.filearray[2].set('status', Y.UploaderAugmented.COMPLETE)

        this.app.fire('alluploadscomplete');

        Y.Assert.isTrue(eventCalled, 'partialcomplete event called');

    },

    "all failures or cancelled files fires incomplete": function () {
        var eventCalled = false;

        this.app.on('incomplete', function () {
            eventCalled = true;
        });

        this.filearray[0].set('status', Y.UploaderAugmented.FAILED)
        this.filearray[1].set('status', Y.UploaderAugmented.CANCELLED)
        this.filearray[2].set('status', Y.UploaderAugmented.FAILED)

        this.app.fire('alluploadscomplete');

        Y.Assert.isTrue(eventCalled, 'incomplete event called');

    },

    "large singular files should report percentage": function () {

        var file = new Y.FileHTML5({name: 'foo.jpg', size: 100, type: 'jpg'}),
            loaded = 0;

        this.app.set('fileList', [file]);
        this.app.fire('fileselect');

        this.app.on('totaluploadprogress', function (e) {
            loaded = e.percentLoaded;
        });

        this.app.start();

        file.fire('uploadprogress', {
            bytesLoaded: 5
        });

        // 5 bytes of 100 bytes is 5 percent
        Y.Assert.areSame(5, loaded);
    },

    "changing drag drop area fully detaches and reattaches our events": function () {

        var dd = Y.Node.create('<div class="foo" />');

        // dd has a drop evt listener
        this.app.set('dragAndDropArea', dd);
        Y.Assert.areSame(1, this.app.getEvent('drop')._subscribers.length);

        // dd has no drop listener
        this.app.set('dragAndDropArea', null);
        Y.Assert.areSame(0, this.app.getEvent('drop')._subscribers.length);

        // listener is reattached
        this.app.set('dragAndDropArea', dd);
        Y.Assert.areSame(1, this.app.getEvent('drop')._subscribers.length);

    },

    "test that uploader does not override beforeunload": function () {

        this.app.destroy();

        var fun = function () { "foo"; };

        window.onbeforeunload = fun;

        this.app = new Y.UploaderAugmented();

        Y.Assert.areSame(fun, window.onbeforeunload);

        this.app.destroy();

        Y.Assert.areSame(fun, window.onbeforeunload);

    }


});

StatusAttribute = new Y.Test.Case({

    name: 'Status attribute',

    setUp: function () {

        this.app = new Y.UploaderAugmented();
        this.app.render();
        this.file1 = new Y.FileHTML5({name: 'foo.jpg', size: 100, type: 'jpg'});
        this.file2 = new Y.FileHTML5({name: 'bar.jpg', size: 100, type: 'jpg'});

    },

    tearDown: function () {
        this.file1.destroy();
        this.file2.destroy();
        this.app.destroy();
        this.app = null;
        this.file1 = null;
        this.file2 = null;
    },

    "test initial": function () {

        Y.Assert.areSame(this.app.get('strings.uploads_empty'), this.app.get('status.text'));

    },

    "test getter": function () {

        this.app.set('status.text', 'foo');
        Y.Assert.areSame('foo', this.app.get('status.text'));


    },

    "test after fileList change": function () {

        var singleExpected = Y.Lang.sub(this.app.get('strings.uploads_pending'), {
            size: '100 B',
            count: 1
        }),
        multipleExpected = Y.Lang.sub(this.app.get('strings.uploads_pending'), {
            size: '200 B',
            count: 2
        });

        this.app.set('fileList', [this.file1]);
        Y.Assert.areSame(singleExpected, this.app.get('status.text'));

        this.app.set('fileList', [this.file1, this.file2]);
        Y.Assert.areSame(multipleExpected, this.app.get('status.text'));

    },


    "test status no files": function () {

        this.app.set('fileList', [this.file1, this.file2]);
        this.app.set('fileList', []);

        Y.Assert.areSame(this.app.get('strings.uploads_empty'), this.app.get('status.text'));


    },

    "test one pending": function () {

        this.app.set('fileList', [this.file1]);

        Y.Assert.areSame(
            Y.Lang.sub(this.app.get('strings.uploads_pending'), {
                count: 1,
                size: '100 B'
            }),
            this.app.get('status.text'));

    },

    "no NaNs in status": function () {
        var zeroLengthFile      = new Y.FileHTML5({ name: 'zero.jpg', size: 0, type: 'jpg' }),
            nullLengthFile      = new Y.FileHTML5({ name: 'null.jpg', size: null, type: 'jpg' }),
            undefinedLengthFile = new Y.FileHTML5({ name: 'undefined.jpg', size: undefined, type: 'jpg' }),
            negativeLengthFile  = new Y.FileHTML5({ name: 'negative.jpg', size: -1, type: 'jpg' }),

            NaNre               = new RegExp('NaN');

        Y.each([zeroLengthFile, nullLengthFile, undefinedLengthFile, negativeLengthFile], function (file) {

            this.app.set('fileList', [file]);
            this.app.fire('totaluploadprogress');
            Y.Assert.isFalse(NaNre.test(this.app.get('status.text')));

        }, this);
    },

    "lacking a proper percentage, show something else": function () {
        var badRe               = new RegExp('NaN|null|undefined'),
            values              = [NaN, null, undefined, "NaN", "null", "undefined"];

        Y.each(values, function (val) {

            this.app.fire('totaluploadprogress', {
                percentLoaded: val
            });
            Y.Assert.isFalse(badRe.test(this.app.get('status.text')), 'no bad values in _totalProgressNode');

        }, this);

    },

    "test percentage is int": function () {

        var third = Y.Lang.sub(this.app.get('strings.totaluploadprogress'), {
            percent: '33'
        }),
        all = Y.Lang.sub(this.app.get('strings.totaluploadprogress'), {
            percent: '100'
        }),
        quarter = Y.Lang.sub(this.app.get('strings.totaluploadprogress'), {
            percent: '25'
        }),
        one = Y.Lang.sub(this.app.get('strings.totaluploadprogress'), {
            percent: '1'
        });

        this.app.fire('totaluploadprogress', {percentLoaded: 33.33 });
        Y.Assert.areSame(third, this.app.get('status.text'));

        this.app.fire('totaluploadprogress', {percentLoaded: 100.0000 });
        Y.Assert.areSame(all, this.app.get('status.text'));

        this.app.fire('totaluploadprogress', {percentLoaded: 25.123123 });
        Y.Assert.areSame(quarter, this.app.get('status.text'));

        this.app.fire('totaluploadprogress', {percentLoaded: 1.1 });
        Y.Assert.areSame(one, this.app.get('status.text'));

    },

    "test incomplete (failures, no completed and no progressing files) uploader": function () {

        this.app.set('fileList', [this.file1, this.file2]);
        this.file1.set('status', Y.UploaderAugmented.FAILED);
        this.file2.set('status', Y.UploaderAugmented.FAILED);
        this.app.fire('uploaderror', {
            file: this.file1
        });
        this.app.fire('uploaderror', {
            file: this.file2
        });

        Y.Assert.areSame(Y.Lang.sub(this.app.get('strings.incomplete'), {
            percent: '33.3'
        }), this.app.get('status.text'));
    }
});

File = new Y.Test.Case({

    name: 'File',

    setUp: function () {

        this.app = new Y.UploaderAugmented();
        this.app.render();
        this.app.set('fileList', [
            new Y.FileHTML5({name:'foo'})
        ]);
        this.file = this.app.get('fileList')[0];

    },

    tearDown: function () {
        this.app.destroy();
    },


    "state: progress": function () {

        this.app.fire('uploadprogress', {
            file: this.file
        });

        Y.Assert.areSame(Y.UploaderAugmented.UPLOADING, this.file.get('status'));

    },

    "state: complete": function () {

        this.app.fire('uploadcomplete', {
            file: this.file
        });

        Y.Assert.areSame(Y.UploaderAugmented.COMPLETE, this.file.get('status'));

    },

    "state: failed": function () {

        this.app.fire('uploaderror', {
            file: this.file
        });

        Y.Assert.areSame(Y.UploaderAugmented.FAILED, this.file.get('status'));

    },

    "state: cancelled": function () {

        this.app.fire('filecancel', {
            file: this.file
        });

        Y.Assert.areSame(Y.UploaderAugmented.CANCELLED, this.file.get('status'));

    },

    "state: started": function () {

        this.app.fire('fileuploadstart', {
            file: this.file
        });

        Y.Assert.areSame(Y.UploaderAugmented.STARTED, this.file.get('status'));

    },

    "complete state is permanent": function () {


        this.app.fire('uploadcomplete', {
            file: this.file
        });

        this.app.fire('filecancel', {
            file: this.file
        });

        Y.Assert.areSame(Y.UploaderAugmented.COMPLETE, this.file.get('status'));

    },

    "test containsState": function () {

        this.file.set('status', Y.UploaderAugmented.COMPLETE);

        Y.Assert.isTrue(this.app.containsState(Y.UploaderAugmented.COMPLETE));
        Y.Assert.isFalse(this.app.containsState(Y.UploaderAugmented.CANCELLED));

    },

    "test containsAllStates": function () {
        this.app.set('fileList', [
            new Y.FileHTML5({name:'foo'}),
            new Y.FileHTML5({name:'bar'}),
            new Y.FileHTML5({name:'baz'})
        ]);
        this.app.get('fileList')[0].set('status', Y.UploaderAugmented.COMPLETE);
        this.app.get('fileList')[1].set('status', Y.UploaderAugmented.CANCELLED);
        this.app.get('fileList')[2].set('status', Y.UploaderAugmented.FAILED);

        Y.Assert.isTrue(this.app.containsAllStates([
            Y.UploaderAugmented.COMPLETE,
            Y.UploaderAugmented.CANCELLED,
            Y.UploaderAugmented.FAILED
        ]));

        Y.Assert.isFalse(this.app.containsAllStates([
            Y.UploaderAugmented.PENDING,
            Y.UploaderAugmented.UPLOADING
        ]));

    },

    "test containsAnyStates": function () {

        this.app.set('fileList', [
            new Y.FileHTML5({name:'foo'}),
            new Y.FileHTML5({name:'bar'}),
            new Y.FileHTML5({name:'baz'})
        ]);

        this.app.get('fileList')[0].set('status', Y.UploaderAugmented.COMPLETE);
        this.app.get('fileList')[1].set('status', Y.UploaderAugmented.CANCELLED);
        this.app.get('fileList')[2].set('status', Y.UploaderAugmented.FAILED);

        Y.Assert.isTrue(this.app.containsAllStates([
            Y.UploaderAugmented.COMPLETE,
            Y.UploaderAugmented.FAILED
        ]));

        Y.Assert.isFalse(this.app.containsAllStates([
            Y.UploaderAugmented.PENDING,
            Y.UploaderAugmented.UPLOADING
        ]));

    },

    "test containsOnlyState": function () {

        this.app.set('fileList', [
            new Y.FileHTML5({name:'foo'}),
            new Y.FileHTML5({name:'bar'}),
            new Y.FileHTML5({name:'baz'})
        ]);

        this.app.get('fileList')[0].set('status', Y.UploaderAugmented.COMPLETE);
        this.app.get('fileList')[1].set('status', Y.UploaderAugmented.COMPLETE);
        this.app.get('fileList')[2].set('status', Y.UploaderAugmented.COMPLETE);

        Y.Assert.isTrue(this.app.containsOnlyState(Y.UploaderAugmented.COMPLETE));

        Y.Assert.isFalse(this.app.containsOnlyState(Y.UploaderAugmented.UPLOADING));

    },

    "test testFileState": function () {
        this.app.fire('uploadcomplete', {
            file: this.file
        });
        Y.Assert.isTrue(this.app.testFileState(this.file, Y.UploaderAugmented.COMPLETE));
        Y.Assert.isFalse(this.app.testFileState(this.file, Y.UploaderAugmented.CANCELLED));

    }

});

Follow = new Y.Test.Case({

    name: 'Follow',

    setUp: function () {

        this.hostNode = Y.Node.create('<div style="position: relative; height: 200px; overflow: auto;" />');

        Y.one('body').append(this.hostNode);

        this.app = new Y.UploaderFollow({
            hostNode: this.hostNode,
            animated: false
        });

    },

    tearDown: function () {
        this.hostNode.remove().destroy();
        this.app.destroy();
    },

    _populateHostNode: function () {

        var i = 0,
            j = 100;

        for(;i<j;i++) {
            this.hostNode.append(Y.Node.create('<div style="height: 100px;" id="file-' + i + '">file-'+i+'</div>'));
        }

    },

    "instance ok": function () {
        Y.Assert.isInstanceOf(Y.UploaderFollow, new Y.UploaderFollow());
    },

    "api: bindFollow": function () {
        this.app.bindFollow();
    },

    "api: startFollowing": function () {
        this.app.startFollowing();
    },

    "api: stopFollowing": function () {
        this.app.stopFollowing();
    },

    "api: getHostHeight": function () {
        this.app.getHostHeight();
    },

    "api: getHostScrolled": function () {
        this.app.getHostScrolled();
    },

    "change on node e.g. 60 will scroll to it": function () {

        this._populateHostNode();

        var file = this.hostNode.one('#file-60');

        this.app.bindFollow();

        this.app._followFile({
            'file': {
                get: Y.bind(function () { return file}, this)
            }
        });

        Y.Assert.areSame(5900, this.app.getHostScrolled());
    }


});

suite.add(Base);
suite.add(Intergration);
suite.add(StatusAttribute);
suite.add(File);
suite.add(Follow);

Y.Test.Runner.add(suite);

}, '@VERSION@', {
        requires: ['uploader', 'uploader-follow']
});
