'use strict';

(function() {
    describe('uploader-myapps', function() {
        var Y;
        var uploader;

        before(function(done) {
            Y = YUI().use(
                'uploader-myapps', 'file-html5', function(Y) {
                    done();
            });
        });

        beforeEach(function (done) {

            uploader = new Y.UploaderMyapps();
            uploader.render();

            // TODO: refactor this submit button out
            uploader.get('contentBox').append(Y.Node.create('<button type="submit" />'));

            done();

        });

        afterEach(function (done) {

            uploader && uploader.destroy();
            done();

        });

        describe('Y.UploaderMyapps', function () {

            it('should be loaded', function() {
                expect(Y.UploaderMyapps).to.exist;
            });

            it('should be instantiable', function() {
                expect(uploader).to.exist;
            });

        });

        describe('File list changes', function () {

            it('should nullify DD and hide select button for one file', function() {

                uploader.set('fileList', [ new Y.FileHTML5({name: 'foo'}) ]);

                expect(uploader.get('selectFilesButton').getStyle('display')).to.eql('none');
                expect(uploader.get('dragAndDropArea')).to.be.null;
            });

            it('should reset DD and show select button for no file', function() {

                uploader.set('fileList', []);

                expect(uploader.get('selectFilesButton').getStyle('display')).to.not.eql('none');
                expect(uploader.get('dragAndDropArea')).to.be.ok;
            });

        });

        describe('Enabled changes', function () {

            it('should disable submit buttons and select files button', function() {

                uploader.set('enabled', false);

                expect(uploader.get('selectFilesButton').get('disabled')).to.be.true;
                expect(Y.one('button[type="submit"]').get('disabled')).to.be.true;
            });

            it('should enable submit buttons and select files button', function() {

                uploader.set('enabled', true);

                expect(uploader.get('selectFilesButton').get('disabled')).to.be.false;
                expect(Y.one('button[type="submit"]').get('disabled')).to.be.false;
            });

        });

        describe('Upload errors', function () {

            var file;

            beforeEach(function (done) {

                file = new Y.FileHTML5({ name: 'fails' });
                uploader.set('fileList', [ file ]);
                done();

            });

            it('should cancel files after retry limit and we should be able to access the response', function() {

                uploader.start();
                uploader.queue.uploadRetries[file.get('id')] = uploader.get('retryCount');

                uploader.on('uploadfailed', function (e) {
                    expect(e.response).to.eql("");
                })

                // simulates upload error which is the last retry
                uploader._dpUploadError({ file: file });

                // fires error which will fire uploadfailed (final failure)
                file.fire('uploaderror');

            });

        });

        describe('Status changes', function () {

            it('should tell user to DD if no files', function () {

                uploader.render();

                expect(uploader.get('contentBox').one('.uploader-dd').get('text')).to.eql('Drag and drop files here');

            });

            it('should tell user about added file', function () {

                var cb = uploader.get('contentBox');

                uploader.set('fileList', [ new Y.FileHTML5({ name: 'foo' })]);
                uploader.syncUI();

                expect(cb.one('.uploader-dd')).to.be.null;
                expect(cb.one('.uploader-file').get('text')).to.eql('foo');

            });


            it('should tell user about progress', function () {

                var cb = uploader.get('contentBox');
                uploader.render();

                uploader.set('fileList', [ new Y.FileHTML5({ name: 'foo', size: 10000 })]);

                uploader.fire('totaluploadprogress', {
                    percentLoaded: 50
                });

                expect(cb.one('.uploader-percentage').get('text')).to.eql('50%');

            });

        });

        describe('#resetAll()', function () {

            beforeEach(function (done) {

                uploader.set('fileList', [ new Y.FileHTML5({ name: 'foo', size: 10000 })]);
                uploader.start();
                done();

            });

            it('should destroy any queue', function () {

                uploader.resetAll();
                expect(uploader.queue).to.be.null;

            });

            it('should reset status text', function () {

                uploader.resetAll();
                expect(uploader.get('status.text')).to.eql(uploader.get('strings.uploads_empty'));

            });

            it('should reset file list', function () {

                uploader.resetAll();
                expect(uploader.get('fileList')).to.eql([]);

            });

        });

        describe('#syncUI()', function () {


            it('should be overloaded', function () {

                uploader.syncUI();

                uploader.syncUI({
                    type: 'uploaderMyapps:fullycomplete'
                });

            });

        });

        describe('template', function () {


            it('should exist', function () {

                expect(uploader.get('template')).to.be.ok;

            });

        });


    });
})();
