YUI.add('module-test', function (Y) {

var Assert = Y.Assert;

Y.Test.Runner.add(new Y.Test.Case({

    name: 'All',

    setUp : function () {

        // TODO carte bleue?
        this.cards = [{
            name: 'American Express',
            nos: [34343434343434]
        },{
            name: 'Dankort',
            nos: [5019717010103742]
        },{
            name: 'Diners Club',
            nos: [36700102000000, 36148900647913, 36422106234408]
        },{
            name: 'Discover',
            nos: [6011000400000000]
        },{
            name: 'Japanese Credit Bank',
            nos: [3528000700000000]
        },{
            name: 'MasterCard',
            nos: [5555555555554444, 5454545454545454]
        },{
            name: 'Visa',
            nos: [4444333322221111, 4911830000000, 4917610000000000, 4242424242424242, 4929756128390553, 4522454802514711]
        }];

        this.easycard = new Y.EasyCard({
            srcNode: '.ccinput'
        });

    },

    tearDown: function () {
        delete this.cards;
    },

    "getCardByIssuerIdentificationNumber() should identify card issuer by issuer ident number (iin)": function() {
        var i, j, card,
        q = this.easycard;

        for (i = this.cards.length, card; i--;) {
            card = this.cards[i];

            for (j = card.nos.length; j--;) {
                Assert.areSame(
                    card.name,
                    q.getCardByIssuerIdentificationNumber(card.nos[j]).name
                );
            }
        }
    },
    "isValidLuhn() should identify invalid and (potentially) valid card numbers": function() {
        var i, j,
        card,
        easycard = this.easycard;

        for (i = this.cards.length, card; i--;) {
            card = this.cards[i];

            for (j = card.nos.length; j--;) {
                Assert.isTrue(
                    easycard.isValidLuhn(card.nos[j])
                );
            }
        }
        Assert.isFalse(
            easycard.isValidLuhn(79927398710)
        );
        Assert.isFalse(
            easycard.isValidLuhn('79927398710a')
        );
        Assert.isFalse(
            easycard.isValidLuhn('79927398710')
        );
        Assert.isFalse(
            easycard.isValidLuhn('//79927398710')
        );
        Assert.isFalse(
            easycard.isValidLuhn()
        );
        /**
          Assert.isFalse(
          easycard.isValidLuhn(false)
          );
          Assert.isFalse(
          easycard.isValidLuhn(true)
          );
          */
    },
    "isValidExpiryDate should work on date strings of different lengths": function() {
        var easycard = this.easycard;

        Assert.isTrue( easycard.isValidExpiryDate('1'));
        Assert.isTrue( easycard.isValidExpiryDate('0'));
        Assert.isTrue( easycard.isValidExpiryDate('09'));
        Assert.isTrue( easycard.isValidExpiryDate('12'));
        Assert.isTrue( easycard.isValidExpiryDate(12));
        Assert.isTrue( easycard.isValidExpiryDate(1214));
        Assert.isTrue( easycard.isValidExpiryDate(1115));
        Assert.isTrue( easycard.isValidExpiryDate('0198'));

        Assert.isFalse( easycard.isValidExpiryDate('2'));
        Assert.isFalse( easycard.isValidExpiryDate('00'));
        Assert.isFalse( easycard.isValidExpiryDate('1312'));
        Assert.isFalse( easycard.isValidExpiryDate('1312'));
        Assert.isFalse( easycard.isValidExpiryDate(13));

        Assert.isFalse( easycard.isValidExpiryDate(0111));

        Assert.isFalse( easycard.isValidExpiryDate('a'));
        Assert.isFalse( easycard.isValidExpiryDate('abc'));
    },

    "isValidCsc should know what a valid csc for the current card looks like": function() {
        var csc = this.easycard.isValidCsc;

    },

    "spliceNumericString should splice numeric string": function() {
        var splice = this.easycard.spliceNumericString;

        Assert.isString(splice('1234', '1', 0));
        Assert.areSame('1234', splice('123', '4', 4));
        Assert.areSame('1234', splice('123/', '4', 4));
        Assert.areSame('1234567890001231123', splice('1234 5678', '9000 12/31 123', 9));

        // deletion
        Assert.isString(splice('1234', '1', 0, 2));

        Assert.areSame('123', splice('1234', '', 3, 1));
        Assert.areSame('123', splice('1234', null, 3, 1));
        Assert.areSame('123', splice('1234', undefined, 3, 1));
        Assert.areSame('123', splice('123/', '', 4, 1));
        Assert.areSame('12345', splice('123456', '', 5, 1));

        Assert.areSame('1234', splice('123456', '', 4, 2));
        Assert.areSame('1256', splice('123456', '', 2, 2));
    },
    "findSelectionStart should return the correct value for selectionStart": function() {
        var cc = new Y.EasyCard({
            srcNode: '.ccinput'
        });

        Assert.isNumber(cc.findSelectionStart('000','0010','0010 ', 3, 3, false));
        Assert.areSame(5, cc.findSelectionStart('000','0010','0010 ', 3, 3, false));
        Assert.areSame(6, cc.findSelectionStart('000','00100','0010 0', 3, 3, false));
        Assert.areSame(23, cc.findSelectionStart('0000111122223333','00001111222233334','0000 1111 2222 3333   4', 18, 18, false));

        Assert.areSame(2, cc.findSelectionStart('12345678','108','108', 1, 7, false));

        Assert.areSame(2, cc.findSelectionStart('0010 0','0000','0000', 3, 3, true), 'test deletions');
        Assert.areSame(1, cc.findSelectionStart('0110 00','0000','0000', 1, 3, true), 'test deletions');

        // when pasting, start and end are the selection indexes *before* the
        // paste event, that is to say they include spaces
        //
        // paste '0000' over '3456'
        Assert.areSame(7, cc.findSelectionStart('1234 5678','12000078','1200 0078', 2, 7, false, true), 'paste test 1');
        // paste '000' over '23'
        Assert.areSame(5, cc.findSelectionStart('1234','10004','1000 4', 1, 3, false, true), 'paste test 2');
        // paste '000' over '41'
        Assert.areSame(7, cc.findSelectionStart('12341234','12341234','1230 0023 4', 3, 6, false, true), 'paste test 3');

        // paste '00000' over '88'
        Assert.areSame(
            21,
            cc.findSelectionStart(
                '1111 1111 1111 88',
                '1111 1111 1111 00000',
                '1111 1111 1111 0000 0',
                15, 17, false, true),
                'paste test 4');

        // paste '00000' over '88'
        Assert.areSame(
            21,
            cc.findSelectionStart(
                '1111 1111 1111 8811',
                '1111 1111 1111 0000011',
                '1111 1111 1111 0000 01/1',
                15, 17, false, true),
                'paste test 5');

        Assert.areSame(
            23,
            cc.findSelectionStart(
                '1111 1111 1111 8811',
                '1111 1111 1111 00000011',
                '1111 1111 1111 0000 00/11',
                15, 17, false, true),
                'paste test 6');

        Assert.areSame(
            24,
            cc.findSelectionStart(
                '1111 1111 1111 1881',
                '1111 1111 1111 10000001',
                '1111 1111 1111 1000 00/01',
                16, 18, false, true),
                'paste test 7');

    },
    "countLeadingFormatChars should blah blah blah": function() {
        var doCount = this.easycard.countLeadingFormatChars;

        Assert.areSame(0, doCount('000'));
        Assert.areSame(1, doCount(' 000'));
        Assert.areSame(1, doCount('/00 0'));
        Assert.areSame(3, doCount('   00'));
    },
    "countMaskFormatChars should count anything that isn't a number": function() {
        var doCount = this.easycard.countMaskFormatChars;

        Assert.areSame(0, doCount(''));
        Assert.areSame(5, doCount('0000 0 0 0 00/00'));
    },
    "maskNumericString should mask a numeric string": function() {
        var doMask = this.easycard.maskNumericString,
            mask = '0000 0000 0000 0000   09/15   123';

        Assert.isString(doMask('123456', mask));

        // when it's an insertion
        Assert.areSame('1234 56', doMask('123456', mask));
        Assert.areSame('1234 5678 9012 3456   78/90   ', doMask('12345678901234567890', mask));
        Assert.areSame('1234 5678 9012 3456   ', doMask('1234567890123456', mask));

        // when it's a deletion
        Assert.areSame('1234 5678 9012 3456', doMask('1234567890123456', mask, true));
        Assert.areSame('1234', doMask('1234', mask, true));
        Assert.areSame('123', doMask('123', mask, true));

        // when it's nothing at all
        Assert.areSame('', doMask('', mask));
        Assert.areSame('', doMask('', mask, true));
    },

    "test short year to full year": function () {
        Assert.areSame(2104, this.easycard.shortYearToFullYear(04));
        Assert.areSame(2050, this.easycard.shortYearToFullYear(50));
        Assert.areSame(2014, this.easycard.shortYearToFullYear("14"));
    },

    "test full year to short year": function () {
        Assert.areSame(14, this.easycard.fullYearToShortYear(2014));
        Assert.areSame(15, this.easycard.fullYearToShortYear("2015"));
    }
}));

}, '@VERSION@', {
        requires: ['widget-cc-input', 'test']
});
